@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: actions.cpp 8562 2007-02-10 17:44:01Z feb $
+//      $Id: actions.cpp 8750 2007-04-14 18:09:46Z jim4 $
 
 //@{
 
@@ -39,7 +39,6 @@
 #include <time.h>
 
 #include "stratagus.h"
-#include "video.h"
 #include "unittype.h"
 #include "animation.h"
 #include "player.h"
@@ -85,7 +84,7 @@ static void UnitRotate(CUnit *unit, int rotate)
 **
 **  @return      The flags of the current script step.
 */
-int UnitShowAnimation(CUnit *unit, const CAnimation *anim) 
+int UnitShowAnimation(CUnit *unit, const CAnimation *anim)
 {
 	return UnitShowAnimationScaled(unit, anim, 8);
 }
@@ -93,11 +92,11 @@ int UnitShowAnimation(CUnit *unit, const CAnimation *anim)
 /**
 **  Show unit animation.
 **
-**  @param unit  Unit of the animation.
-**  @param anim  Animation script to handle.
-**  @param scale scaling factor of the wait times in animation (8 means no scaling).
+**  @param unit   Unit of the animation.
+**  @param anim   Animation script to handle.
+**  @param scale  Scaling factor of the wait times in animation (8 means no scaling).
 **
-**  @return      The flags of the current script step.
+**  @return       The flags of the current script step.
 */
 int UnitShowAnimationScaled(CUnit *unit, const CAnimation *anim, int scale)
 {
@@ -137,12 +136,6 @@ int UnitShowAnimationScaled(CUnit *unit, const CAnimation *anim, int scale)
 
 			case AnimationWait:
 				unit->Anim.Wait = unit->Anim.Anim->D.Wait.Wait << scale >> 8;
-				if (unit->Variable[SLOW_INDEX].Value) { // unit is slowed down
-					unit->Anim.Wait <<= 1;
-				}
-				if (unit->Variable[HASTE_INDEX].Value && unit->Anim.Wait > 1) { // unit is accelerated
-					unit->Anim.Wait >>= 1;
-				}
 				if (unit->Anim.Wait <= 0)
 					unit->Anim.Wait = 1;
 				break;
@@ -176,7 +169,6 @@ int UnitShowAnimationScaled(CUnit *unit, const CAnimation *anim, int scale)
 				} else {
 					FireMissile(unit);
 				}
-				unit->Variable[INVISIBLE_INDEX].Value = 0; // unit is invisible until attacks
 				break;
 
 			case AnimationRotate:
@@ -250,24 +242,12 @@ static void HandleActionNone(CUnit *unit)
 		UnitNumber(unit) _C_ unit->Type->Ident.c_str());
 }
 
-/**
-**  Unit has not written function.
-**
-**  @param unit  Unit pointer for not written action.
-*/
-static void HandleActionNotWritten(CUnit *unit)
-{
-	DebugPrint("FIXME: Not written!\n");
-	DebugPrint("FIXME: Unit (%d) %s has action %d.!\n" _C_
-		UnitNumber(unit) _C_ unit->Type->Ident.c_str() _C_ unit->Orders[0]->Action);
-}
-
 /**
 **  Jump table for actions.
 **
 **  @note can move function into unit structure.
 */
-static void (*HandleActionTable[256])(CUnit *) = {
+static void (*HandleActionTable[])(CUnit *) = {
 	HandleActionNone,
 	HandleActionStill,
 	HandleActionStandGround,
@@ -278,8 +258,6 @@ static void (*HandleActionTable[256])(CUnit *) = {
 	HandleActionDie,
 	HandleActionSpellCast,
 	HandleActionTrain,
-	HandleActionUpgradeTo,
-	HandleActionResearch,
 	HandleActionBuilt,
 	HandleActionBoard,
 	HandleActionUnload,
@@ -287,89 +265,6 @@ static void (*HandleActionTable[256])(CUnit *) = {
 	HandleActionBuild,
 	HandleActionRepair,
 	HandleActionResource,
-	HandleActionReturnGoods,
-	HandleActionTransformInto,
-	HandleActionNotWritten,
-
-	// Enough for the future ?
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
-	HandleActionNotWritten, HandleActionNotWritten, HandleActionNotWritten,
 };
 
 /**
@@ -379,9 +274,8 @@ static void (*HandleActionTable[256])(CUnit *) = {
 */
 static void HandleRegenerations(CUnit *unit)
 {
-	int f;
+	int f = 0;
 
-	f = 0;
 	// Burn
 	if (!unit->Removed && !unit->Destroyed && unit->Variable[HP_INDEX].Max &&
 			unit->Orders[0]->Action != UnitActionBuilt &&
@@ -407,10 +301,6 @@ static void HandleRegenerations(CUnit *unit)
 */
 static void HandleBuffs(CUnit *unit, int amount)
 {
-	int deadunit;
-	int i;        // iterator for variables.
-
-	deadunit = 0;
 	//
 	// Look if the time to live is over.
 	//
@@ -425,18 +315,8 @@ static void HandleBuffs(CUnit *unit, int amount)
 		}
 	}
 
-	//
-	//  decrease spells effects time.
-	//
-
-	unit->Variable[BLOODLUST_INDEX].Increase = -amount;
-	unit->Variable[HASTE_INDEX].Increase = -amount;
-	unit->Variable[SLOW_INDEX].Increase = -amount;
-	unit->Variable[INVISIBLE_INDEX].Increase = -amount;
-	unit->Variable[UNHOLYARMOR_INDEX].Increase = -amount;
-
 	// User defined variables
-	for (i = 0; i < UnitTypeVar.NumberVariable; i++) {
+	for (int i = 0; i < UnitTypeVar.NumberVariable; i++) {
 		if (unit->Variable[i].Enable && unit->Variable[i].Increase) {
 			unit->Variable[i].Value += unit->Variable[i].Increase;
 			if (unit->Variable[i].Value <= 0) {
@@ -448,24 +328,24 @@ static void HandleBuffs(CUnit *unit, int amount)
 	}
 }
 
+
+static void RunAction(unsigned char action, CUnit *unit)
+{
+	HandleActionTable[action](unit);
+}
+
+
 /**
-**  Handle the action of an unit.
+**  Handle the action of a unit.
 **
 **  @param unit  Pointer to handled unit.
 */
 static void HandleUnitAction(CUnit *unit)
 {
-	int z;
-
 	//
 	// If current action is breakable proceed with next one.
 	//
 	if (!unit->Anim.Unbreakable) {
-		if (unit->CriticalOrder.Action != UnitActionStill) {
-			HandleActionTable[unit->CriticalOrder.Action](unit);
-			unit->CriticalOrder.Action = UnitActionStill;
-		}
-
 		//
 		// o Look if we have a new order and old finished.
 		// o Or the order queue should be flushed.
@@ -494,12 +374,8 @@ static void HandleUnitAction(CUnit *unit)
 				Assert(!(unit->Orders[0]->Action == UnitActionStill && !unit->SubAction));
 				unit->Orders[0]->Goal->RefsDecrease();
 			}
-			if (unit->CurrentResource) {
-				if (unit->Type->ResInfo[unit->CurrentResource]->LoseResources &&
-					unit->ResourcesHeld < unit->Type->ResInfo[unit->CurrentResource]->ResourceCapacity) {
-					unit->ResourcesHeld = 0;
-				}
-			}
+
+			UnitRemoveConsumingResources(unit);
 
 			//
 			// Shift queue with structure assignment.
@@ -507,7 +383,7 @@ static void HandleUnitAction(CUnit *unit)
 			unit->OrderCount--;
 			unit->OrderFlush = 0;
 			delete unit->Orders[0];
-			for (z = 0; z < unit->OrderCount; ++z) {
+			for (int z = 0; z < unit->OrderCount; ++z) {
 				unit->Orders[z] = unit->Orders[z + 1];
 			}
 			unit->Orders.pop_back();
@@ -524,9 +400,9 @@ static void HandleUnitAction(CUnit *unit)
 	}
 
 	//
-	// Select action. FIXME: should us function pointers in unit structure.
+	// Select action.
 	//
-	HandleActionTable[unit->Orders[0]->Action](unit);
+	RunAction(unit->Orders[0]->Action, unit);
 }
 
 /**
@@ -632,9 +508,7 @@ void UnitActions(void)
 			unit->Player ? unit->Player->Index : -1, unit->Refs,SyncRandSeed,
 			unit->X, unit->Y, unit->IX, unit->IY);
 
-#if 0
-		SaveUnit(unit,logf);
-#endif
+//		SaveUnit(unit,logf);
 		fflush(NULL);
 		}
 #endif
