@@ -9,7 +9,7 @@
 //
 /**@name script_ai.cpp - The AI ccl functions. */
 //
-//      (c) Copyright 2000-2006 by Lutz Sammer, Ludovic Pollet,
+//      (c) Copyright 2000-2007 by Lutz Sammer, Ludovic Pollet,
 //                                 and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: script_ai.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: script_ai.cpp 8702 2007-04-08 02:27:33Z jim4 $
 
 //@{
 
@@ -117,55 +117,6 @@ static std::vector<CUnitType *> getReparableUnits()
 	return res;
 }
 
-/**
-**  Get sorted list of unittype with Supply not null.
-**
-**  @note Better (supply / cost) first.
-*/
-static std::vector<CUnitType *> getSupplyUnits()
-{
-	std::vector<CUnitType *> res;
-	std::vector<CUnitType *> sorted_res;
-
-
-	for (std::vector<CUnitType *>::const_iterator i = UnitTypes.begin(); i != UnitTypes.end(); ++i) {
-		CUnitType *type = *i;
-
-		if (type->Supply > 0) {
-			res.push_back(type);
-		}
-	}
-	// Now, sort them, best first.
-	while (!res.empty()) {
-		float bestscore;
-		CUnitType *besttype;
-
-		bestscore = 0;
-		for (std::vector<CUnitType *>::const_iterator i = res.begin(); i != res.end(); ++i) {
-			CUnitType *type = *i;
-			float score;
-			unsigned int cost = 0;
-
-			for (unsigned j = 0; j < MaxCosts; ++j) {
-				cost += type->_Costs[j];
-			}
-			score = ((float) type->Supply) / cost;
-			if (score > bestscore) {
-				bestscore = score;
-				besttype = type;
-			}
-		}
-		sorted_res.push_back(besttype);
-		for (std::vector<CUnitType *>::iterator i = res.begin(); i != res.end(); ++i) {
-			if (*i == besttype) {
-				i = res.erase(i);
-				break;
-			}
-		}
-	}
-	return sorted_res;
-}
-
 /**
 **  Init AiHelper.
 **
@@ -178,11 +129,6 @@ static void InitAiHelper(AiHelper &aiHelper)
 	extern std::vector<ButtonAction *> UnitButtonTable;
 
 	std::vector<CUnitType *> reparableUnits = getReparableUnits();
-	std::vector<CUnitType *> supplyUnits = getSupplyUnits();
-
-	for (std::vector<CUnitType *>::const_iterator i = supplyUnits.begin(); i != supplyUnits.end(); ++i) {
-		AiHelperInsert(aiHelper.UnitLimit, 0, *i);
-	}
 
 	for (int i = 0; i < (int)UnitButtonTable.size(); ++i) {
 		const ButtonAction &button = *UnitButtonTable[i];
@@ -214,24 +160,6 @@ static void InitAiHelper(AiHelper &aiHelper)
 				}
 				break;
 			}
-			case ButtonUpgradeTo :
-			{
-				CUnitType *upgradeToType = UnitTypeByIdent(button.ValueStr);
-
-				for (std::vector<CUnitType *>::const_iterator j = unitmask.begin(); j != unitmask.end(); ++j) {
-					AiHelperInsert(aiHelper.Upgrade, upgradeToType->Slot, *j);
-				}
-				break;
-			}
-			case ButtonResearch :
-			{
-				int researchId = UpgradeIdByIdent(button.ValueStr);
-
-				for (std::vector<CUnitType *>::const_iterator j = unitmask.begin(); j != unitmask.end(); ++j) {
-					AiHelperInsert(aiHelper.Research, researchId, *j);
-				}
-				break;
-			}
 			default:
 				break;
 		}
@@ -271,16 +199,12 @@ static int CclDefineAiHelper(lua_State *l)
 		++k;
 
 		//
-		// Type build,train,research/upgrade.
+		// Type build,train.
 		//
 		if (!strcmp(value, "build")) {
 			what = -1;
 		} else if (!strcmp(value, "train")) {
 			what = -1;
-		} else if (!strcmp(value, "upgrade")) {
-			what = -1;
-		} else if (!strcmp(value, "research")) {
-			what = -1;
 		} else if (!strcmp(value, "unit-limit")) {
 			what = -1;
 		} else if (!strcmp(value, "unit-equiv")) {
@@ -428,45 +352,6 @@ static AiRequestType *FindInUnitTypeRequests(const CUnitType *type)
 	return NULL;
 }
 
-/**
-**  Find unit-type in upgrade-to table.
-**
-**  @param type  Unit-type to be found.
-*/
-static int FindInUpgradeToRequests(const CUnitType *type)
-{
-	int i;
-	int n;
-
-	n = AiPlayer->UpgradeToRequests.size();
-	for (i = 0; i < n; ++i) {
-		if (AiPlayer->UpgradeToRequests[i] == type) {
-			return 1;
-		}
-	}
-	return 0;
-}
-
-/**
-**  Append unit-type to request table.
-**
-**  @param type  Unit-type to be appended.
-*/
-static void InsertUpgradeToRequests(CUnitType *type)
-{
-	AiPlayer->UpgradeToRequests.push_back(type);
-}
-
-/**
-**  Append unit-type to request table.
-**
-**  @param upgrade  Upgrade to be appended.
-*/
-static void InsertResearchRequests(CUpgrade *upgrade)
-{
-	AiPlayer->ResearchRequests.push_back(upgrade);
-}
-
 //----------------------------------------------------------------------------
 
 /**
@@ -594,7 +479,7 @@ static int CclAiSet(lua_State *l)
 }
 
 /**
-**  Wait for an unit.
+**  Wait for a unit.
 **
 **  @param l  Lua State.
 **
@@ -631,13 +516,6 @@ static int CclAiWait(lua_State *l)
 				}
 			}
 		}
-		//
-		// Look if we have an upgrade-to request.
-		//
-		if (FindInUpgradeToRequests(type)) {
-			lua_pushboolean(l, 1);
-			return 1;
-		}
 		DebugPrint("Broken? waiting on %s which wasn't requested.\n" _C_ type->Ident.c_str());
 		lua_pushboolean(l, 0);
 		return 1;
@@ -861,47 +739,6 @@ static int CclAiSleep(lua_State *l)
 	return 1;
 }
 
-/**
-**  Research an upgrade.
-**
-**  @param l  Lua state.
-*/
-static int CclAiResearch(lua_State *l)
-{
-	const char *str;
-	CUpgrade *upgrade;
-
-	LuaCheckArgs(l, 1);
-	if ((str = LuaToString(l, 1))) {
-		upgrade = CUpgrade::Get(str);
-	} else {
-		LuaError(l, "Upgrade needed");
-		upgrade = 0;
-	}
-
-	InsertResearchRequests(upgrade);
-
-	lua_pushboolean(l, 0);
-	return 1;
-}
-
-/**
-**  Upgrade an unit to an new unit-type.
-**
-**  @param l  Lua state.
-*/
-static int CclAiUpgradeTo(lua_State *l)
-{
-	CUnitType *type;
-
-	LuaCheckArgs(l, 1);
-	type = CclGetUnitType(l);
-	InsertUpgradeToRequests(type);
-
-	lua_pushboolean(l, 0);
-	return 1;
-}
-
 /**
 **  Return the player of the running AI.
 **
@@ -916,53 +753,6 @@ static int CclAiPlayer(lua_State *l)
 	return 1;
 }
 
-/**
-**  Set AI player resource reserve.
-**
-**  @param l  Lua state.
-**
-**  @return     Old resource vector
-*/
-static int CclAiSetReserve(lua_State *l)
-{
-	int i;
-
-	LuaCheckArgs(l, 1);
-	if (!lua_istable(l, 1)) {
-		LuaError(l, "incorrect argument");
-	}
-	lua_newtable(l);
-	for (i = 0; i < MaxCosts; ++i) {
-		lua_pushnumber(l, AiPlayer->Reserve[i]);
-		lua_rawseti(l, -2, i + 1);
-	}
-	for (i = 0; i < MaxCosts; ++i) {
-		lua_rawgeti(l, 1, i + 1);
-		AiPlayer->Reserve[i] = LuaToNumber(l, -1);
-		lua_pop(l, 1);
-	}
-	return 1;
-}
-
-/**
-**  Set AI player resource collect percent.
-**
-**  @param l  Lua state.
-*/
-static int CclAiSetCollect(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	if (!lua_istable(l, 1)) {
-		LuaError(l, "incorrect argument");
-	}
-	for (int i = 0; i < MaxCosts; ++i) {
-		lua_rawgeti(l, 1, i + 1);
-		AiPlayer->Collect[i] = LuaToNumber(l, -1);
-		lua_pop(l, 1);
-	}
-	return 0;
-}
-
 /**
 **  Dump some AI debug informations.
 **
@@ -980,10 +770,6 @@ static int CclAiDump(lua_State *l)
 	// Script
 	//
 	printf("------\n");
-	for (i = 0; i < MaxCosts; ++i) {
-		printf("%s(%4d) ", DefaultResourceNames[i], AiPlayer->Player->Resources[i]);
-	}
-	printf("\n");
 	printf("%d:", AiPlayer->Player->Index);
 #if 0
 	gh_display(gh_car(AiPlayer->Script));
@@ -997,18 +783,6 @@ static int CclAiDump(lua_State *l)
 		printf("%s ", AiPlayer->UnitTypeRequests[i].Type->Ident.c_str());
 	}
 	printf("\n");
-	n = (int)AiPlayer->UpgradeToRequests.size();
-	printf("UpgradeToRequests(%d):\n", n);
-	for (i = 0; i < n; ++i) {
-		printf("%s ", AiPlayer->UpgradeToRequests[i]->Ident.c_str());
-	}
-	printf("\n");
-	n = (int)AiPlayer->ResearchRequests.size();
-	printf("ResearchRequests(%d):\n", n);
-	for (i = 0; i < n; ++i) {
-		printf("%s ", AiPlayer->ResearchRequests[i]->Ident.c_str());
-	}
-	printf("\n");
 
 	//
 	// Building queue
@@ -1043,14 +817,14 @@ static int CclAiDump(lua_State *l)
 **
 **  @param name  Resource name.
 **
-**  @return   The number of the resource in DefaultResourceNames
+**  @return      The number of the resource in DefaultResourceNames
 */
 static int DefaultResourceNumber(const char *name)
 {
 	int i;
 
 	for (i = 0; i < MaxCosts; ++i) {
-		if (!strcmp(DefaultResourceNames[i], name)) {
+		if (DefaultResourceNames[i] == name) {
 			return i;
 		}
 	}
@@ -1223,42 +997,6 @@ static int CclDefineAiPlayer(lua_State *l)
 					LuaError(l, "Unsupported tag: %s" _C_ value);
 				}
 			}
-		} else if (!strcmp(value, "reserve")) {
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			for (k = 0; k < subargs; ++k) {
-				const char *type;
-				int num;
-
-				lua_rawgeti(l, j + 1, k + 1);
-				type = LuaToString(l, -1);
-				lua_pop(l, 1);
-				++k;
-				lua_rawgeti(l, j + 1, k + 1);
-				num = LuaToNumber(l, -1);
-				lua_pop(l, 1);
-				ai->Reserve[DefaultResourceNumber(type)] = num;
-			}
-		} else if (!strcmp(value, "used")) {
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			for (k = 0; k < subargs; ++k) {
-				const char *type;
-				int num;
-
-				lua_rawgeti(l, j + 1, k + 1);
-				type = LuaToString(l, -1);
-				lua_pop(l, 1);
-				++k;
-				lua_rawgeti(l, j + 1, k + 1);
-				num = LuaToNumber(l, -1);
-				lua_pop(l, 1);
-				ai->Used[DefaultResourceNumber(type)] = num;
-			}
 		} else if (!strcmp(value, "needed")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
@@ -1277,24 +1015,6 @@ static int CclDefineAiPlayer(lua_State *l)
 				lua_pop(l, 1);
 				ai->Needed[DefaultResourceNumber(type)] = num;
 			}
-		} else if (!strcmp(value, "collect")) {
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			for (k = 0; k < subargs; ++k) {
-				const char *type;
-				int num;
-
-				lua_rawgeti(l, j + 1, k + 1);
-				type = LuaToString(l, -1);
-				lua_pop(l, 1);
-				++k;
-				lua_rawgeti(l, j + 1, k + 1);
-				num = LuaToNumber(l, -1);
-				lua_pop(l, 1);
-				ai->Collect[DefaultResourceNumber(type)] = num;
-			}
 		} else if (!strcmp(value, "need-mask")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
@@ -1308,9 +1028,6 @@ static int CclDefineAiPlayer(lua_State *l)
 				lua_pop(l, 1);
 				ai->NeededMask |= (1 << DefaultResourceNumber(type));
 			}
-		} else if (!strcmp(value, "need-supply")) {
-			ai->NeedSupply = true;
-			--j;
 		} else if (!strcmp(value, "exploration")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
@@ -1392,32 +1109,6 @@ static int CclDefineAiPlayer(lua_State *l)
 				ai->UnitTypeRequests[i].Count = count;
 				++i;
 			}
-		} else if (!strcmp(value, "upgrade")) {
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			for (k = 0; k < subargs; ++k) {
-				const char *ident;
-
-				lua_rawgeti(l, j + 1, k + 1);
-				ident = LuaToString(l, -1);
-				lua_pop(l, 1);
-				ai->UpgradeToRequests.push_back(UnitTypeByIdent(ident));
-			}
-		} else if (!strcmp(value, "research")) {
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			for (k = 0; k < subargs; ++k) {
-				const char *ident;
-
-				lua_rawgeti(l, j + 1, k + 1);
-				ident = LuaToString(l, -1);
-				lua_pop(l, 1);
-				ai->ResearchRequests.push_back(CUpgrade::Get(ident));
-			}
 		} else if (!strcmp(value, "building")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
@@ -1501,12 +1192,8 @@ void AiCclRegister(void)
 
 	lua_register(Lua, "AiAttackWithForce", CclAiAttackWithForce);
 	lua_register(Lua, "AiSleep", CclAiSleep);
-	lua_register(Lua, "AiResearch", CclAiResearch);
-	lua_register(Lua, "AiUpgradeTo", CclAiUpgradeTo);
 
 	lua_register(Lua, "AiPlayer", CclAiPlayer);
-	lua_register(Lua, "AiSetReserve", CclAiSetReserve);
-	lua_register(Lua, "AiSetCollect", CclAiSetCollect);
 
 	lua_register(Lua, "AiDump", CclAiDump);
 
