@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: editloop.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: editloop.cpp 8893 2007-06-16 18:28:33Z jim4 $
 
 //@{
 
@@ -128,6 +128,8 @@ static EditorSliderListener *editorSliderListener;
 --  Functions
 ----------------------------------------------------------------------------*/
 
+extern void InitDefinedVariables();
+
 /*----------------------------------------------------------------------------
 --  Edit
 ----------------------------------------------------------------------------*/
@@ -199,7 +201,7 @@ void EditTile(int x, int y, int tile)
 	//
 	// Change the flags
 	//
-	mf = &Map.Fields[y * Map.Info.MapWidth + x];
+	mf = Map.Field(x, y);
 	mf->Flags &= ~(MapFieldLandAllowed | MapFieldCoastAllowed |
 		MapFieldWaterAllowed | MapFieldNoBuilding | MapFieldUnpassable);
 
@@ -297,6 +299,10 @@ static void EditUnitInternal(int x, int y, CUnitType *type, CPlayer *player)
 	CUnit *unit;
 	CBuildRestrictionOnTop *b;
 
+	if (type->Neutral) {
+		player = &Players[PlayerNumNeutral];
+	}
+
 	// FIXME: vladi: should check place when mirror editing is enabled...?
 	unit = MakeUnitAndPlace(x, y, type, player);
 	b = OnTopDetails(unit, NULL);
@@ -307,7 +313,8 @@ static void EditUnitInternal(int x, int y, CUnitType *type, CPlayer *player)
 		n = UnitCacheOnTile(x, y, table);
 		while (n--) {
 			if (table[n]->Type == b->Parent) {
-				unit->ResourcesHeld = table[n]->ResourcesHeld; // We capture the value of what is beneath.
+				// We capture the value of what is beneath.
+				memcpy(unit->ResourcesHeld, table[n]->ResourcesHeld, sizeof(unit->ResourcesHeld));
 				table[n]->Remove(NULL); // Destroy building beneath
 				UnitLost(table[n]);
 				UnitClearOrders(table[n]);
@@ -318,8 +325,8 @@ static void EditUnitInternal(int x, int y, CUnitType *type, CPlayer *player)
 
 	}
 	if (unit != NoUnitP) {
-		if (type->GivesResource) {
-			unit->ResourcesHeld = DefaultResourceAmounts[type->GivesResource];
+		if (type->CanHarvestFrom) {
+			memcpy(unit->ResourcesHeld, DefaultResourceAmounts, sizeof(unit->ResourcesHeld));
 		}
 	} else {
 		DebugPrint("Unable to allocate Unit");
@@ -458,7 +465,10 @@ class CEditResourceOKActionListener : public gcn::ActionListener
 {
 public:
 	virtual void action(const std::string &eventId) {
-		UnitUnderCursor->ResourcesHeld = atoi(editResourceTextField->getText().c_str());
+		int amount = atoi(editResourceTextField->getText().c_str());
+		for (int i = 0; i < MaxCosts; ++i) {
+			UnitUnderCursor->ResourcesHeld[i] = amount * CYCLES_PER_SECOND;
+		}
 		editResourceMenu->stop();
 	}
 };
@@ -490,12 +500,12 @@ static void EditorEditResource(void)
 	editResourceMenu->setBorderSize(1);
 	editResourceMenu->setDrawMenusUnder(false);
 
-	std::string s(std::string("Amount of ") + DefaultResourceNames[UnitUnderCursor->Type->GivesResource] + ":");
+	std::string s(std::string("Amount of ") + DefaultResourceNames[1] + ":");
 	editResourceLabel = new gcn::Label(s);
 	editResourceMenu->add(editResourceLabel, 288 / 2 - editResourceLabel->getWidth() / 2, 11);
 
 	char buf[30];
-	sprintf(buf, "%d", UnitUnderCursor->ResourcesHeld);
+	sprintf(buf, "%d", UnitUnderCursor->ResourcesHeld[1] / CYCLES_PER_SECOND);
 	editResourceTextField = new gcn::TextField(buf);
 	editResourceTextField->setBaseColor(gcn::Color(200, 200, 120));
 	editResourceTextField->setForegroundColor(gcn::Color(200, 200, 120));
@@ -542,70 +552,6 @@ static void CleanEditAi()
 	editAiCancelButton = NULL;
 }
 
-class CEditAiOKActionListener : public gcn::ActionListener
-{
-public:
-	virtual void action(const std::string &eventId) {
-		UnitUnderCursor->Active = editAiCheckBox->isMarked() ? 1 : 0;
-		editAiMenu->stop();
-	}
-};
-static CEditAiOKActionListener EditAiOKListener;
-
-class CEditAiCancelActionListener : public gcn::ActionListener
-{
-public:
-	virtual void action(const std::string &eventId) {
-		editAiMenu->stop();
-	}
-};
-static CEditAiCancelActionListener EditAiCancelListener;
-
-/**
-**  Edit ai properties
-*/
-static void EditorEditAiProperties(void)
-{
-	CleanEditAi();
-
-	editAiMenu = new MenuScreen();
-
-	editAiMenu->setOpaque(true);
-	editAiMenu->setBaseColor(gcn::Color(38, 38, 78, 130));
-	editAiMenu->setSize(288, 128);
-	editAiMenu->setPosition((Video.Width - editAiMenu->getWidth()) / 2,
-		(Video.Height - editAiMenu->getHeight()) / 2);
-	editAiMenu->setBorderSize(1);
-	editAiMenu->setDrawMenusUnder(false);
-
-	editAiLabel = new gcn::Label(_("Artificial Intelligence"));
-	editAiMenu->add(editAiLabel, 288 / 2 - editAiLabel->getWidth() / 2, 11);
-
-	editAiCheckBox = new gcn::CheckBox("Active", UnitUnderCursor->Active);
-	editAiCheckBox->setBaseColor(gcn::Color(200, 200, 120));
-	editAiCheckBox->setForegroundColor(gcn::Color(200, 200, 120));
-	editAiCheckBox->setBackgroundColor(gcn::Color(38, 38, 78));
-	editAiMenu->add(editAiCheckBox, 100, 34);
-
-	editAiOKButton = new gcn::Button(_("~!OK"));
-	editAiOKButton->setHotKey("o");
-	editAiOKButton->setSize(106, 28);
-	editAiOKButton->setBackgroundColor(gcn::Color(38, 38, 78, 130));
-	editAiOKButton->setBaseColor(gcn::Color(38, 38, 78, 130));
-	editAiOKButton->addActionListener(&EditAiOKListener);
-	editAiMenu->add(editAiOKButton, 24, 88);
-
-	editAiCancelButton = new gcn::Button(_("~!Cancel"));
-	editAiCancelButton->setHotKey("c");
-	editAiCancelButton->setSize(106, 28);
-	editAiCancelButton->setBackgroundColor(gcn::Color(38, 38, 78, 130));
-	editAiCancelButton->setBaseColor(gcn::Color(38, 38, 78, 130));
-	editAiCancelButton->addActionListener(&EditAiCancelListener);
-	editAiMenu->add(editAiCancelButton, 154, 88);
-
-	editAiMenu->run(false);
-}
-
 /*----------------------------------------------------------------------------
 --  Display
 ----------------------------------------------------------------------------*/
@@ -691,25 +637,22 @@ static void DrawPlayers(void)
 	y = UI.InfoPanel.Y + 4 + IconHeight + 10;
 
 	for (i = 0; i < PlayerMax; ++i) {
-		if (i == PlayerMax / 2) {
-			y += 20;
-		}
 		if (i == Editor.CursorPlayer && Map.Info.PlayerType[i] != PlayerNobody) {
-			Video.DrawRectangle(ColorWhite, x + i % 8 * 20, y, 20, 20);
+			Video.DrawRectangle(ColorWhite, x + i * 20, y, 20, 20);
 		}
 		Video.DrawRectangle(
 			i == Editor.CursorPlayer && Map.Info.PlayerType[i] != PlayerNobody ?
 				ColorWhite : ColorGray,
-			x + i % 8 * 20, y, 19, 19);
+			x + i * 20, y, 19, 19);
 		if (Map.Info.PlayerType[i] != PlayerNobody) {
-			Video.FillRectangle(Players[i].Color, x + 1 + i % 8 * 20, y + 1,
+			Video.FillRectangle(Players[i].Color, x + 1 + i * 20, y + 1,
 				17, 17);
 		}
 		if (i == Editor.SelectedPlayer) {
-			Video.DrawRectangle(ColorGreen, x + 1 + i % 8 * 20, y + 1, 17, 17);
+			Video.DrawRectangle(ColorGreen, x + 1 + i * 20, y + 1, 17, 17);
 		}
 		sprintf(buf, "%d", i);
-		VideoDrawTextCentered(x + i % 8 * 20 + 10, y + 7, SmallFont, buf);
+		VideoDrawTextCentered(x + i * 20 + 9, y + 3, SmallFont, buf);
 	}
 	
 	x = UI.InfoPanel.X + 4;
@@ -976,7 +919,7 @@ static void DrawMapCursor(void)
 			PopClipping();
 		} else {
 			//
-			// If there is an unit under the cursor, it's selection thing
+			// If there is a unit under the cursor, it's selection thing
 			//  is drawn somewhere else (Check DrawUnitSelection.)
 			//
 			if (!UnitUnderCursor) {
@@ -1043,9 +986,9 @@ static void DrawEditorInfo(void)
 	//
 	// Flags info
 	//
-	flags = Map.Fields[x + y * Map.Info.MapWidth].Flags;
+	flags = Map.Field(x, y)->Flags;
 	sprintf(buf, "%02X|%04X|%c%c%c%c%c%c%c%c%c%c%c%c%c",
-		Map.Fields[x + y * Map.Info.MapWidth].Value, flags,
+		Map.Field(x, y)->Value, flags,
 		flags & MapFieldUnpassable   ? 'u' : '-',
 		flags & MapFieldNoBuilding   ? 'n' : '-',
 		flags & MapFieldLandAllowed  ? 'L' : '-',
@@ -1060,7 +1003,7 @@ static void DrawEditorInfo(void)
 	//
 	// Tile info
 	//
-	tile = Map.Fields[x + y * Map.Info.MapWidth].Tile;
+	tile = Map.Field(x, y)->Tile;
 
 	for (i = 0; i < Map.Tileset.NumTiles; ++i) {
 		if (tile == Map.Tileset.Table[i]) {
@@ -1090,11 +1033,10 @@ static void ShowUnitInfo(const CUnit *unit)
 	char buf[256];
 	int i;
 
-	i = sprintf(buf, "#%d '%s' Player:#%d %s", UnitNumber(unit),
-		unit->Type->Name.c_str(), unit->Player->Index,
-		unit->Active ? "active" : "passive");
-	if (unit->Type->GivesResource) {
-		sprintf(buf + i," Amount %d", unit->ResourcesHeld);
+	i = sprintf(buf, "#%d '%s' Player:#%d", UnitNumber(unit),
+		unit->Type->Name.c_str(), unit->Player->Index);
+	if (unit->Type->CanHarvestFrom) {
+		sprintf(buf + i, " Amount %d", unit->ResourcesHeld[1] / CYCLES_PER_SECOND);
 	}
 	UI.StatusLine.Set(buf);
 }
@@ -1120,7 +1062,7 @@ void EditorUpdateDisplay(void)
 			UI.Fillers[i].X, UI.Fillers[i].Y);
 	}
 
-	if (CursorOn == CursorOnMap) {
+	if (CursorOn == CursorOnMap && Gui->getTop() == editorContainer) {
 		DrawMapCursor(); // cursor on map
 	}
 
@@ -1324,10 +1266,8 @@ static void EditorCallbackButtonDown(unsigned button)
 	//
 	if (Editor.State == EditorSelecting) {
 		if ((MouseButtons & RightButton && UnitUnderCursor)) {
-			if (UnitUnderCursor->Type->GivesResource) {
+			if (UnitUnderCursor->Type->CanHarvestFrom) {
 				EditorEditResource();
-			} else {
-				EditorEditAiProperties();
 			}
 			return;
 		}
@@ -1392,15 +1332,15 @@ static void EditorCallbackButtonDown(unsigned button)
 */
 static void EditorCallbackKeyDown(unsigned key, unsigned keychar)
 {
-	char *ptr;
+	size_t p;
 
 	if (HandleKeyModifiersDown(key, keychar)) {
 		return;
 	}
 
 	// FIXME: don't handle unicode well. Should work on all latin keyboard.
-	if ((ptr = strchr(UiGroupKeys, key))) {
-		key = '0' + ptr - UiGroupKeys;
+	if ((p = UiGroupKeys.find(key)) != std::string::npos) {
+		key = '0' + p;
 		if (key > '9') {
 			key = SDLK_BACKQUOTE;
 		}
@@ -1712,10 +1652,6 @@ static void EditorCallbackMouse(int x, int y)
 		bx = UI.InfoPanel.X + 8;
 		by = UI.InfoPanel.Y + 4 + IconHeight + 10;
 		for (i = 0; i < PlayerMax; ++i) {
-			if (i == PlayerMax / 2) {
-				bx = UI.InfoPanel.X + 8;
-				by += 20;
-			}
 			if (bx < x && x < bx + 20 && by < y && y < by + 20) {
 				if (Map.Info.PlayerType[i] != PlayerNobody) {
 					sprintf(buf,"Select player #%d",i);
@@ -1736,12 +1672,12 @@ static void EditorCallbackMouse(int x, int y)
 		i = Editor.UnitIndex;
 		by = UI.ButtonPanel.Y + 24;
 		while (by < UI.ButtonPanel.Y + ButtonPanelHeight - IconHeight) {
-			if (i >= (int) Editor.ShownUnitTypes.size()) {
+			if (i >= (int)Editor.ShownUnitTypes.size()) {
 				break;
 			}
 			bx = UI.ButtonPanel.X + 10;
 			while (bx < UI.ButtonPanel.X + 146) {
-				if (i >= (int) Editor.ShownUnitTypes.size()) {
+				if (i >= (int)Editor.ShownUnitTypes.size()) {
 					break;
 				}
 				if (bx < x && x < bx + IconWidth &&
@@ -1894,10 +1830,10 @@ static void EditorCallbackMouse(int x, int y)
 		CursorOn = CursorOnMap;
 
 		//
-		// Look if there is an unit under the cursor.
+		// Look if there is a unit under the cursor.
 		// FIXME: use Viewport2MapX Viewport2MapY
 		//
-		UnitUnderCursor = UnitOnScreen(NULL,
+		UnitUnderCursor = UnitOnScreen(
 			CursorX - UI.MouseViewport->X +
 				UI.MouseViewport->MapX * TileSizeX + UI.MouseViewport->OffsetX,
 			CursorY - UI.MouseViewport->Y +
@@ -2016,7 +1952,7 @@ void CEditor::Init(void)
 	RecalculateShownUnits();
 
 	ButtonPanelWidth = 200;
-	ButtonPanelHeight = 144;
+	ButtonPanelHeight = 160 + (Video.Height - 480);
 
 	EditorCallbacks.ButtonPressed = EditorCallbackButtonDown;
 	EditorCallbacks.ButtonReleased = EditorCallbackButtonUp;
@@ -2077,7 +2013,7 @@ void EditorMainLoop(void)
 	editorSliderListener = new EditorSliderListener();
 
 	editorSlider = new gcn::Slider();
-	editorSlider->setBaseColor(gcn::Color(38, 38, 78, 130));
+	editorSlider->setBaseColor(gcn::Color(38, 38, 78));
 	editorSlider->setForegroundColor(gcn::Color(200, 200, 120));
 	editorSlider->setBackgroundColor(gcn::Color(200, 200, 120));
 	editorSlider->setSize(176, 16);
@@ -2133,6 +2069,7 @@ void EditorMainLoop(void)
 		}
 
 		CleanModules();
+		InitDefinedVariables();
 
 		LoadCcl(); // Reload the main config file
 
@@ -2160,6 +2097,12 @@ void EditorMainLoop(void)
 */
 void StartEditor(const char *filename)
 {
+	std::string nc, rc;
+
+	GetDefaultTextColors(nc, rc);
+
+	DebugPrint("StartEditor - %s\n" _C_ filename ? filename : "new map");
+
 	if (filename) {
 		if (strcpy_s(CurrentMapPath, sizeof(CurrentMapPath), filename) != 0) {
 			filename = NULL;
@@ -2169,8 +2112,21 @@ void StartEditor(const char *filename)
 		// new map, choose some default values
 		strcpy_s(CurrentMapPath, sizeof(CurrentMapPath), "");
 		Map.Info.Description.clear();
-		Map.Info.MapWidth = 64;
-		Map.Info.MapHeight = 64;
+		// Make sure we have good values
+		if (Map.Info.MapWidth < 32) {
+			fprintf(stderr, "Invalid map width, using default value\n");
+			Map.Info.MapWidth = 32;
+		} else if (Map.Info.MapWidth > 256) {
+			fprintf(stderr, "Invalid map width, using default value\n");
+			Map.Info.MapWidth = 256;
+		}
+		if (Map.Info.MapHeight < 32) {
+			fprintf(stderr, "Invalid map height, using default value\n");
+			Map.Info.MapHeight = 32;
+		}else if (Map.Info.MapHeight > 256) {
+			fprintf(stderr, "Invalid map height, using default value\n");
+			Map.Info.MapHeight = 256;
+		}
 	}
 	
 	// Run the editor.
@@ -2186,6 +2142,8 @@ void StartEditor(const char *filename)
 	CleanPlayers();
 	CleanEditResource();
 	CleanEditAi();
+
+	SetDefaultTextColors(nc, rc);
 }
 
 //@}
