@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: game.cpp 8559 2007-02-10 17:25:47Z jim4 $
+//      $Id: game.cpp 8972 2007-07-01 02:14:16Z jim4 $
 
 //@{
 
@@ -51,7 +51,6 @@
 #include "font.h"
 #include "sound.h"
 #include "sound_server.h"
-#include "depend.h"
 #include "interface.h"
 #include "cursor.h"
 #include "spells.h"
@@ -186,13 +185,14 @@ static int WriteMapPresentation(const std::string &mapname, CMap *map, char *map
 		f->printf("-- File licensed under the GNU GPL version 2.\n\n");
 
 		f->printf("DefinePlayerTypes(");
-		f->printf("\"%s\"", type[map->Info.PlayerType[0]]);
 		while (topplayer > 0 && map->Info.PlayerType[topplayer] == PlayerNobody) {
 			--topplayer;
 		}
-		for (i = 1; i <= topplayer; ++i) {
-			f->printf(", \"%s\"", type[map->Info.PlayerType[i]]);
-			++numplayers;
+		for (i = 0; i <= topplayer; ++i) {
+			f->printf("%s\"%s\"", (i ? ", " : ""), type[map->Info.PlayerType[i]]);
+			if (map->Info.PlayerType[i] == PlayerPerson) {
+				++numplayers;
+			}
 		}
 		f->printf(")\n");
 	
@@ -237,24 +237,16 @@ int WriteMapSetup(const char *mapsetup, CMap *map, int writeTerrain)
 		f->printf("-- File licensed under the GNU GPL version 2.\n\n");
 	
 		f->printf("-- player configuration\n");
-		for (i = 0; i < PlayerMax; ++i) {
-			f->printf("SetStartView(%d, %d, %d)\n", i, Players[i].StartX, Players[i].StartY);
-			f->printf("SetPlayerData(%d, \"Resources\", \"%s\", %d)\n",
-				i, DefaultResourceNames[WoodCost], 
-				Players[i].Resources[WoodCost]);
-			f->printf("SetPlayerData(%d, \"Resources\", \"%s\", %d)\n",
-				i, DefaultResourceNames[GoldCost], 
-				Players[i].Resources[GoldCost]);
-			f->printf("SetPlayerData(%d, \"Resources\", \"%s\", %d)\n",
-				i, DefaultResourceNames[OilCost], 
-				Players[i].Resources[OilCost]);
-			f->printf("SetAiType(%d, \"%s\")\n",
-				i, Players[i].AiName.c_str());
+		for (i = 0; i < PlayerMax - 1; ++i) {
+			f->printf("Players[%d]:SetStartView(%d, %d)\n", i, Players[i].StartX, Players[i].StartY);
+			f->printf("Players[%d].EnergyStored = %d\n", i, Players[i].GetEnergyStored());
+			f->printf("Players[%d].MagmaStored = %d\n", i, Players[i].GetMagmaStored());
+			f->printf("Players[%d].AiName = \"%s\"\n", i, Players[i].AiName.c_str());
 		}
 		f->printf("\n");
 
 		f->printf("-- load tilesets\n");
-		f->printf("LoadTileModels(\"%s\")\n\n", map->TileModelsFileName);
+		f->printf("LoadTileModels(\"%s\")\n\n", map->TileModelsFileName.c_str());
 	
 		if (writeTerrain) {
 			f->printf("-- Tiles Map\n");
@@ -263,8 +255,8 @@ int WriteMapSetup(const char *mapsetup, CMap *map, int writeTerrain)
 					int tile;
 					int n;
 			
-					tile = map->Fields[j+i*map->Info.MapWidth].Tile;
-					for (n=0; n < map->Tileset.NumTiles && tile != map->Tileset.Table[n]; ++n) {
+					tile = map->Field(j, i)->Tile;
+					for (n = 0; n < map->Tileset.NumTiles && tile != map->Tileset.Table[n]; ++n) {
 					}
 					f->printf("SetTile(%3d, %d, %d)\n", n, j, i);
 				}
@@ -273,13 +265,13 @@ int WriteMapSetup(const char *mapsetup, CMap *map, int writeTerrain)
 
 		f->printf("-- place units\n");
 		for (i = 0; i < NumUnits; ++i) {
-			f->printf("unit= CreateUnit(\"%s\", %d, {%d, %d})\n",
-				Units[i]->Type->Ident.c_str(),
-				Units[i]->Player->Index,
-				Units[i]->X, Units[i]->Y);
-			if (Units[i]->Type->GivesResource) {
-				f->printf("SetResourcesHeld(unit, %d)\n", Units[i]->ResourcesHeld);
+			f->printf("CreateUnit(\"%s\", ", Units[i]->Type->Ident.c_str());
+			if (Units[i]->Player->Index == PlayerMax - 1) {
+				f->printf("PlayerNumNeutral");
+			} else {
+				f->printf("%d", Units[i]->Player->Index);
 			}
+			f->printf(", {%d, %d})\n", Units[i]->X, Units[i]->Y);
 		}
 		f->printf("\n\n");
 	} catch (const FileException &) {
@@ -504,7 +496,7 @@ void CreateGame(const std::string &filename, CMap *map)
 	int i;
 
 	if (SaveGameLoading) {
-		SaveGameLoading = 0;
+		SaveGameLoading = false;
 		// Load game, already created game with Init/LoadModules
 		CommandLog(NULL, NoUnitP, FlushCommands, -1, -1, NoUnitP, NULL, -1);
 		return;
@@ -552,8 +544,8 @@ void CreateGame(const std::string &filename, CMap *map)
 		DebugPrint("Client setup: Calling InitNetwork2\n");
 		InitNetwork2();
 	} else {
-		if (LocalPlayerName && strcmp(LocalPlayerName, "Anonymous")) {
-		  ThisPlayer->SetName(LocalPlayerName);
+		if (!LocalPlayerName.empty() && LocalPlayerName != "Anonymous") {
+			ThisPlayer->SetName(LocalPlayerName);
 		}
 	}
 
@@ -663,16 +655,6 @@ void CreateGame(const std::string &filename, CMap *map)
 	DebugPlayers();
 	PlayersInitAi();
 
-	//
-	// Upgrades
-	//
-	InitUpgrades();
-
-	//
-	// Dependencies
-	//
-	InitDependencies();
-
 	//
 	// Buttons (botpanel)
 	//
