@@ -9,7 +9,7 @@
 //
 /**@name ui.h - The user interface header file. */
 //
-//      (c) Copyright 1999-2006 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1999-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -24,17 +24,12 @@
 //      along with this program; if not, write to the Free Software
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
-//
-//      $Id: ui.h 8550 2007-02-10 12:33:10Z feb $
 
 #ifndef __UI_H__
 #define __UI_H__
 
 //@{
 
-/// @todo this only the start of the new user interface
-/// @todo all user interface variables should go here and be configurable
-
 /*----------------------------------------------------------------------------
 --  Includes
 ----------------------------------------------------------------------------*/
@@ -145,7 +140,7 @@ public:
 **
 **  CViewport::Unit
 **
-**    Viewport is bound to an unit. If the unit moves the viewport
+**    Viewport is bound to a unit. If the unit moves the viewport
 **    changes the position together with the unit.
 **    @todo binding to a group.
 */
@@ -470,7 +465,7 @@ public:
 	int TextX;                        /// text X position
 	int TextY;                        /// text Y position
 };
-#define MaxResourceInfo  MaxCosts + 2 /// +2 for food and score
+#define MaxResourceInfo  MaxCosts + 1 /// +2 for score
 
 class CInfoPanel
 {
@@ -565,10 +560,6 @@ public:
 	int TrainingTextX;                  /// Multiple Training X Text position
 	int TrainingTextY;                  /// Multiple Training Y Text position
 
-	CUIButton *UpgradingButton;         /// Button info for upgrade
-
-	CUIButton *ResearchingButton;       /// Button info for researching
-
 	std::vector<CUIButton> TransportingButtons;/// Button info for transporting
 
 	// Completed bar
@@ -651,8 +642,7 @@ extern std::map<std::string, ButtonStyle *> ButtonStyleHash;
 extern bool RightButtonAttacks;         /// right button attacks
 extern ButtonAction *CurrentButtons;    /// Current Selected Buttons
 
-extern char DefaultGroupKeys[];         /// Default group keys
-extern char *UiGroupKeys;               /// Up to 11 keys used for group selection
+extern std::string UiGroupKeys;         /// Up to 11 keys used for group selection
 
 // only exported to save them
 
@@ -666,6 +656,9 @@ extern void InitUserInterface(void);
 extern void SaveUserInterface(CFile *file);
 	/// Clean up the ui module
 extern void CleanUserInterface(void);
+#ifdef DEBUG
+extern void FreeButtonStyles();
+#endif
 	/// Register ccl features
 extern void UserInterfaceCclRegister(void);
 
