@@ -9,7 +9,7 @@
 //
 /**@name unit.h - The unit headerfile. */
 //
-//      (c) Copyright 1998-2006 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -24,8 +24,6 @@
 //      along with this program; if not, write to the Free Software
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
-//
-//      $Id: unit.h 8550 2007-02-10 12:33:10Z feb $
 
 #ifndef __UNIT_H__
 #define __UNIT_H__
@@ -213,8 +211,7 @@
 **
 **  CUnit::Destroyed
 **
-** @todo docu.
-**  If you need more informations, please send me an email or write it self.
+**  Unit is destroyed. pending reference.
 **
 **  CUnit::Removed
 **
@@ -243,11 +240,6 @@
 **
 **  This is 1 if the unit is on board a transporter.
 **
-**
-**  CUnit::XP
-**
-**  Number of XP of the unit.
-**
 **  CUnit::Kills
 **
 **  How many units have been killed by the unit.
@@ -267,7 +259,6 @@
 **
 **  This values hold the amount of resources in a resource or in
 **  in a harvester.
-**  @todo continue documentation
 **
 **  CUnit::SubAction
 **
@@ -288,7 +279,6 @@
 **
 **  CUnit::Reset
 **
-**  @todo continue documentation
 **
 **  CUnit::Blink
 **
@@ -345,9 +335,6 @@
 **
 **  Generic goal pointer. Used by teleporters to point to circle of power.
 **
-**
-** @todo continue documentation
-**
 */
 
 /*----------------------------------------------------------------------------
@@ -356,6 +343,7 @@
 
 #include <vector>
 #include "SDL.h"
+#include "upgrade_structs.h"
 
 /*----------------------------------------------------------------------------
 --  Declarations
@@ -369,13 +357,15 @@ class SpellType;
 class CUnitColors;
 class CConstructionFrame;
 class CVariable;
-class CUpgrade;
 class CBuildRestrictionOnTop;
 class CFile;
 struct lua_State;
 class CViewport;
 class CAnimation;
 
+	/// Called whenever the selected unit was updated
+extern void SelectedUnitChanged(void);
+
 /**
 **  Unit references over network, or for memory saving.
 */
@@ -402,8 +392,6 @@ typedef enum _unit_action_ {
 	UnitActionSpellCast,    /// unit casts spell
 
 	UnitActionTrain,        /// building is training
-	UnitActionUpgradeTo,    /// building is upgrading itself
-	UnitActionResearch,     /// building is researching spell
 	UnitActionBuilt,      /// building is under construction
 
 // Compound actions
@@ -414,8 +402,6 @@ typedef enum _unit_action_ {
 
 	UnitActionRepair,       /// unit repairing
 	UnitActionResource,     /// unit harvesting resources
-	UnitActionReturnGoods,  /// unit returning any resource
-	UnitActionTransformInto /// unit transform into type.
 } UnitAction;
 
 /**
@@ -457,9 +443,7 @@ public:
 			int X;                    /// X position for patroling.
 			int Y;                    /// Y position for patroling.
 		} Patrol;                     /// position.
-		int ResourcePos;              /// ResourcePos == (X<<16 | Y).
 		SpellType *Spell;             /// spell when casting.
-		CUpgrade *Upgrade;            /// upgrade.
 	} Arg1;             /// Extra command argument.
 };
 
@@ -537,7 +521,6 @@ public:
 		Selected = 0;
 		TeamSelected = 0;
 		Constructed = 0;
-		Active = 0;
 		Boarded = 0;
 		RescuedFrom = NULL;
 		memset(VisCount, 0, sizeof(VisCount));
@@ -546,7 +529,8 @@ public:
 		TTL = 0;
 		GroupId = 0;
 		LastGroup = 0;
-		ResourcesHeld = 0;
+		memset(ResourcesHeld, 0, sizeof(ResourcesHeld));
+		ProductionEfficiency = 100;
 		SubAction = 0;
 		Wait = 0;
 		State = 0;
@@ -554,13 +538,11 @@ public:
 		Moving = 0;
 		ReCast = 0;
 		memset(&Anim, 0, sizeof(Anim));
-		CurrentResource = 0;
 		OrderCount = 0;
 		OrderFlush = 0;
 		Orders.clear();
 		SavedOrder.Init();
 		NewOrder.Init();
-		CriticalOrder.Init();
 		AutoCastSpell = NULL;
 		AutoRepair = 0;
 		memset(&Data, 0, sizeof(Data));
@@ -573,8 +555,8 @@ public:
 	CUnit **UnitSlot;     /// Slot pointer of Units
 	CUnit **PlayerSlot;   /// Slot pointer of Player->Units
 
-	CUnit        *Next;          /// Generic link pointer (on map)
-	unsigned      CacheLock : 1; /// Used to lock unit out of the cache.
+	CUnit   *Next;          /// Generic link pointer (on map)
+	unsigned CacheLock : 1; /// Used to lock unit out of the cache.
 
 	int    InsideCount;   /// Number of units inside.
 	int    BoardCount;    /// Number of units transported inside.
@@ -583,8 +565,8 @@ public:
 	CUnit *NextContained; /// Next unit in the container.
 	CUnit *PrevContained; /// Previous unit in the container.
 
-	int X; /// Map position X
-	int Y; /// Map position Y
+	int X;                /// Map position X
+	int Y;                /// Map position Y
 
 	CUnitType  *Type;              /// Pointer to unit-type (peon,...)
 	CPlayer    *Player;            /// Owner of this unit
@@ -605,16 +587,15 @@ public:
 	unsigned Destroyed : 1; /// unit is destroyed pending reference
 	unsigned Removed : 1;   /// unit is removed (not on map)
 	unsigned Selected : 1;  /// unit is selected
-	unsigned TeamSelected;  /// unit is selected by a team member.
 
 	unsigned Constructed : 1;    /// Unit is in construction
-	unsigned Active : 1;         /// Unit is active for AI
 	unsigned Boarded : 1;        /// Unit is on board a transporter.
+	unsigned TeamSelected;       /// unit is selected by a team member.
 	CPlayer *RescuedFrom;        /// The original owner of a rescued unit.
 							     /// NULL if the unit was not rescued.
 	/* Seen stuff. */
 	int VisCount[PlayerMax];     /// Unit visibility counts
-	struct _seen_stuff_ {
+	struct _unit_seen_ {
 		unsigned            ByPlayer : PlayerMax;    /// Track unit seen by player
 		int                 Frame;                   /// last seen frame/stage of buildings
 		CUnitType          *Type;                    /// Pointer to last seen unit-type
@@ -625,17 +606,18 @@ public:
 		unsigned            Constructed : 1;         /// Unit seen construction
 		unsigned            State : 3;               /// Unit seen build/upgrade state
 		unsigned            Destroyed : PlayerMax;   /// Unit seen destroyed or not
-		CConstructionFrame  *CFrame;                  /// Seen construction frame
+		CConstructionFrame *CFrame;                  /// Seen construction frame
 	} Seen;
 
 	CVariable *Variable; /// array of User Defined variables.
 
-	unsigned long TTL;  /// time to live
+	unsigned long TTL;   /// time to live
 
-	int GroupId;        /// unit belongs to this group id
-	int LastGroup;      /// unit belongs to this last group
+	int GroupId;         /// unit belongs to this group id
+	int LastGroup;       /// unit belongs to this last group
 
-	int ResourcesHeld;      /// Resources Held by a unit
+	int ResourcesHeld[MaxCosts];       /// Resources held by a unit
+	unsigned ProductionEfficiency : 8; /// Production efficiency
 
 	unsigned SubAction : 8; /// sub-action of unit
 	unsigned Wait;          /// action counter
@@ -645,22 +627,19 @@ public:
 	unsigned ReCast : 1;    /// Recast again next cycle
 
 	struct _unit_anim_ {
-		const CAnimation *Anim;                     /// Anim
-		const CAnimation *CurrAnim;                 /// CurrAnim
-		int Wait;                                   /// Wait
-		int Unbreakable;                            /// Unbreakable
+		const CAnimation *Anim;     /// Anim
+		const CAnimation *CurrAnim; /// CurrAnim
+		int Wait;                   /// Wait
+		int Unbreakable;            /// Unbreakable
 	} Anim;
 
-	unsigned char CurrentResource;
-
-	char OrderCount;            /// how many orders in queue
-	char OrderFlush;            /// cancel current order, take next
+	char OrderCount;              /// how many orders in queue
+	char OrderFlush;              /// cancel current order, take next
 	std::vector<COrder *> Orders; /// orders to process
-	COrder SavedOrder;           /// order to continue after current
-	COrder NewOrder;             /// order for new trained units
-	COrder CriticalOrder;        /// order to do as possible in breakable animation.
-	char *AutoCastSpell;        /// spells to auto cast
-	unsigned AutoRepair : 1;    /// True if unit tries to repair on still action.
+	COrder SavedOrder;            /// order to continue after current
+	COrder NewOrder;              /// order for new trained units
+	char *AutoCastSpell;          /// spells to auto cast
+	unsigned AutoRepair : 1;      /// True if unit tries to repair on still action.
 
 	union _order_data_ {
 	struct _order_move_ {
@@ -673,30 +652,20 @@ public:
 		CUnit *Worker;              /// Worker building this unit
 		int Progress;               /// Progress counter, in 1/100 cycles.
 		int Cancel;                 /// Cancel construction
-		CConstructionFrame *Frame;   /// Construction frame
+		CConstructionFrame *Frame;  /// Construction frame
 	} Built; /// ActionBuilt,...
-	struct _order_build_ {
-		int Cycles;                 /// Cycles unit has been building for
-	} Build; /// ActionBuild
+	struct _order_harvest_ {
+		int CurrentProduction[MaxCosts];
+	} Harvest; /// Harvest
 	struct _order_resource_ {
 		int Active; /// how many units are harvesting from the resource.
 	} Resource; /// Resource still
-	struct _order_resource_worker_ {
-		int TimeToHarvest;          /// how much time until we harvest some more.
-		unsigned DoneHarvesting:1;  /// Harvesting done, wait for action to break.
-	} ResWorker; /// Worker harvesting
-	struct _order_repair_ {
-		int Cycles;                 /// Cycles unit has been repairing for
-	} Repair; /// Repairing unit
-	struct _order_research_ {
-		CUpgrade *Upgrade;          /// Upgrade researched
-	} Research; /// Research action
-	struct _order_upgradeto_ {
-		int Ticks; /// Ticks to complete
-	} UpgradeTo; /// Upgrade to action
 	struct _order_train_ {
 		int Ticks;                  /// Ticks to complete
 	} Train; /// Train units action
+	struct _order_repair_ {
+		int Progress;               /// Progress counter
+	} Repair; /// Repair
 	} Data; /// Storage room for different commands
 
 	CUnit *Goal; /// Generic goal pointer
@@ -706,6 +675,14 @@ public:
 		return Orders[0]->Action == UnitActionStill && OrderCount == 1;
 	}
 
+	inline void ClearAction() {
+		Orders[0]->Action = UnitActionStill;
+		SubAction = 0;
+		if (Selected) {
+			SelectedUnitChanged();
+		}
+	}
+
 	/// Increase a unit's reference count
 	void RefsIncrease();
 	/// Decrease a unit's reference count
@@ -763,12 +740,12 @@ public:
 	bool IsUnusable() const;
 };
 
-#define NoUnitP (CUnit *)0         /// return value: for no unit found
-#define InfiniteDistance INT_MAX /// the distance is unreachable
+#define NoUnitP (CUnit *)0        /// return value: for no unit found
+#define InfiniteDistance INT_MAX  /// the distance is unreachable
 
-#define FlushCommands 1          /// Flush commands in queue
+#define FlushCommands 1           /// Flush commands in queue
 
-#define MAX_UNIT_SLOTS 65535     /// Maximal number of used slots
+#define MAX_UNIT_SLOTS 65535      /// Maximal number of used slots
 
 /**
 **  Returns unit number (unique to this unit)
@@ -776,7 +753,7 @@ public:
 #define UnitNumber(unit) ((unit)->Slot)
 
 /**
-** How many groups supported
+**  How many groups supported
 */
 #define NUM_GROUPS 10
 
@@ -787,11 +764,11 @@ class CPreference {
 public:
 	CPreference() : ShowSightRange(false), ShowReactionRange(false),
 		ShowAttackRange(false), ShowOrders(0) {};
-public:
+
 	bool ShowSightRange;     /// Show right range.
 	bool ShowReactionRange;  /// Show reaction range.
 	bool ShowAttackRange;    /// Show attack range.
-	int ShowOrders;          /// How many second show orders of unit on map.
+	int  ShowOrders;         /// How many second show orders of unit on map.
 };
 
 extern CPreference Preference;
@@ -806,17 +783,14 @@ extern unsigned int UnitSlotFree;        /// First free unit slot
 extern CUnit *Units[MAX_UNIT_SLOTS]; /// Units used
 extern int NumUnits;                 /// Number of units used
 
-// in unit_draw.c
+// in unit_draw.cpp
 /// @todo could be moved into the user interface ?
 extern unsigned long ShowOrdersCount;   /// Show orders for some time
-extern int XpDamage;                    /// unit XP adds more damage to attacks
-extern bool EnableTrainingQueue;        /// Config: training queues enabled
 extern bool EnableBuildingCapture;      /// Config: building capture enabled
-extern bool RevealAttacker;             /// Config: reveal attacker enabled
-extern const CViewport *CurrentViewport; /// CurrentViewport
+extern const CViewport *CurrentViewport;/// CurrentViewport
 extern void DrawUnitSelection(const CUnit *);
-extern void (*DrawSelection)(Uint32, int, int, int, int);
-extern int MaxSelectable;                  /// How many units could be selected
+extern void DrawSelection(Uint32 color, int x1, int y1, int x2, int y2);
+extern int MaxSelectable;                   /// How many units could be selected
 
 extern CUnit **Selected;                    /// currently selected units
 extern CUnit **TeamSelected[PlayerMax];     /// teams currently selected units
@@ -832,9 +806,9 @@ extern CUnit *ReleasedTail;                 /// Tail of the released unit list.
 
 	/// Prepare unit memory allocator
 extern void InitUnitsMemory(void);
-	///  Mark the field with the FieldFlags.
+	/// Mark the field with the FieldFlags.
 void MarkUnitFieldFlags(const CUnit *unit);
-	///  Unmark the field with the FieldFlags.
+	/// Unmark the field with the FieldFlags.
 void UnmarkUnitFieldFlags(const CUnit *unit);
 	/// Update unit->CurrentSightRange.
 void UpdateUnitSightRange(CUnit *unit);
@@ -888,10 +862,10 @@ extern bool CanBuildOn(int x, int y, int mask);
 	/// FIXME: more docu
 extern CUnit *CanBuildUnitType(const CUnit *unit, const CUnitType *type, int x, int y, int real);
 
+	/// Holds resources
+extern bool UnitHoldsResources(const CUnit *unit);
 	/// Find resource
-extern CUnit *UnitFindResource(const CUnit *unit, int x, int y, int range, int resource);
-	/// Find nearest deposit
-extern CUnit *FindDeposit(const CUnit *unit, int x, int y, int range, int resource);
+extern CUnit *UnitFindResource(const CUnit *unit, int x, int y, int range, int resource = -1);
 	/// Find the next idle worker
 extern CUnit *FindIdleWorker(const CPlayer *player, const CUnit *last);
 
@@ -900,7 +874,9 @@ extern int FindTerrainType(int movemask, int resmask, int rvresult, int range,
 		const CPlayer *player, int x, int y, int *px, int *py);
 
 	/// @todo more docu
-extern CUnit *UnitOnScreen(CUnit *unit, int x, int y);
+extern CUnit *UnitOnScreen(int x, int y);
+	/// Check if a unit should be removed from UnitsConsumingResources
+extern void UnitRemoveConsumingResources(CUnit *unit);
 
 	/// Let a unit die
 extern void LetUnitDie(CUnit *unit);
@@ -947,7 +923,7 @@ extern void InitUnits(void);
 	/// Clean unit module
 extern void CleanUnits(void);
 
-// in unitcache.c
+// in unitcache.cpp
 	/// Insert new unit into cache
 extern void UnitCacheInsert(CUnit *unit);
 	/// Remove unit from cache
@@ -959,21 +935,7 @@ extern int UnitCacheOnTile(int x, int y, CUnit **table);
 	/// Initialize unit-cache
 extern void InitUnitCache(void);
 
-// in unit_draw.c
-//--------------------
-	/// Draw nothing around unit
-extern void DrawSelectionNone(Uint32, int, int, int, int);
-	/// Draw circle around unit
-extern void DrawSelectionCircle(Uint32, int, int, int, int);
-	/// Draw circle filled with alpha around unit
-extern void DrawSelectionCircleWithTrans(Uint32, int, int, int, int);
-	/// Draw rectangle around unit
-extern void DrawSelectionRectangle(Uint32, int, int, int, int);
-	/// Draw rectangle filled with alpha around unit
-extern void DrawSelectionRectangleWithTrans(Uint32, int, int, int, int);
-	/// Draw corners around unit
-extern void DrawSelectionCorners(Uint32, int, int, int, int);
-
+// in unit_draw.cpp
 	/// Register CCL decorations features
 extern void DecorationCclRegister(void);
 	/// Load the decorations (health,mana) of units
@@ -989,7 +951,7 @@ extern int FindAndSortUnits(const CViewport *vp, CUnit **table);
 	/// Show a unit's orders.
 extern void ShowOrder(const CUnit *unit);
 
-// in unit_find.c
+// in unit_find.cpp
 	/// Select unit on X,Y of type naval,fly,land
 extern CUnit *UnitCacheOnXY(int x, int y, unsigned type);
 	/// Find all units of this type
@@ -1002,9 +964,7 @@ extern CUnit *UnitOnMapTile(int tx, int ty);
 extern CUnit *TargetOnMap(const CUnit *unit, int x1, int y1, int x2, int y2);
 
 	/// Return resource, if on map tile
-extern CUnit *ResourceOnMap(int tx, int ty, int resource);
-	/// Return resource deposit, if on map tile
-extern CUnit *ResourceDepositOnMap(int tx, int ty, int resource);
+extern CUnit *ResourceOnMap(int tx, int ty, int resource = -1);
 
 	/// Find best enemy in numeric range to attack
 extern CUnit *AttackUnitsInDistance(const CUnit *unit, int range);
@@ -1013,8 +973,7 @@ extern CUnit *AttackUnitsInRange(const CUnit *unit);
 	/// Find best enemy in reaction range to attack
 extern CUnit *AttackUnitsInReactRange(const CUnit *unit);
 
-// in groups.c
-
+// in groups.cpp
 	/// Initialize data structures for groups
 extern void InitGroups(void);
 	/// Save groups
@@ -1039,8 +998,7 @@ extern void RemoveUnitFromGroups(CUnit *unit);
 	/// Register CCL group features
 extern void GroupCclRegister(void);
 
-// in selection.c
-
+// in selection.cpp
 	/// Check if unit is the currently only selected
 #define IsOnlySelected(unit) (NumSelected == 1 && Selected[0] == (unit))
 
@@ -1094,8 +1052,7 @@ extern void CleanSelections(void);
 	/// Register CCL selection features
 extern void SelectionCclRegister(void);
 
-// in ccl_unit.c
-
+// in script_unit.cpp
 	/// Parse order
 extern void CclParseOrder(lua_State *l, COrder *order);
 	/// register CCL units features
