@@ -9,7 +9,7 @@
 //
 /**@name unittype.h - The unit-types headerfile. */
 //
-//      (c) Copyright 1998-2006 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -24,8 +24,6 @@
 //      along with this program; if not, write to the Free Software
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
-//
-//      $Id: unittype.h 8550 2007-02-10 12:33:10Z feb $
 
 #ifndef __UNITTYPE_H__
 #define __UNITTYPE_H__
@@ -122,9 +120,6 @@
 **    Corpse unit-type pointer, only this should be used during run
 **    time. Many unit-types can share the same corpse.
 **
-**
-**  @todo continue this documentation
-**
 **  CUnitType::Construction
 **
 **    What is shown in construction phase.
@@ -138,19 +133,11 @@
 **    Maximum hit points
 **
 **
-**  CUnitType::_Costs[::MaxCosts]
-**
-**    How many resources needed
-**
 **  CUnitType::RepairHP
 **
 **    The HP given to a unit each cycle it's repaired.
 **    If zero, unit cannot be repaired
 **
-**    CUnitType::RepairCosts[::MaxCosts]
-**
-**    Costs per repair cycle to fix a unit.
-**
 **  CUnitType::TileWidth
 **
 **    Tile size on map width
@@ -246,16 +233,6 @@
 **
 **    Death explosion animated
 **
-**  CUnitType::RandomMovementProbability
-**
-**    When the unit is idle this is the probability that it will
-**    take a step in a random direction, in percents.
-**
-**  CUnitType::ClicksToExplode
-**
-**    If this is non-zero, then after that many clicks the unit will
-**    commit suicide. Doesn't work with resource workers/resources.
-**
 **  CUnitType::Building
 **
 **    Unit is a Building
@@ -281,38 +258,19 @@
 **
 **    Maximum units on board (for transporters), and resources
 **
-**  CUnitType::StartingResources
-**    Amount of Resources a unit has when It's Built
+**  CUnitType::CanHarvestFrom
 **
-**  CUnitType::GivesResource
-**
-**    This equals to the resource Id of the resource given
-**    or 0 (TimeCost) for other buildings.
-**
-**  CUnitType::CanHarvest
-**
-**    Resource can be harvested. It's false for things like
-**    oil patches.
-**  @todo crappy name.
+**    Resource can be harvested from.
 **
 **  CUnitType::Harvester
 **
-**    Unit is a resource worker. Faster than examining ResInfo
-**
-**  CUnitType::ResInfo[::MaxCosts]
-**
-**    Information about resource harvesting. If NULL, it can't
-**    harvest it.
+**    Unit is a resource worker.
 **
 **  CUnitType::NeutralMinimapColorRGB
 **
 **    Says what color a unit will have when it's neutral and
 **    is displayed on the minimap.
 **
-**  CUnitType::CanStore[::MaxCosts]
-**
-**    What resource types we can store here.
-**
 **  CUnitType::Vanishes
 **
 **    Corpes & destroyed places
@@ -369,22 +327,10 @@
 **
 **  CUnitType::Weapon
 **
-**    Currently sound for weapon
+**    Current sound for weapon
 **
 **  @todo temporary solution
 **
-**  CUnitType::Supply
-**
-**    How much food does this unit supply.
-**
-**  CUnitType::Demand
-**
-**    Food demand
-**
-**  CUnitType::ImproveIncomes[::MaxCosts]
-**
-**    Gives the player an improved income.
-**
 **  CUnitType::FieldFlags
 **
 **    Flags that are set, if a unit enters a map field or cleared, if
@@ -417,71 +363,6 @@
 **
 **    Sprite images of the player colors.  This image is drawn
 **    over CUnitType::Sprite.  Used with OpenGL only.
-**
-**
-*/
-/**
-**
-**  @class ResourceInfo unittype.h
-**
-** \#include "unittype.h"
-**
-**    This class contains information about how a unit will harvest a resource.
-**
-**  ResourceInfo::FileWhenLoaded
-**
-**    The harvester's animation file will change when it's loaded.
-**
-**  ResourceInfo::FileWhenEmpty;
-**
-**    The harvester's animation file will change when it's empty.
-**    The standard animation is used only when building/repairing.
-**
-**
-**  ResourceInfo::HarvestFromOutside
-**
-**    Unit will harvest from the outside. The unit will use it's
-**    Attack animation (seems it turned into a generic Action anim.)
-**
-**  ResourceInfo::ResourceId
-**
-**    The resource this is for. Mostly redundant.
-**
-**  ResourceInfo::FinalResource
-**
-**    The resource is converted to this at the depot. Usefull for
-**    a fisherman who harvests fish, but it all turns to food at the
-**    depot.
-**
-**  ResourceInfo::WaitAtResource
-**
-**    Cycles the unit waits while inside a resource.
-**
-**  ResourceInfo::ResourceStep
-**
-**    The unit makes so-caled mining cycles. Each mining cycle
-**    it does some sort of animation and gains ResourceStep
-**    resources. You can stop after any number of steps.
-**    when the quantity in the harvester reaches the maximum
-**    (ResourceCapacity) it will return home. I this is 0 then
-**    it's considered infinity, and ResourceCapacity will now
-**    be the limit.
-**
-**  ResourceInfo::ResourceCapacity
-**
-**    Maximum amount of resources a harvester can carry. The
-**    actual amount can be modified while unloading.
-**
-**  ResourceInfo::LoseResources
-**
-**    Special lossy behaviour for loaded harvesters. Harvesters
-**    with loads other than 0 and ResourceCapacity will lose their
-**    cargo on any new order.
-**
-**  ResourceInfo::WaitAtDepot
-**
-**    Cycles the unit waits while inside the depot to unload.
-**
 */
 
 /*----------------------------------------------------------------------------
@@ -516,34 +397,12 @@ CUnitType *UnitTypeByIdent(const std::string &ident);
 */
 class MissileConfig {
 public:
-	MissileConfig() : Name(NULL), Missile(NULL) {}
+	MissileConfig() : Missile(NULL) {}
 
-	char *Name;              /// Config missile name
+	std::string Name;        /// Config missile name
 	MissileType *Missile;    /// Identifier to use to run time
 };
 
-class ResourceInfo {
-public:
-	ResourceInfo() : HarvestFromOutside(0), WaitAtResource(0), ResourceStep(0),
-		ResourceCapacity(0), WaitAtDepot(0), ResourceId(0), FinalResource(0),
-		LoseResources(0), SpriteWhenLoaded(NULL), SpriteWhenEmpty(NULL)
-	{}
-
-	std::string FileWhenLoaded;     /// Change the graphic when the unit is loaded.
-	std::string FileWhenEmpty;      /// Change the graphic when the unit is empty.
-	unsigned HarvestFromOutside;    /// Unit harvests without entering the building.
-	unsigned WaitAtResource;        /// Cycles the unit waits while mining.
-	unsigned ResourceStep;          /// Resources the unit gains per mining cycle.
-	int      ResourceCapacity;      /// Max amount of resources to carry.
-	unsigned WaitAtDepot;           /// Cycles the unit waits while returning.
-	unsigned ResourceId;            /// Id of the resource harvested. Redundant.
-	unsigned FinalResource;         /// Convert resource when delivered.
-	unsigned LoseResources;         /// The unit will lose it's resource when distracted.
-	//  Runtime info:
-	CPlayerColorGraphic *SpriteWhenLoaded; /// The graphic corresponding to FileWhenLoaded.
-	CPlayerColorGraphic *SpriteWhenEmpty;  /// The graphic corresponding to FileWhenEmpty
-};
-
 /**
 **  User defined variable type.
 **
@@ -578,7 +437,7 @@ enum {
 	CANATTACK_INDEX,
 	BUILDEROUTSIDE_INDEX,
 	BUILDERLOST_INDEX,
-	CANHARVEST_INDEX,
+	CANHARVESTFROM_INDEX,
 	HARVESTER_INDEX,
 	SELECTABLEBYRECTANGLE_INDEX,
 	ISNOTSELECTABLE_INDEX,
@@ -593,15 +452,9 @@ enum {
 	BUILD_INDEX,
 	MANA_INDEX,
 	TRANSPORT_INDEX,
-	RESEARCH_INDEX,
 	TRAINING_INDEX,
-	UPGRADINGTO_INDEX,
 	GIVERESOURCE_INDEX,
-	CARRYRESOURCE_INDEX,
-	XP_INDEX,
 	KILL_INDEX,
-	SUPPLY_INDEX,
-	DEMAND_INDEX,
 	ARMOR_INDEX,
 	SIGHTRANGE_INDEX,
 	ATTACKRANGE_INDEX,
@@ -612,11 +465,6 @@ enum {
 	RADAR_INDEX,
 	RADARJAMMER_INDEX,
 	AUTOREPAIRRANGE_INDEX,
-	BLOODLUST_INDEX,
-	HASTE_INDEX,
-	SLOW_INDEX,
-	INVISIBLE_INDEX,
-	UNHOLYARMOR_INDEX,
 	SLOT_INDEX,
 	NVARALREADYDEFINED,
 };
@@ -626,10 +474,7 @@ class CUnitType;
 
 
 /**
-**  Decoration for userdefined variable.
-**
-**    It is used to show variables graphicly.
-**  @todo add more stuff in this struct.
+**  Decoration for user defined variable.
 */
 class CDecoVar {
 public:
@@ -648,13 +493,11 @@ public:
 	int OffsetXPercent;         /// Percent offset (TileWidth) in X coord.
 	int OffsetYPercent;         /// Percent offset (TileHeight) in Y coord.
 
-	char IsCenteredInX;         /// if true, use center of deco instead of left border
-	char IsCenteredInY;         /// if true, use center of deco instead of upper border
+	bool IsCenteredInX;         /// if true, use center of deco instead of left border
+	bool IsCenteredInY;         /// if true, use center of deco instead of upper border
 
-	char ShowIfNotEnable;       /// if false, Show only if var is enable
-	char ShowWhenNull;          /// if false, don't show if var is null (F.E poison)
-	char HideHalf;              /// if true, don't show when 0 < var < max.
-	char ShowWhenMax;           /// if false, don't show if var is to max. (Like mana)
+	bool ShowIfNotEnable;       /// if false, Show only if var is enable
+	bool ShowWhenMax;           /// if false, don't show if var is to max. (Like mana)
 	bool ShowOnlySelected;      /// if true, show only for selected units.
 
 	bool HideNeutral;           /// if true, don't show for neutral unit.
@@ -662,34 +505,6 @@ public:
 	bool ShowOpponent;          /// if true, show for opponent unit.
 };
 
-class CDecoVarBar : public CDecoVar
-{
-public:
-	/// function to draw the decorations.
-	virtual void Draw(int x, int y, const CUnit *unit) const;
-
-	char IsVertical;            /// if true, vertical bar, else horizontal.
-	char SEToNW;                /// (SouthEastToNorthWest), if false value 0 is on the left or up of the bar.
-	int Height;                 /// Height of the bar.
-	int Width;                  /// Width of the bar.
-	char ShowFullBackground;    /// if true, show background like value equal to max.
-	char BorderSize;            /// Size of the border, 0 for no border.
-// FIXME color depend of percent (red, Orange, Yellow, Green...)
-	Uint32 Color;               /// Color of bar.
-	Uint32 BColor;              /// Color of background.
-};
-
-class CDecoVarText : public CDecoVar
-{
-public:
-	CDecoVarText() : Font(NULL) {};
-	/// function to draw the decorations.
-	virtual void Draw(int x, int y, const CUnit *unit) const;
-
-	CFont *Font;                   /// Font to use to display value.
-// FIXME : Add Color, format
-};
-
 /// Sprite contains frame from full (left)to empty state (right).
 class CDecoVarSpriteBar : public CDecoVar
 {
@@ -702,19 +517,6 @@ public:
 // FIXME Sprite info. better way ?
 };
 
-/// use to show specific frame in a sprite.
-class CDecoVarStaticSprite : public CDecoVar
-{
-public:
-	CDecoVarStaticSprite() : NSprite(-1), n(0) {}
-	/// function to draw the decorations.
-	virtual void Draw(int x, int y, const CUnit *unit) const;
-
-// FIXME Sprite info. and Replace n with more appropriate var.
-	char NSprite;               /// Index of sprite. (@see DefineSprites and @see GetSpriteIndex)
-	int n;                      /// identifiant in SpellSprite
-};
-
 typedef enum {
 	UnitTypeLand,               /// Unit lives on land
 	UnitTypeFly,                /// Unit lives in air
@@ -807,39 +609,35 @@ public:
 };
 
 	/// Base structure of unit-type
-	/// @todo n0body: AutoBuildRate not implemented.
 class CUnitType {
 public:
 	CUnitType() : Slot(0), Width(0), Height(0), OffsetX(0), OffsetY(0), DrawLevel(0),
 		ShadowWidth(0), ShadowHeight(0), ShadowOffsetX(0), ShadowOffsetY(0),
-		Animations(NULL), StillFrame(0),
-		CorpseName(NULL), CorpseType(NULL),
+		Animations(NULL), StillFrame(0), CorpseType(NULL),
 		Construction(NULL),  RepairHP(0), TileWidth(0), TileHeight(0),
 		BoxWidth(0), BoxHeight(0), NumDirections(0), MinAttackRange(0),
 		ReactRangeComputer(0), ReactRangePerson(0), Priority(0),
 		BurnPercent(0), BurnDamageRate(0), RepairRange(0),
-		CanCastSpell(NULL), AutoCastActive(NULL), AutoBuildRate(0),
-		RandomMovementProbability(0), ClicksToExplode(0),
-		CanTransport(NULL), MaxOnBoard(0), StartingResources(0),
+		CanCastSpell(NULL), AutoCastActive(NULL),
+		CanTransport(NULL), MaxOnBoard(0),
 		UnitType(UnitTypeLand), DecayRate(0), AnnoyComputerFactor(0),
 		MouseAction(0), Points(0), CanTarget(0),
 		Flip(0), Revealer(0), LandUnit(0), AirUnit(0), SeaUnit(0),
 		ExplodeWhenKilled(0), Building(0), VisibleUnderFog(0),
 		Coward(0), AttackFromTransporter(0),
 		Vanishes(0), GroundAttack(0), ShoreBuilding(0), CanAttack(0),
-		BuilderOutside(0), BuilderLost(0), CanHarvest(0), Harvester(0),
-		BoolFlag(NULL), Variable(NULL), CanTargetFlag(NULL),
-		SelectableByRectangle(0), IsNotSelectable(0), Decoration(0),
+		BuilderOutside(0), BuilderLost(0), CanHarvestFrom(0), Harvester(0),
+		Neutral(0), SelectableByRectangle(0), IsNotSelectable(0), Decoration(0),
 		Indestructible(0), Teleporter(0),
-		GivesResource(0), Supply(0), Demand(0), FieldFlags(0), MovementMask(0),
+		BoolFlag(NULL), Variable(NULL), CanTargetFlag(NULL),
+		ProductionEfficiency(100), FieldFlags(0), MovementMask(0),
 		Sprite(NULL), ShadowSprite(NULL)
 	{
-		memset(_Costs, 0, sizeof(_Costs));
-		memset(RepairCosts, 0, sizeof(RepairCosts));
-		memset(CanStore, 0, sizeof(CanStore));
-		memset(ResInfo, 0, sizeof(ResInfo));
 		memset(&NeutralMinimapColorRGB, 0, sizeof(NeutralMinimapColorRGB));
-		memset(ImproveIncomes, 0, sizeof(ImproveIncomes));
+		memset(ProductionRate, 0, sizeof(ProductionRate));
+		memset(MaxUtilizationRate, 0, sizeof(MaxUtilizationRate));
+		memset(ProductionCosts, 0, sizeof(ProductionCosts));
+		memset(StorageCapacity, 0, sizeof(StorageCapacity));
 	}
 
 	std::string Ident;              /// Identifier
@@ -866,15 +664,12 @@ public:
 	MissileConfig Missile;          /// Missile weapon
 	MissileConfig Explosion;        /// Missile for unit explosion
 
-	char *CorpseName;               /// Corpse type name
+	std::string CorpseName;         /// Corpse type name
 	CUnitType *CorpseType;          /// Corpse unit-type
 
-	// this is taken from the UDTA section
 	CConstruction *Construction;    /// What is shown in construction phase
 
-	int _Costs[MaxCosts];           /// How many resources needed
 	int RepairHP;                   /// Amount of HP per repair
-	int RepairCosts[MaxCosts];      /// How much it costs to repair
 
 	int TileWidth;                  /// Tile size on map width
 	int TileHeight;                 /// Tile size on map height
@@ -890,12 +685,8 @@ public:
 	int RepairRange;                /// Units repair range.
 	char *CanCastSpell;             /// Unit is able to use spells.
 	char *AutoCastActive;           /// Default value for autocast.
-	int AutoBuildRate;              /// The rate at which the building builds itself
-	int RandomMovementProbability;  /// Probability to move randomly.
-	int ClicksToExplode;            /// Number of consecutive clicks until unit suicides.
 	char *CanTransport;             /// Can transport units with this flag.
 	int MaxOnBoard;                 /// Number of Transporter slots.
-	int StartingResources;          /// Amount of Resources on build
 	/// originally only visual effect, we do more with this!
 	UnitTypeType UnitType;          /// Land / fly / naval
 	int DecayRate;                  /// Decay rate in 1/6 seconds
@@ -930,11 +721,9 @@ public:
 	unsigned CanAttack : 1;         /// Unit can attack.
 	unsigned BuilderOutside : 1;    /// The builder stays outside during the build.
 	unsigned BuilderLost : 1;       /// The builder is lost after the build.
-	unsigned CanHarvest : 1;        /// Resource can be harvested.
+	unsigned CanHarvestFrom : 1;    /// Resource can be harvested.
 	unsigned Harvester : 1;         /// unit is a resource harvester.
-	unsigned char *BoolFlag;        /// User defined flag. Used for (dis)allow target.
-	CVariable *Variable;            /// Array of user defined variables.
-	unsigned char *CanTargetFlag;   /// Flag needed to target with missile.
+	unsigned Neutral : 1;           /// Unit is neutral, used by the editor
 
 	unsigned SelectableByRectangle : 1; /// Selectable with mouse rectangle.
 	unsigned IsNotSelectable : 1;       /// Unit should not be selected during game.
@@ -942,21 +731,43 @@ public:
 	unsigned Indestructible : 1;        /// Unit is indestructible (take no damage).
 	unsigned Teleporter : 1;            /// Can teleport other units.
 
-	int CanStore[MaxCosts];             /// Resources that we can store here.
-	int GivesResource;                  /// The resource this unit gives.
-	ResourceInfo *ResInfo[MaxCosts];    /// Resource information.
-	std::vector<CBuildRestriction *> BuildingRules;   /// Rules list for building a building.
+	unsigned char *BoolFlag;        /// User defined flag. Used for (dis)allow target.
+	CVariable *Variable;            /// Array of user defined variables.
+	unsigned char *CanTargetFlag;   /// Flag needed to target with missile.
+
+	std::vector<CBuildRestriction *> BuildingRules;/// Rules list for building a building.
 	SDL_Color NeutralMinimapColorRGB;   /// Minimap Color for Neutral Units.
 
-	CUnitSound Sound;               /// Sounds for events
+	CUnitSound Sound;                   /// Sounds for events
 
-	int Supply;                     /// Food supply
-	int Demand;                     /// Food demand
+	int ProductionRate[MaxCosts];       /// Rate that resources are produced
+	int MaxUtilizationRate[MaxCosts];   /// Max resource rate that can be used
+	int ProductionCosts[MaxCosts];      /// Total cost to produce this type
+	int StorageCapacity[MaxCosts];      /// Storage capacity of resources
+	unsigned ProductionEfficiency : 8;  /// Production efficiency
+
+	inline void SetEnergyProductionRate(int v) { ProductionRate[EnergyCost] = v; }
+	inline int GetEnergyProductionRate() { return ProductionRate[EnergyCost]; }
+	inline void SetMagmaProductionRate(int v) { ProductionRate[MagmaCost] = v; }
+	inline int GetMagmaProductionRate() { return ProductionRate[MagmaCost]; }
+
+	inline void SetMaxEnergyUtilizationRate(int v) { MaxUtilizationRate[EnergyCost] = v; }
+	inline int GetMaxEnergyUtilizationRate() { return MaxUtilizationRate[EnergyCost]; }
+	inline void SetMaxMagmaUtilizationRate(int v) { MaxUtilizationRate[MagmaCost] = v; }
+	inline int GetMaxMagmaUtilizationRate() { return MaxUtilizationRate[MagmaCost]; }
+
+	inline void SetEnergyValue(int v) { ProductionCosts[EnergyCost] = CYCLES_PER_SECOND * v; }
+	inline int GetEnergyValue() { return ProductionCosts[EnergyCost] / CYCLES_PER_SECOND; }
+	inline void SetMagmaValue(int v) { ProductionCosts[MagmaCost] = CYCLES_PER_SECOND * v; }
+	inline int GetMagmaValue() { return ProductionCosts[MagmaCost] / CYCLES_PER_SECOND; }
+
+	inline void SetEnergyStorageCapacity(int v) { StorageCapacity[EnergyCost] = CYCLES_PER_SECOND * v; }
+	inline int GetEnergyStorageCapacity() { return StorageCapacity[EnergyCost] / CYCLES_PER_SECOND; }
+	inline void SetMagmaStorageCapacity(int v) { StorageCapacity[MagmaCost] = CYCLES_PER_SECOND * v; }
+	inline int GetMagmaStorageCapacity() { return StorageCapacity[MagmaCost] / CYCLES_PER_SECOND; }
 
 // --- FILLED UP ---
 
-	int ImproveIncomes[MaxCosts];   /// Gives player an improved income
-
 	unsigned FieldFlags;            /// Unit map field flags
 	unsigned MovementMask;          /// Unit check this map flags for move
 
@@ -1009,7 +820,6 @@ extern void UpdateStats(int reset_to_default);       /// Update unit stats
 extern CUnitType *UnitTypeByIdent(const std::string &ident);/// Get unit-type by ident
 extern int GetVariableIndex(const char *VarName);    /// Get index of the variable
 
-extern void SaveUnitTypes(CFile *file);              /// Save the unit-type table
 extern CUnitType *NewUnitTypeSlot(const std::string &ident);/// Allocate an empty unit-type slot
 	/// Draw the sprite frame of unit-type
 extern void DrawUnitType(const CUnitType *type, CPlayerColorGraphic *sprite,
