@@ -9,7 +9,7 @@
 //
 /**@name map_fog.cpp - The map fog of war handling. */
 //
-//      (c) Copyright 1999-2006 by Lutz Sammer, Vladi Shabanski,
+//      (c) Copyright 1999-2007 by Lutz Sammer, Vladi Shabanski,
 //                                 Russell Smith, and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: map_fog.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: map_fog.cpp 8826 2007-05-27 17:27:35Z jim4 $
 
 //@{
 
@@ -97,7 +97,7 @@ unsigned short CMap::IsTileVisible(const CPlayer *player, int x, int y) const
 	unsigned short visiontype;
 	unsigned short *visible;
 
-	visible = this->Fields[y * this->Info.MapWidth + x].Visible;
+	visible = this->Field(x, y)->Visible;
 	visiontype = visible[player->Index];
 
 	if (visiontype > 1) {
@@ -173,7 +173,7 @@ void MapMarkTileSight(const CPlayer *player, int x, int y)
 
 	unsigned short *v;
 
-	v = &Map.Fields[x + y * Map.Info.MapWidth].Visible[player->Index];
+	v = &Map.Field(x, y)->Visible[player->Index];
 	if (*v == 0 || *v == 1) { // Unexplored or unseen
 		// When there is no fog only unexplored tiles are marked.
 		if (!Map.NoFogOfWar || *v == 0) {
@@ -203,7 +203,7 @@ void MapUnmarkTileSight(const CPlayer *player, int x, int y)
 
 	unsigned short *v;
 
-	v = &Map.Fields[x + y * Map.Info.MapWidth].Visible[player->Index];
+	v = &Map.Field(x, y)->Visible[player->Index];
 	switch (*v) {
 		case 0:  // Unexplored
 		case 1:
@@ -616,72 +616,75 @@ void CMap::InitFogOfWar(void)
 	FogGraphic->Load();
 
 #ifndef USE_OPENGL
-	//
-	// Generate Only Fog surface.
-	//
-	s = SDL_CreateRGBSurface(SDL_SWSURFACE, TileSizeX, TileSizeY,
-		32, RMASK, GMASK, BMASK, AMASK);
+	if (!AlphaFogG) {
+		//
+		// Generate Only Fog surface.
+		//
+		s = SDL_CreateRGBSurface(SDL_SWSURFACE, TileSizeX, TileSizeY,
+			32, RMASK, GMASK, BMASK, AMASK);
 
-	// FIXME: Make the color configurable
-	SDL_GetRGB(ColorBlack, TheScreen->format, &r, &g, &b);
-	color = Video.MapRGB(s->format, r, g, b);
+		// FIXME: Make the color configurable
+		SDL_GetRGB(ColorBlack, TheScreen->format, &r, &g, &b);
+		color = Video.MapRGB(s->format, r, g, b);
 
-	SDL_FillRect(s, NULL, color);
-	OnlyFogSurface = SDL_DisplayFormat(s);
-	SDL_SetAlpha(OnlyFogSurface, SDL_SRCALPHA | SDL_RLEACCEL, FogOfWarOpacity);
-	SDL_FreeSurface(s);
+		SDL_FillRect(s, NULL, color);
+		OnlyFogSurface = SDL_DisplayFormat(s);
+		SDL_SetAlpha(OnlyFogSurface, SDL_SRCALPHA | SDL_RLEACCEL, FogOfWarOpacity);
+		SDL_FreeSurface(s);
 
-	//
-	// Generate Alpha Fog surface.
-	//
-	if (FogGraphic->Surface->format->BytesPerPixel == 1) {
-		s = SDL_DisplayFormat(FogGraphic->Surface);
-		SDL_SetAlpha(s, SDL_SRCALPHA | SDL_RLEACCEL, FogOfWarOpacity);
-	} else {
-		int i;
-		int j;
-		Uint32 c;
-		Uint8 a;
-		SDL_PixelFormat *f;
-
-		// Copy the top row to a new surface
-		f = FogGraphic->Surface->format;
-		s = SDL_CreateRGBSurface(SDL_SWSURFACE, FogGraphic->Surface->w, TileSizeY,
-			f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask);
-		SDL_LockSurface(s);
-		SDL_LockSurface(FogGraphic->Surface);
-		for (i = 0; i < s->h; ++i) {
-			memcpy((Uint8 *)s->pixels + i * s->pitch,
-				(Uint8 *)FogGraphic->Surface->pixels + i * FogGraphic->Surface->pitch,
-				FogGraphic->Surface->w * f->BytesPerPixel);
-		}
-		SDL_UnlockSurface(s);
-		SDL_UnlockSurface(FogGraphic->Surface);
-
-		// Convert any non-transparent pixels to use FogOfWarOpacity as alpha
-		SDL_LockSurface(s);
-		for (j = 0; j < s->h; ++j) {
-			for (i = 0; i < s->w; ++i) {
-				c = *(Uint32 *)&((Uint8*)s->pixels)[i * 4 + j * s->pitch];
-				Video.GetRGBA(c, s->format, &r, &g, &b, &a);
-				if (a) {
-					c = Video.MapRGBA(s->format, r, g, b, FogOfWarOpacity);
-					*(Uint32 *)&((Uint8*)s->pixels)[i * 4 + j * s->pitch] = c;
-				}
+		//
+		// Generate Alpha Fog surface.
+		//
+		if (FogGraphic->Surface->format->BytesPerPixel == 1) {
+			s = SDL_DisplayFormat(FogGraphic->Surface);
+			SDL_SetAlpha(s, SDL_SRCALPHA | SDL_RLEACCEL, FogOfWarOpacity);
+		} else {
+			int i;
+			int j;
+			Uint32 c;
+			Uint8 a;
+			SDL_PixelFormat *f;
 
+			// Copy the top row to a new surface
+			f = FogGraphic->Surface->format;
+			s = SDL_CreateRGBSurface(SDL_SWSURFACE, FogGraphic->Surface->w, TileSizeY,
+				f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask);
+			SDL_LockSurface(s);
+			SDL_LockSurface(FogGraphic->Surface);
+			for (i = 0; i < s->h; ++i) {
+				memcpy((Uint8 *)s->pixels + i * s->pitch,
+					(Uint8 *)FogGraphic->Surface->pixels + i * FogGraphic->Surface->pitch,
+					FogGraphic->Surface->w * f->BytesPerPixel);
 			}
+			SDL_UnlockSurface(s);
+			SDL_UnlockSurface(FogGraphic->Surface);
+
+			// Convert any non-transparent pixels to use FogOfWarOpacity as alpha
+			SDL_LockSurface(s);
+			for (j = 0; j < s->h; ++j) {
+				for (i = 0; i < s->w; ++i) {
+					c = *(Uint32 *)&((Uint8*)s->pixels)[i * 4 + j * s->pitch];
+					Video.GetRGBA(c, s->format, &r, &g, &b, &a);
+					if (a) {
+						c = Video.MapRGBA(s->format, r, g, b, FogOfWarOpacity);
+						*(Uint32 *)&((Uint8*)s->pixels)[i * 4 + j * s->pitch] = c;
+					}
+
+				}
+			}
+			SDL_UnlockSurface(s);
 		}
-		SDL_UnlockSurface(s);
+		AlphaFogG = CGraphic::New("");
+		AlphaFogG->Surface = s;
+		AlphaFogG->Width = TileSizeX;
+		AlphaFogG->Height = TileSizeY;
+		AlphaFogG->GraphicWidth = s->w;
+		AlphaFogG->GraphicHeight = s->h;
+		AlphaFogG->NumFrames = 1;
 	}
-	AlphaFogG = CGraphic::New("");
-	AlphaFogG->Surface = s;
-	AlphaFogG->Width = TileSizeX;
-	AlphaFogG->Height = TileSizeY;
-	AlphaFogG->GraphicWidth = s->w;
-	AlphaFogG->GraphicHeight = s->h;
-	AlphaFogG->NumFrames = 1;
 #endif
 
+	delete[] VisibleTable;
 	VisibleTable = new unsigned short[Info.MapWidth * Info.MapHeight];
 }
 
