@@ -1,15 +1,15 @@
-//     ____                _       __               
+//     ____                _       __
 //    / __ )____  _____   | |     / /___ ___________
 //   / __  / __ \/ ___/   | | /| / / __ `/ ___/ ___/
-//  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  ) 
-// /_____/\____/____/     |__/|__/\__,_/_/  /____/  
-//                                              
+//  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  )
+// /_____/\____/____/     |__/|__/\__,_/_/  /____/
+//
 //       A futuristic real-time strategy game.
 //          This file is part of Bos Wars.
 //
 /**@name minimap.cpp - The minimap. */
 //
-//      (c) Copyright 1998-2006 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: minimap.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: minimap.cpp 8949 2007-06-29 11:25:27Z Jarod42 $
 
 //@{
 
@@ -136,8 +136,8 @@ void CMinimap::Create(void)
 	MinimapScaleX = (W * MINIMAP_FAC + n - 1) / n;
 	MinimapScaleY = (H * MINIMAP_FAC + n - 1) / n;
 
-	XOffset = (W - (Map.Info.MapWidth * MinimapScaleX) / MINIMAP_FAC) / 2;
-	YOffset = (H - (Map.Info.MapHeight * MinimapScaleY) / MINIMAP_FAC) / 2;
+	XOffset = (W - (Map.Info.MapWidth * MinimapScaleX) / MINIMAP_FAC + 1) / 2;
+	YOffset = (H - (Map.Info.MapHeight * MinimapScaleY) / MINIMAP_FAC + 1) / 2;
 
 	DebugPrint("MinimapScale %d %d (%d %d), X off %d, Y off %d\n" _C_
 		MinimapScaleX / MINIMAP_FAC _C_ MinimapScaleY / MINIMAP_FAC _C_
@@ -172,7 +172,7 @@ void CMinimap::Create(void)
 	MinimapTerrainSurface = SDL_CreateRGBSurface(SDL_SWSURFACE,
 		W, H, f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask);
 	MinimapSurface = SDL_CreateRGBSurface(SDL_SWSURFACE,
-		W, H, 32, TheScreen->format->Rmask, TheScreen->format->Gmask, 
+		W, H, 32, TheScreen->format->Rmask, TheScreen->format->Gmask,
 		TheScreen->format->Bmask, 0);
 #else
 	for (MinimapTextureWidth = 1; MinimapTextureWidth < W; MinimapTextureWidth <<= 1) {
@@ -438,7 +438,7 @@ void CMinimap::UpdateXY(int tx, int ty)
 }
 
 /**
-**  Draw an unit on the minimap.
+**  Draw a unit on the minimap.
 */
 static void DrawUnitOn(CUnit *unit, int red_phase)
 {
@@ -458,7 +458,7 @@ static void DrawUnitOn(CUnit *unit, int red_phase)
 		type = unit->Type;
 	} else {
 		type = unit->Seen.Type;
-		// This will happen for radar if the unit has not been seen and we 
+		// This will happen for radar if the unit has not been seen and we
 		// have it on radar.
 		if (!type) {
 			type = unit->Type;
@@ -505,7 +505,7 @@ static void DrawUnitOn(CUnit *unit, int red_phase)
 				*(Uint16 *)&((Uint8*)MinimapSurface->pixels)[(mx + w) * bpp + (my + h) * MinimapSurface->pitch] =
 					color;
 			} else {
-				*(Uint32 *)&((Uint8*)MinimapSurface->pixels)[(mx + w) * bpp + (my + h) * MinimapSurface->pitch] = 
+				*(Uint32 *)&((Uint8*)MinimapSurface->pixels)[(mx + w) * bpp + (my + h) * MinimapSurface->pitch] =
 					color;
 			}
 #else
@@ -558,7 +558,7 @@ void CMinimap::Update(void)
 		memcpy(MinimapSurface, MinimapTerrainSurface, MinimapTextureWidth * MinimapTextureHeight * 4);
 #endif
 	}
-	
+
 #ifndef USE_OPENGL
 	SDL_LockSurface(MinimapSurface);
 	SDL_LockSurface(MinimapTerrainSurface);
@@ -570,7 +570,7 @@ void CMinimap::Update(void)
 			} else {
 				visiontype = Map.IsTileVisible(ThisPlayer, Minimap2MapX[mx], Minimap2MapY[my] / Map.Info.MapWidth);
 			}
-			
+
 			if ( visiontype == 0 || (visiontype == 1 && ((mx & 1) != (my & 1))))  {
 #ifndef USE_OPENGL
 				if (bpp == 2) {
@@ -585,8 +585,8 @@ void CMinimap::Update(void)
 					Video.MapRGB(0, 0, 0, 0);
 #endif
 			}
-			
-			
+
+
 		}
 	}
 
@@ -633,7 +633,7 @@ static void DrawEvents(void)
 void CMinimap::Draw(int vx, int vy)
 {
 #ifndef USE_OPENGL
-	SDL_Rect drect = {X, Y};
+	SDL_Rect drect = {X, Y, 0, 0};
 	SDL_BlitSurface(MinimapSurface, NULL, TheScreen, &drect);
 #else
 	glBindTexture(GL_TEXTURE_2D, MinimapTexture);
