@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: network.cpp 8553 2007-02-10 13:11:22Z feb $
+//      $Id: network.cpp 8954 2007-06-30 02:11:24Z jim4 $
 
 //@{
 
@@ -211,12 +211,13 @@
 //  Includes
 //----------------------------------------------------------------------------
 
+#include "stratagus.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
 #include <string.h>
-
-#include "stratagus.h"
+#include <list>
 
 #include "net_lowlevel.h"
 #include "unit.h"
@@ -229,6 +230,7 @@
 #include "commands.h"
 #include "interface.h"
 #include "results.h"
+#include "sound.h"
 
 
 //----------------------------------------------------------------------------
@@ -436,8 +438,8 @@ int CNetworkPacket::Deserialize(unsigned char *p, unsigned int len)
 /**
 **  Send message to all clients.
 **
-**  @param buf  Buffer of outgoing message.
-**  @param len  Buffer length.
+**  @param packet       Packet to send.
+**  @param numcommands  Number of commands.
 */
 static void NetworkBroadcast(const CNetworkPacket *packet, int numcommands)
 {
@@ -1041,12 +1043,6 @@ void NetworkEvent(void)
 				allowed = true;
 				break;
 			case MessageCommandDismiss:
-				// Allow to explode critters.
-				if ((UnitSlots[ntohs(nc->Unit)]->Player->Index == PlayerNumNeutral) &&
-					UnitSlots[ntohs(nc->Unit)]->Type->ClicksToExplode) {
-					allowed = true;
-					break;
-				}
 				// Fall through!
 			default:
 				if (UnitSlots[ntohs(nc->Unit)]->Player->Index == player ||
@@ -1063,6 +1059,9 @@ void NetworkEvent(void)
 			NetworkIn[packet.Header.Cycle][player][i].Data = *nc;
 		} else {
 			SetMessage(_("%s sent bad command"), Players[player].Name.c_str());
+			DebugPrint("%s sent bad command: 0x%x\n" _C_ 
+				Players[player].Name.c_str() _C_ 
+				packet.Header.Type[i] & 0x7F);
 		}
 	}
 
@@ -1176,6 +1175,7 @@ static void ParseNetworkCommand(const CNetworkCommandQueue *ncq)
 			if (ncq->Type == MessageChatTerm) {
 				NetMsgBuf[ply][127] = '\0';
 				SetMessage("%s", NetMsgBuf[ply]);
+				PlayGameSound(GameSounds.ChatMessage.Sound, MaxSampleVolume);
 				NetMsgBufLen[ply] = 0;
 			}
 			}
