@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: missile.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: missile.cpp 8839 2007-06-02 17:05:34Z jim4 $
 
 //@{
 
@@ -295,25 +295,18 @@ static void FreeMissile(std::vector<Missile *> &missiles, std::vector<Missile*>:
 **
 **  @param attacker_stats  Attacker attributes.
 **  @param goal_stats      Goal attributes.
-**  @param bloodlust       If attacker has bloodlust
-**  @param xp              Experience of attacker.
 **
 **  @return                damage inflicted to goal.
 */
 static int CalculateDamageStats(const CUnitStats *attacker_stats,
-	const CUnitStats *goal_stats, int bloodlust, int xp)
+	const CUnitStats *goal_stats)
 {
 	int damage;
 	int basic_damage;
 	int piercing_damage;
 
-	basic_damage = attacker_stats->Variables[BASICDAMAGE_INDEX].Value +
-		isqrt(xp / 100) * XpDamage;
+	basic_damage = attacker_stats->Variables[BASICDAMAGE_INDEX].Value;
 	piercing_damage = attacker_stats->Variables[PIERCINGDAMAGE_INDEX].Value;
-	if (bloodlust) {
-		basic_damage *= 2;
-		piercing_damage *= 2;
-	}
 
 	damage = (basic_damage - goal_stats->Variables[ARMOR_INDEX].Value) > 1 ?
 		(basic_damage - goal_stats->Variables[ARMOR_INDEX].Value) : 1;
@@ -340,8 +333,7 @@ static int CalculateDamage(const CUnit *attacker, const CUnit *goal)
 	Assert(goal);
 
 	if (!Damage) { // Use old method.
-		return CalculateDamageStats(attacker->Stats, goal->Stats,
-			attacker->Variable[XP_INDEX].Value, attacker->Variable[BLOODLUST_INDEX].Value);
+		return CalculateDamageStats(attacker->Stats, goal->Stats);
 	}
 	Assert(Damage);
 
@@ -1149,7 +1141,7 @@ void SaveMissiles(CFile *file)
 	std::vector<Missile *>::const_iterator i;
 
 	file->printf("\n--- -----------------------------------------\n");
-	file->printf("--- MODULE: missiles $Id: missile.cpp 8550 2007-02-10 12:33:10Z feb $\n\n");
+	file->printf("--- MODULE: missiles $Id: missile.cpp 8839 2007-06-02 17:05:34Z jim4 $\n\n");
 
 	for (i = GlobalMissiles.begin(); i != GlobalMissiles.end(); ++i) {
 		(*i)->SaveMissile(file);
@@ -1238,6 +1230,17 @@ void CleanMissiles(void)
 	LocalMissiles.clear();
 }
 
+#ifdef DEBUG
+void FreeBurningBuildingFrames()
+{
+	for (std::vector<BurningBuildingFrame *>::iterator i = BurningBuildingFrames.begin();
+		i != BurningBuildingFrames.end(); ++i) {
+		delete *i;
+	}
+	BurningBuildingFrames.clear();
+}
+#endif
+
 /*----------------------------------------------------------------------------
 --    Functions (Spells Controllers/Callbacks) TODO: move to another file?
 ----------------------------------------------------------------------------*/
