@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: player.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: player.cpp 8902 2007-06-20 05:51:48Z jim4 $
 
 //@{
 
@@ -37,6 +37,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <algorithm>
 
 #include "stratagus.h"
 #include "video.h"
@@ -105,7 +106,7 @@ void InitPlayers(void)
 void CleanPlayers(void)
 {
 	ThisPlayer = NULL;
-	for (int i = 0; i < PlayerMax; i++) {
+	for (int i = 0; i < PlayerMax; ++i) {
 		Players[i].Clear();
 	}
 	NumPlayers = 0;
@@ -113,6 +114,251 @@ void CleanPlayers(void)
 	NoRescueCheck = 0;
 }
 
+#ifdef DEBUG
+void FreePlayerColors()
+{
+	for (int i = 0; i < PlayerMax; ++i) {
+		delete[] Players[i].UnitColors.Colors;
+		delete[] PlayerColorsRGB[i];
+		PlayerColorsRGB[i] = NULL;
+		delete[] PlayerColors[i];
+		PlayerColors[i] = NULL;
+	}
+}
+#endif
+
+/**
+**  Add to UnitsConsumingResources
+*/
+void CPlayer::AddToUnitsConsumingResources(CUnit *unit, int costs[MaxCosts])
+{
+	Assert(UnitsConsumingResourcesActual[unit] == NULL);
+	Assert(UnitsConsumingResourcesRequested[unit] == NULL);
+
+	int *c;
+	
+	c = new int[MaxCosts];
+	memset(c, 0, MaxCosts * sizeof(int));
+	UnitsConsumingResourcesActual[unit] = c;
+	// no need to change ActualUtilizationRate here
+
+	c = new int[MaxCosts];
+	memcpy(c, costs, MaxCosts * sizeof(int));
+	UnitsConsumingResourcesRequested[unit] = c;
+	for (int i = 0; i < MaxCosts; ++i) {
+		RequestedUtilizationRate[i] += costs[i];
+	}
+}
+
+/**
+**  Remove from UnitsConsumingResources
+*/
+void CPlayer::RemoveFromUnitsConsumingResources(CUnit *unit)
+{
+	int *cactual = UnitsConsumingResourcesActual[unit];
+	int *crequested = UnitsConsumingResourcesRequested[unit];
+
+	for (int i = 0; i < MaxCosts; ++i) {
+		ActualUtilizationRate[i] -= cactual[i];
+		RequestedUtilizationRate[i] -= crequested[i];
+	}
+
+	delete[] cactual;
+	UnitsConsumingResourcesActual.erase(unit);
+	delete[] crequested;
+	UnitsConsumingResourcesRequested.erase(unit);
+}
+
+/**
+**  Update costs for unit in UnitsConsumingResources
+*/
+void CPlayer::UpdateUnitsConsumingResources(CUnit *unit, int costs[MaxCosts])
+{
+	int *c = UnitsConsumingResourcesActual[unit];
+
+	for (int i = 0; i < MaxCosts; ++i) {
+		ActualUtilizationRate[i] -= c[i];
+		c[i] = costs[i];
+		ActualUtilizationRate[i] += c[i];
+	}
+}
+
+/**
+**  Calculate how many resources the unit needs to request
+**
+**  @param utype   builder unit type
+**  @param bcosts  costs of unit type being built or harvested
+**  @param costs   returns the resource amount
+*/
+void CalculateRequestedAmount(CUnitType *utype, int bcosts[MaxCosts], int costs[MaxCosts])
+{
+	if (bcosts[EnergyCost] == 0) {
+		costs[EnergyCost] = 0;
+		costs[MagmaCost] = utype->MaxUtilizationRate[MagmaCost];
+	} else if (bcosts[MagmaCost] == 0) {
+		costs[EnergyCost] = utype->MaxUtilizationRate[EnergyCost];
+		costs[MagmaCost] = 0;
+	} else {
+		int f = 100 * bcosts[EnergyCost] * utype->MaxUtilizationRate[MagmaCost] /
+			(bcosts[MagmaCost] * utype->MaxUtilizationRate[EnergyCost]);
+		if (f > 100) {
+			costs[EnergyCost] = utype->MaxUtilizationRate[EnergyCost];
+			costs[MagmaCost] = (utype->MaxUtilizationRate[MagmaCost] * 100 + f / 2) / f;
+		} else if (f < 100) {
+			costs[EnergyCost] = (utype->MaxUtilizationRate[EnergyCost] * f + 50) / 100;
+			costs[MagmaCost] = utype->MaxUtilizationRate[MagmaCost];
+		} else {
+			costs[EnergyCost] = utype->MaxUtilizationRate[EnergyCost];
+			costs[MagmaCost] = utype->MaxUtilizationRate[MagmaCost];
+		}
+	}
+
+	for (int i = 0; i < MaxCosts; ++i) {
+		if (costs[i] > bcosts[i]) {
+			costs[i] = bcosts[i];
+		}
+	}
+}
+
+/** 
+**  Go through the list of units owned by the player and rebuild
+**  the UnitsConsumingResources list.
+*/
+void CPlayer::RebuildUnitsConsumingResourcesList()
+{
+	for (int i = 0; i < TotalNumUnits; ++i) {
+		int costs[MaxCosts];
+		CUnit *unit = Units[i];
+
+		if (unit->Orders[0]->Action == UnitActionTrain && unit->SubAction != 0) {
+			CalculateRequestedAmount(unit->Type, unit->Orders[0]->Type->ProductionCosts, costs);
+			AddToUnitsConsumingResources(unit, costs);
+		} else if (unit->Orders[0]->Action == UnitActionBuilt && !unit->Type->BuilderOutside) {
+			CalculateRequestedAmount(unit->Type, unit->Type->ProductionCosts, costs);
+			unit->Player->AddToUnitsConsumingResources(unit, costs);
+		} else if (unit->Orders[0]->Action == UnitActionRepair && unit->SubAction == 20) {
+			CalculateRequestedAmount(unit->Type, unit->Orders[0]->Goal->Type->ProductionCosts, costs);
+			AddToUnitsConsumingResources(unit, costs);
+		}
+	}
+}
+
+/**
+**  Clear all resource state variables.
+*/
+void CPlayer::ClearResourceVariables()
+{
+	std::map<CUnit *, int *>::iterator i;
+	for (i = UnitsConsumingResourcesActual.begin();
+			i != UnitsConsumingResourcesActual.end(); ++i) {
+		delete[] (*i).second;
+	}
+	for (i = UnitsConsumingResourcesRequested.begin();
+			i != UnitsConsumingResourcesRequested.end(); ++i) {
+		delete[] (*i).second;
+	}
+
+	UnitsConsumingResourcesActual.clear();
+	UnitsConsumingResourcesRequested.clear();
+	memset(ProductionRate, 0, sizeof(ProductionRate));
+	memset(ActualUtilizationRate, 0, sizeof(ActualUtilizationRate));
+	memset(RequestedUtilizationRate, 0, sizeof(RequestedUtilizationRate));
+	memset(StoredResources, 0, sizeof(StoredResources));
+	memset(StorageCapacity, 0, sizeof(StorageCapacity));
+
+}
+
+// p(type) = total production(type) + min(storage(type), storagerate(type))
+static int AvailableResourcesRate(int type, CPlayer *p)
+{
+	// FIXME: change 50 to rate we can get resources out of storage
+	int RateFromStorage = 50;
+	return p->ProductionRate[type] + std::min<int>(p->StoredResources[type], RateFromStorage);
+}
+
+// f(type) = min(p(type)/ total requested(type), 1.0)
+static int SpecificEfficiency(int type, CPlayer *p)
+{
+	if (p->RequestedUtilizationRate[type] == 0) {
+		return 100;
+	}
+	return std::min<int>((100 * AvailableResourcesRate(type, p) +
+		p->RequestedUtilizationRate[type] / 2) / p->RequestedUtilizationRate[type], 100);
+}
+
+// base efficiency = min(f(energy), f(magma))
+static int BaseEfficiency(CPlayer *p)
+{
+	return std::min<int>(SpecificEfficiency(EnergyCost, p), SpecificEfficiency(MagmaCost, p));
+}
+
+// total p(type) - total requested(type) * base efficiency
+static int ExtraProduction(int type, CPlayer *p, int be)
+{
+	return 100 * AvailableResourcesRate(type, p) - p->RequestedUtilizationRate[type] * be;
+}
+
+// sum of needs(type) of all units who only require this resource)
+static int UniqueNeeds(int type, CPlayer *p)
+{
+	// FIXME: this doesn't have to be recalculated every time
+	std::map<CUnit *, int *>::iterator i = p->UnitsConsumingResourcesRequested.begin();
+	std::map<CUnit *, int *>::iterator end = p->UnitsConsumingResourcesRequested.end();
+	int type2 = (type == EnergyCost ? MagmaCost : EnergyCost);
+	int needs = 0;
+
+	for (; i != end; ++i) {
+		if ((*i).second[type] != 0 && (*i).second[type2] == 0) {
+			needs += (*i).second[type];
+		}
+	}
+	return needs;
+}
+
+// min(extra production(type) / unique needs(type) + base efficiency, 1)
+static int UniqueEfficiency(int type, CPlayer *p)
+{
+	int be = BaseEfficiency(p);
+	return std::min<int>(ExtraProduction(type, p, be) / UniqueNeeds(type, p) + be, 100);
+}
+
+static int MaxRate(CUnit *unit, int res)
+{
+	int *costs = unit->Player->UnitsConsumingResourcesRequested[unit];
+	return costs[res];
+}
+
+static void CalculateCosts(CUnit *unit, int costs[MaxCosts])
+{
+	int i;
+	int usedtypes = 0;
+	int type = -1;
+	int *c = unit->Player->UnitsConsumingResourcesRequested[unit];
+
+	for (i = 0; i < MaxCosts; ++i) {
+		if (c[i] != 0) {
+			++usedtypes;
+			type = i;
+		}
+	}
+	Assert(usedtypes > 0);
+
+	// if unit requires both types:
+	if (usedtypes > 1) {
+		// unit effective rate(type) = unit max rate(type) * base efficiency
+		int be = BaseEfficiency(unit->Player);
+		for (i = 0; i < MaxCosts; ++i) {
+			costs[i] = MaxRate(unit, i) * be / 100;
+		}
+	} else {
+		// unit effective rate(type) = unit max rate(type) * unique efficiency(type)
+		int ue = UniqueEfficiency(type, unit->Player);
+		for (i = 0; i < MaxCosts; ++i) {
+			costs[i] = MaxRate(unit, i) * ue / 100;
+		}
+	}
+}
+
 /**
 **  Save state of players to file.
 **
@@ -124,100 +370,77 @@ void SavePlayers(CFile *file)
 {
 	int j;
 	Uint8 r, g, b;
+	CPlayer *p;
 
 	file->printf("\n--------------------------------------------\n");
-	file->printf("--- MODULE: players $Id: player.cpp 8550 2007-02-10 12:33:10Z feb $\n\n");
+	file->printf("--- MODULE: players $Id: player.cpp 8902 2007-06-20 05:51:48Z jim4 $\n\n");
 
 	//
 	//  Dump all players
 	//
 	for (int i = 0; i < NumPlayers; ++i) {
+		p = &Players[i];
 		file->printf("Player(%d,\n", i);
-		file->printf("  \"name\", \"%s\",\n", Players[i].Name.c_str());
+		file->printf("  \"name\", \"%s\",\n", p->Name.c_str());
 		file->printf("  \"type\", ");
-		switch (Players[i].Type) {
+		switch (p->Type) {
 			case PlayerNeutral:       file->printf("\"neutral\",");         break;
 			case PlayerNobody:        file->printf("\"nobody\",");          break;
 			case PlayerComputer:      file->printf("\"computer\",");        break;
 			case PlayerPerson:        file->printf("\"person\",");          break;
 			case PlayerRescuePassive: file->printf("\"rescue-passive\",");break;
 			case PlayerRescueActive:  file->printf("\"rescue-active\","); break;
-			default:                  file->printf("%d,",Players[i].Type);break;
+			default:                  file->printf("%d,",p->Type);break;
 		}
-		file->printf(" \"ai-name\", \"%s\",\n", Players[i].AiName.c_str());
-		file->printf("  \"team\", %d,", Players[i].Team);
+		file->printf(" \"ai-name\", \"%s\",\n", p->AiName.c_str());
+		file->printf("  \"team\", %d,", p->Team);
 
 		file->printf(" \"enemy\", \"");
 		for (j = 0; j < PlayerMax; ++j) {
-			file->printf("%c",(Players[i].Enemy & (1 << j)) ? 'X' : '_');
+			file->printf("%c",(p->Enemy & (1 << j)) ? 'X' : '_');
 		}
 		file->printf("\", \"allied\", \"");
 		for (j = 0; j < PlayerMax; ++j) {
-			file->printf("%c", (Players[i].Allied & (1 << j)) ? 'X' : '_');
+			file->printf("%c", (p->Allied & (1 << j)) ? 'X' : '_');
 		}
 		file->printf("\", \"shared-vision\", \"");
 		for (j = 0; j < PlayerMax; ++j) {
-			file->printf("%c", (Players[i].SharedVision & (1 << j)) ? 'X' : '_');
+			file->printf("%c", (p->SharedVision & (1 << j)) ? 'X' : '_');
 		}
-		file->printf("\",\n  \"start\", {%d, %d},\n", Players[i].StartX,
-			Players[i].StartY);
+		file->printf("\",\n  \"start\", {%d, %d},\n", p->StartX,
+			p->StartY);
 
-		// Resources
-		file->printf("  \"resources\", {");
+		// ProductionRate
+		file->printf("  \"production-rate\", {");
 		for (j = 0; j < MaxCosts; ++j) {
 			if (j) {
-				if (j == MaxCosts / 2) {
-					file->printf("\n ");
-				} else {
-					file->printf(" ");
-				}
+				file->printf(" ");
 			}
-			file->printf("\"%s\", %d,", DefaultResourceNames[j],
-				Players[i].Resources[j]);
+			file->printf("\"%s\", %d,", DefaultResourceNames[j].c_str(),
+				p->ProductionRate[j]);
 		}
-		// Last Resources
-		file->printf("},\n  \"last-resources\", {");
+		// StoredResources
+		file->printf("},\n  \"stored-resources\", {");
 		for (j = 0; j < MaxCosts; ++j) {
 			if (j) {
-				if (j == MaxCosts / 2) {
-					file->printf("\n ");
-				} else {
-					file->printf(" ");
-				}
+				file->printf(" ");
 			}
-			file->printf("\"%s\", %d,", DefaultResourceNames[j],
-				Players[i].LastResources[j]);
+			file->printf("\"%s\", %d,", DefaultResourceNames[j].c_str(),
+				p->StoredResources[j]);
 		}
-		// Incomes
-		file->printf("},\n  \"incomes\", {");
+		// StorageCapacity
+		file->printf("},\n  \"storage-capacity\", {");
 		for (j = 0; j < MaxCosts; ++j) {
 			if (j) {
-				if (j == MaxCosts / 2) {
-					file->printf("\n ");
-				} else {
-					file->printf(" ");
-				}
+				file->printf(" ");
 			}
-			file->printf("\"%s\", %d,", DefaultResourceNames[j],
-				Players[i].Incomes[j]);
-		}
-		// Revenue
-		file->printf("},\n  \"revenue\", {");
-		for (j = 0; j < MaxCosts; ++j) {
-			if (j) {
-				if (j == MaxCosts / 2) {
-					file->printf("\n ");
-				} else {
-					file->printf(" ");
-				}
-			}
-			file->printf("\"%s\", %d,", DefaultResourceNames[j],
-				Players[i].Revenue[j]);
+			file->printf("\"%s\", %d,", DefaultResourceNames[j].c_str(),
+				p->StorageCapacity[j]);
 		}
 
 		// UnitTypesCount done by load units.
 
-		file->printf("},\n  \"%s\",\n", Players[i].AiEnabled ?
+		file->printf("},\n  \"%s\",\n", p->AiEnabled ?
 			"ai-enabled" : "ai-disabled");
 
 		// Ai done by load ais.
@@ -225,40 +448,31 @@ void SavePlayers(CFile *file)
 		// TotalNumUnits done by load units.
 		// NumBuildings done by load units.
 
-		file->printf(" \"supply\", %d,", Players[i].Supply);
-		file->printf(" \"unit-limit\", %d,", Players[i].UnitLimit);
-		file->printf(" \"building-limit\", %d,", Players[i].BuildingLimit);
-		file->printf(" \"total-unit-limit\", %d,", Players[i].TotalUnitLimit);
+		file->printf(" \"unit-limit\", %d,", p->UnitLimit);
+		file->printf(" \"building-limit\", %d,", p->BuildingLimit);
+		file->printf(" \"total-unit-limit\", %d,", p->TotalUnitLimit);
 
-		file->printf("\n  \"score\", %d,", Players[i].Score);
-		file->printf("\n  \"total-units\", %d,", Players[i].TotalUnits);
-		file->printf("\n  \"total-buildings\", %d,", Players[i].TotalBuildings);
+		file->printf("\n  \"score\", %d,", p->Score);
+		file->printf("\n  \"total-units\", %d,", p->TotalUnits);
+		file->printf("\n  \"total-buildings\", %d,", p->TotalBuildings);
 		file->printf("\n  \"total-resources\", {");
 		for (j = 0; j < MaxCosts; ++j) {
 			if (j) {
 				file->printf(" ");
 			}
-			file->printf("%d,", Players[i].TotalResources[j]);
+			file->printf("%d,", p->TotalResources[j]);
 		}
 		file->printf("},");
-		file->printf("\n  \"total-razings\", %d,", Players[i].TotalRazings);
-		file->printf("\n  \"total-kills\", %d,", Players[i].TotalKills);
+		file->printf("\n  \"total-razings\", %d,", p->TotalRazings);
+		file->printf("\n  \"total-kills\", %d,", p->TotalKills);
 
-		SDL_GetRGB(Players[i].Color, TheScreen->format, &r, &g, &b);
-		file->printf("\n  \"color\", { %d, %d, %d },", r, g, b);
+		SDL_GetRGB(p->Color, TheScreen->format, &r, &g, &b);
+		file->printf("\n  \"color\", { %d, %d, %d }", r, g, b); // no comma after last parameter
 
 		// UnitColors done by init code.
-
 		// Allow saved by allow.
 
-		file->printf("\n  \"timers\", {");
-		for (j = 0; j < UpgradeMax; ++j) {
-			if (j) {
-				file->printf(" ");
-			}
-			file->printf("%d,", Players[i].UpgradeTimers.Upgrades[j]);
-		}
-		file->printf("})\n\n");
+		file->printf(")\n\n");
 	}
 
 	DebugPrint("FIXME: must save unit-stats?\n");
@@ -266,7 +480,7 @@ void SavePlayers(CFile *file)
 	//
 	//  Dump local variables
 	//
-	file->printf("SetThisPlayer(%d)\n\n", ThisPlayer->Index);
+	file->printf("ThisPlayer = Players[%d]\n\n", ThisPlayer->Index);
 }
 
 /**
@@ -403,17 +617,8 @@ void CreatePlayer(int type)
 		}
 	}
 
-	//
-	//  Initial default incomes.
-	//
-	for (i = 0; i < MaxCosts; ++i) {
-		player->Incomes[i] = DefaultIncomes[i];
-	}
-
 	memset(player->UnitTypesCount, 0, sizeof(player->UnitTypesCount));
 
-	player->Supply = 0;
-	player->Demand = 0;
 	player->NumBuildings = 0;
 	player->TotalNumUnits = 0;
 	player->Score = 0;
@@ -458,18 +663,12 @@ void CPlayer::Clear()
 	SharedVision = 0;
 	StartX = 0;
 	StartY = 0;
-	memset(Resources, 0, sizeof(Resources));
-	memset(LastResources, 0, sizeof(LastResources));
-	memset(Incomes, 0, sizeof(Incomes));
-	memset(Revenue, 0, sizeof(Revenue));
 	memset(UnitTypesCount, 0, sizeof(UnitTypesCount));
 	AiEnabled = 0;
 	Ai = 0;
 	memset(Units, 0, sizeof(Units));
 	TotalNumUnits = 0;
 	NumBuildings = 0;
-	Supply = 0;
-	Demand = 0;
 	// FIXME: can't clear limits since it's initialized already
 //	UnitLimit = 0;
 //	BuildingLimit = 0;
@@ -481,29 +680,13 @@ void CPlayer::Clear()
 	TotalRazings = 0;
 	TotalKills = 0;
 	Color = 0;
-	// Allow has already been initialised by DefineAllow and DefineUnitAllow.
-	// Thus we cannot simply clear it.
-	// But we have to revert upgrades that could have happened during a
-	// the previous game.
-	Allow.RevertUpgrades();
-	UpgradeTimers.Clear();
+	ClearResourceVariables();
 }
 
 /*----------------------------------------------------------------------------
 --  Resource management
 ----------------------------------------------------------------------------*/
 
-/**
-**  Change the player resource.
-**
-**  @param resource  Resource to change.
-**  @param value     How many of this resource.
-*/
-void CPlayer::SetResource(int resource, int value)
-{
-	this->Resources[resource] = value;
-}
-
 /**
 **  Check if the unit-type didn't break any unit limits.
 **
@@ -519,25 +702,21 @@ int CPlayer::CheckLimits(const CUnitType *type) const
 	//  Check game limits.
 	//
 	if (NumUnits < UnitMax) {
-		if (type->Building && this->NumBuildings >= this->BuildingLimit) {
+		if (type->Building && NumBuildings >= BuildingLimit) {
 			Notify(NotifyYellow, -1, -1, _("Building Limit Reached"));
 			return -1;
 		}
-		if (!type->Building && (this->TotalNumUnits - this->NumBuildings) >= this->UnitLimit) {
+		if (!type->Building && (TotalNumUnits - NumBuildings) >= UnitLimit) {
 			Notify(NotifyYellow, -1, -1, _("Unit Limit Reached"));
 			return -2;
 		}
-		if (this->Demand + type->Demand > this->Supply && type->Demand) {
-			Notify(NotifyYellow, -1, -1, _("Insufficient Supply, increase Supply."));
-			return -3;
-		}
-		if (this->TotalNumUnits >= this->TotalUnitLimit) {
+		if (TotalNumUnits >= TotalUnitLimit) {
 			Notify(NotifyYellow, -1, -1, _("Total Unit Limit Reached"));
 			return -4;
 		}
-		if (this->UnitTypesCount[type->Slot] >=  this->Allow.Units[type->Slot]) {
+		if (UnitTypesCount[type->Slot] >=  Allow.Units[type->Slot]) {
 			Notify(NotifyYellow, -1, -1, _("Limit of %d reached for this unit type"),
-				this->Allow.Units[type->Slot]);
+				Allow.Units[type->Slot]);
 			return -6;
 		}
 		return 1;
@@ -550,112 +729,6 @@ int CPlayer::CheckLimits(const CUnitType *type) const
 	}
 }
 
-/**
-**  Check if enough resources for are available.
-**
-**  @param costs   How many costs.
-**
-**  @return        False if all enough, otherwise a bit mask.
-**
-**  @note The return values of the PlayerCheck functions are inconsistent.
-*/
-int CPlayer::CheckCosts(const int *costs) const
-{
-	int err = 0;
-	for (int i = 1; i < MaxCosts; ++i) {
-		if (this->Resources[i] < costs[i]) {
-			Notify(NotifyYellow, -1, -1, "Not enough %s...%s more %s.",
-				DefaultResourceNames[i], DefaultActions[i], DefaultResourceNames[i]);
-
-			err |= 1 << i;
-		}
-	}
-	return err;
-}
-
-/**
-**  Check if enough resources for new unit is available.
-**
-**  @param type    Type of unit.
-**
-**  @return        False if all enough, otherwise a bit mask.
-*/
-int CPlayer::CheckUnitType(const CUnitType *type) const
-{
-	return this->CheckCosts(type->Stats[this->Index].Costs);
-}
-
-/**
-**  Add costs to the resources
-**
-**  @param costs   How many costs.
-*/
-void CPlayer::AddCosts(const int *costs)
-{
-	for (int i = 1; i < MaxCosts; ++i) {
-		this->Resources[i] += costs[i];
-	}
-}
-
-/**
-**  Add the costs of an unit type to resources
-**
-**  @param type    Type of unit.
-*/
-void CPlayer::AddUnitType(const CUnitType *type)
-{
-	// FIXME: a player could make money by upgrading and than cancel
-	AddCosts(type->Stats[this->Index].Costs);
-}
-
-/**
-**  Add a factor of costs to the resources
-**
-**  @param costs   How many costs.
-**  @param factor  Factor of the costs to apply.
-*/
-void CPlayer::AddCostsFactor(const int *costs, int factor)
-{
-	for (int i = 1; i < MaxCosts; ++i) {
-		this->Resources[i] += costs[i] * factor / 100;
-	}
-}
-
-/**
-**  Subtract costs from the resources
-**
-**  @param costs   How many costs.
-*/
-void CPlayer::SubCosts(const int *costs)
-{
-	for (int i = 1; i < MaxCosts; ++i) {
-		this->Resources[i] -= costs[i];
-	}
-}
-
-/**
-**  Substract the costs of new unit from resources
-**
-**  @param type    Type of unit.
-*/
-void CPlayer::SubUnitType(const CUnitType *type)
-{
-	this->SubCosts(type->Stats[this->Index].Costs);
-}
-
-/**
-**  Substract a factor of costs from the resources
-**
-**  @param costs   How many costs.
-**  @param factor  Factor of the costs to apply.
-*/
-void CPlayer::SubCostsFactor(const int *costs, int factor)
-{
-	for (int i = 1; i < MaxCosts; ++i) {
-		this->Resources[i] -= costs[i] * 100 / factor;
-	}
-}
-
 /**
 **  Have unit of type.
 **
@@ -665,7 +738,7 @@ void CPlayer::SubCostsFactor(const int *costs, int factor)
 */
 int CPlayer::HaveUnitTypeByType(const CUnitType *type) const
 {
-	return this->UnitTypesCount[type->Slot];
+	return UnitTypesCount[type->Slot];
 }
 
 /**
@@ -679,7 +752,7 @@ int CPlayer::HaveUnitTypeByType(const CUnitType *type) const
 */
 int CPlayer::HaveUnitTypeByIdent(const std::string &ident) const
 {
-	return this->UnitTypesCount[UnitTypeByIdent(ident)->Slot];
+	return UnitTypesCount[UnitTypeByIdent(ident)->Slot];
 }
 
 /**
@@ -700,8 +773,40 @@ void PlayersInitAi(void)
 void PlayersEachCycle(void)
 {
 	for (int player = 0; player < NumPlayers; ++player) {
-		if (Players[player].AiEnabled) {
-			AiEachCycle(&Players[player]);
+		CPlayer *p = &Players[player];
+
+		// Update rate based economy
+		for (int res = 0; res < MaxCosts; ++res) {
+			int rate = p->ProductionRate[res] - p->ActualUtilizationRate[res];
+			if (rate > 0) {
+				if (p->StoredResources[res] < p->StorageCapacity[res]) {
+					p->StoredResources[res] += rate;
+					if (p->StoredResources[res] > p->StorageCapacity[res]) {
+						p->StoredResources[res] = p->StorageCapacity[res];
+					}
+				}
+			} else if (rate < 0) {
+				rate = -rate;
+//				Assert(p->StoredResources[res] >= rate);
+				p->StoredResources[res] -= rate;
+				if (p->StoredResources[res] < 0) {
+					p->StoredResources[res] = 0;
+				}
+			}
+			p->TotalResources[res] += p->ProductionRate[res];
+		}
+
+		// Recalculate costs
+		std::map<CUnit *, int *>::iterator it;
+		for (it = p->UnitsConsumingResourcesActual.begin(); it != p->UnitsConsumingResourcesActual.end(); ++it) {
+			int costs[MaxCosts];
+			CalculateCosts((*it).first, costs);
+			p->UpdateUnitsConsumingResources((*it).first, costs);
+		}
+
+		// Ai
+		if (p->AiEnabled) {
+			AiEachCycle(p);
 		}
 	}
 }
@@ -713,16 +818,6 @@ void PlayersEachCycle(void)
 */
 void PlayersEachSecond(int player)
 {
-	if ((GameCycle / CYCLES_PER_SECOND) % 10 == 0) {
-		for (int res = 0; res < MaxCosts; ++res) {
-			Players[player].Revenue[res] =
-				Players[player].Resources[res] -
-				Players[player].LastResources[res];
-			Players[player].Revenue[res] *= 6;  // estimate per minute
-			Players[player].LastResources[res] =
-				Players[player].Resources[res];
-		}
-	}
 	if (Players[player].AiEnabled) {
 		AiEachSecond(&Players[player]);
 	}
@@ -820,7 +915,7 @@ void CPlayer::Notify(int type, int x, int y, const char *fmt, ...) const
 	va_list va;
 
 	// Notify me, and my TEAM members
-	if (this != ThisPlayer && !this->IsTeamed(ThisPlayer)) {
+	if (this != ThisPlayer && !IsTeamed(ThisPlayer)) {
 		return;
 	}
 
@@ -845,7 +940,7 @@ void CPlayer::Notify(int type, int x, int y, const char *fmt, ...) const
 */
 bool CPlayer::IsEnemy(const CPlayer *x) const
 {
-	return (this->Enemy & (1 << x->Index)) != 0;
+	return (Enemy & (1 << x->Index)) != 0;
 }
 
 /**
@@ -853,7 +948,7 @@ bool CPlayer::IsEnemy(const CPlayer *x) const
 */
 bool CPlayer::IsEnemy(const CUnit *x) const
 {
-	return this->IsEnemy(x->Player);
+	return IsEnemy(x->Player);
 }
 
 /**
@@ -861,7 +956,7 @@ bool CPlayer::IsEnemy(const CUnit *x) const
 */
 bool CPlayer::IsAllied(const CPlayer *x) const
 {
-	return (this->Allied & (1 << x->Index)) != 0;
+	return (Allied & (1 << x->Index)) != 0;
 }
 
 /**
@@ -869,7 +964,7 @@ bool CPlayer::IsAllied(const CPlayer *x) const
 */
 bool CPlayer::IsAllied(const CUnit *x) const
 {
-	return this->IsAllied(x->Player);
+	return IsAllied(x->Player);
 }
 
 /**
@@ -877,7 +972,7 @@ bool CPlayer::IsAllied(const CUnit *x) const
 */
 bool CPlayer::IsSharedVision(const CPlayer *x) const
 {
-	return (this->SharedVision & (1 << x->Index)) != 0;
+	return (SharedVision & (1 << x->Index)) != 0;
 }
 
 /**
@@ -885,7 +980,7 @@ bool CPlayer::IsSharedVision(const CPlayer *x) const
 */
 bool CPlayer::IsSharedVision(const CUnit *x) const
 {
-	return this->IsSharedVision(x->Player);
+	return IsSharedVision(x->Player);
 }
 
 /**
@@ -893,8 +988,8 @@ bool CPlayer::IsSharedVision(const CUnit *x) const
 */
 bool CPlayer::IsBothSharedVision(const CPlayer *x) const
 {
-	return (this->SharedVision & (1 << x->Index)) != 0 &&
-		(x->SharedVision & (1 << this->Index)) != 0;
+	return (SharedVision & (1 << x->Index)) != 0 &&
+		(x->SharedVision & (1 << Index)) != 0;
 }
 
 /**
@@ -902,7 +997,7 @@ bool CPlayer::IsBothSharedVision(const CPlayer *x) const
 */
 bool CPlayer::IsBothSharedVision(const CUnit *x) const
 {
-	return this->IsBothSharedVision(x->Player);
+	return IsBothSharedVision(x->Player);
 }
 
 /**
@@ -910,7 +1005,7 @@ bool CPlayer::IsBothSharedVision(const CUnit *x) const
 */
 bool CPlayer::IsTeamed(const CPlayer *x) const
 {
-	return this->Team == x->Team;
+	return Team == x->Team;
 }
 
 /**
@@ -918,7 +1013,7 @@ bool CPlayer::IsTeamed(const CPlayer *x) const
 */
 bool CPlayer::IsTeamed(const CUnit *x) const
 {
-	return this->IsTeamed(x->Player);
+	return IsTeamed(x->Player);
 }
 
 //@}
