@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: script.cpp 8593 2007-03-04 13:13:57Z feb $
+//      $Id: script.cpp 9000 2007-07-07 20:30:01Z jim4 $
 
 //@{
 
@@ -48,7 +48,6 @@
 #include "iolib.h"
 #include "script.h"
 #include "missile.h"
-#include "depend.h"
 #include "upgrade.h"
 #include "construct.h"
 #include "unit.h"
@@ -58,7 +57,6 @@
 #include "ui.h"
 #include "interface.h"
 #include "font.h"
-#include "pathfinder.h"
 #include "ai.h"
 #include "results.h"
 #include "trigger.h"
@@ -78,15 +76,14 @@
 
 lua_State *Lua;                       /// Structure to work with lua files.
 
-char *CclStartFile;                   /// CCL start file
+std::string CclStartFile;             /// CCL start file
 std::string UserDirectory;
 int CclInConfigFile;                  /// True while config file parsing
-int SaveGameLoading;                  /// If a Saved Game is Loading
+bool SaveGameLoading;                 /// If a Saved Game is Loading
 std::string CurrentLuaFile;           /// Lua file currently being interpreted
 
 int NoRandomPlacementMultiplayer = 0; /// Disable the random placement of players in muliplayer mode
 
-bool UseHPForXp = false;              /// true if gain XP by dealing damage, false if by killing.
 NumberDesc *Damage;                   /// Damage calculation for missile.
 
 static int NumberCounter = 0; /// Counter for lua function.
@@ -130,18 +127,21 @@ static void laction(int i)
 }
 
 /**
-**  Print error message and exit.
+**  Print error message and possibly exit.
 **
 **  @param pname  Source of the error.
 **  @param msg    error message to print.
+**  @param exit   exit the program
 */
-static void l_message(const char *pname, const char *msg)
+static void l_message(const char *pname, const char *msg, bool exit)
 {
 	if (pname) {
 		fprintf(stderr, "%s: ", pname);
 	}
 	fprintf(stderr, "%s\n", msg);
-	exit(1);
+	if (exit) {
+		::exit(1);
+	}
 }
 
 /**
@@ -149,10 +149,11 @@ static void l_message(const char *pname, const char *msg)
 **  if status is different of 0.
 **
 **  @param status  status of the last lua call. (0: success)
+**  @param exit    exit the program on error
 **
 **  @return        0 in success, else exit.
 */
-static int report(int status)
+static int report(int status, bool exit)
 {
 	const char *msg;
 
@@ -161,7 +162,7 @@ static int report(int status)
 		if (msg == NULL) {
 			msg = "(error with no message)";
 		}
-		l_message(NULL, msg);
+		l_message(NULL, msg, exit);
 		lua_pop(Lua, 1);
 	}
 	return status;
@@ -208,13 +209,13 @@ int LuaCall(int narg, int clear)
 	signal(SIGINT, SIG_DFL);
 	lua_remove(Lua, base);  // remove traceback function
 
-	return report(status);
+	return report(status, true);
 }
 
 /**
 **  Load a file into a buffer
 */
-static void LuaLoadBuffer(const char *file, char **buffer, int *buffersize)
+static void LuaLoadBuffer(const std::string &file, std::string &buffer)
 {
 	CFile fp;
 	int size;
@@ -223,12 +224,11 @@ static void LuaLoadBuffer(const char *file, char **buffer, int *buffersize)
 	int read;
 	char *buf;
 
-	*buffer = NULL;
-	*buffersize = 0;
+	buffer.clear();
 
-	if (fp.open(file, CL_OPEN_READ) == -1) {
+	if (fp.open(file.c_str(), CL_OPEN_READ) == -1) {
 		fprintf(stderr, "Can't open file '%s': %s\n",
-			file, strerror(errno));
+			file.c_str(), strerror(errno));
 		return;
 	}
 
@@ -259,8 +259,8 @@ static void LuaLoadBuffer(const char *file, char **buffer, int *buffersize)
 	}
 	fp.close();
 
-	*buffer = buf;
-	*buffersize = location;
+	buffer.assign(buf, location);
+	delete[] buf;
 }
 
 /**
@@ -274,23 +274,21 @@ int LuaLoadFile(const std::string &file)
 {
 	int status;
 	std::string PreviousLuaFile;
-	char *buf;
-	int size;
+	std::string buf;
 
 	PreviousLuaFile = CurrentLuaFile;
 	CurrentLuaFile = file;
 
-	LuaLoadBuffer(file.c_str(), &buf, &size);
-	if (!buf) {
+	LuaLoadBuffer(file, buf);
+	if (buf.empty()) {
 		return -1;
 	}
 
-	if (!(status = luaL_loadbuffer(Lua, buf, size, file.c_str()))) {
+	if (!(status = luaL_loadbuffer(Lua, buf.c_str(), buf.size(), file.c_str()))) {
 		LuaCall(0, 1);
 	} else {
-		report(status);
+		report(status, true);
 	}
-	delete[] buf;
 	CurrentLuaFile = PreviousLuaFile;
 
 	return status;
@@ -359,15 +357,13 @@ static int CclLoad(lua_State *l)
 static int CclLoadBuffer(lua_State *l)
 {
 	char file[1024];
-	char *buf;
-	int size;
+	std::string buf;
 
 	LuaCheckArgs(l, 1);
 	LibraryFileName(LuaToString(l, 1), file, sizeof(file));
-	LuaLoadBuffer(file, &buf, &size);
-	if (buf) {
-		lua_pushstring(l, buf);
-		delete[] buf; // Lua creates an internal copy
+	LuaLoadBuffer(file, buf);
+	if (!buf.empty()) {
+		lua_pushstring(l, buf.c_str());
 		return 1;
 	}
 	return 0;
@@ -1869,10 +1865,8 @@ static int CclFilteredListDirectory(lua_State *l, int type, int mask)
 	}
 
 	// security: disallow all special characters
-	for (i = pathtype; i < n; i++) {
-		if (!(isalnum(userdir[i]) || userdir[i]=='/')) {
-			LuaError(l, "Forbidden directory");
-		}
+	if (strpbrk(userdir, ":*?\"<>|") != 0 || strstr(userdir, "..") != 0) {
+		LuaError(l, "Forbidden directory");
 	}
 
 	if (pathtype == 1) {
@@ -1884,7 +1878,7 @@ static int CclFilteredListDirectory(lua_State *l, int type, int mask)
 	lua_pop(l, 1);
 	lua_newtable(l);
 	n = ReadDataDirectory(directory, NULL, flp);
-	for (i = 0, j = 0; i < n; i++) {
+	for (i = 0, j = 0; i < n; ++i) {
 		if ((flp[i].type & mask) == type) {
 			lua_pushnumber(l, j + 1);
 			lua_pushstring(l, flp[i].name);
@@ -1921,31 +1915,6 @@ static int CclListDirsInDirectory(lua_State *l)
 	return CclFilteredListDirectory(l, 0x0, 0x1);
 }
 
-/**
-**  Return the stratagus game-cycle
-**
-**  @param l  Lua state.
-**
-**  @return   Current game cycle.
-*/
-static int CclGameCycle(lua_State *l)
-{
-	lua_pushnumber(l, GameCycle);
-	return 1;
-}
-
-/**
-**  Set the stratagus game-cycle
-**
-**  @param l  Lua state.
-*/
-static int CclSetGameCycle(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	GameCycle = LuaToNumber(l, 1);
-	return 0;
-}
-
 /**
 **  Set the game paused or unpaused
 **
@@ -1985,8 +1954,10 @@ static int CclSetVideoSyncSpeed(lua_State *l)
 static int CclSetGameSpeed(lua_State *l)
 {
 	LuaCheckArgs(l, 1);
-	VideoSyncSpeed = LuaToNumber(l, 1) * 100 / CYCLES_PER_SECOND;
-	SetVideoSync();
+	if (FastForwardCycle < GameCycle) {
+		VideoSyncSpeed = LuaToNumber(l, 1) * 100 / CYCLES_PER_SECOND;
+		SetVideoSync();
+	}
 	return 0;
 }
 
@@ -2009,11 +1980,8 @@ static int CclGetGameSpeed(lua_State *l)
 */
 static int CclSetLocalPlayerName(lua_State *l)
 {
-	const char *str;
-
 	LuaCheckArgs(l, 1);
-	str = LuaToString(l, 1);
-	strncpy_s(LocalPlayerName, sizeof(LocalPlayerName), str, _TRUNCATE);
+	LocalPlayerName = LuaToString(l, 1);
 	return 0;
 }
 
@@ -2025,25 +1993,10 @@ static int CclSetLocalPlayerName(lua_State *l)
 static int CclGetLocalPlayerName(lua_State *l)
 {
 	LuaCheckArgs(l, 0);
-	lua_pushstring(l, LocalPlayerName);
+	lua_pushstring(l, LocalPlayerName.c_str());
 	return 1;
 }
 
-
-/**
-**  Affect UseHPForXp.
-**
-**  @param l  Lua state.
-**
-**  @return 0.
-*/
-static int ScriptSetUseHPForXp(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	UseHPForXp = LuaToBoolean(l, 1);
-	return 0;
-}
-
 /**
 **  Removes Randomization of Player position in Multiplayer mode
 **
@@ -2099,7 +2052,7 @@ static int CclSetSpeedResourcesHarvest(lua_State *l)
 	LuaCheckArgs(l, 2);
 	resource = LuaToString(l, 1);
 	for (i = 0; i < MaxCosts; ++i) {
-		if (!strcmp(resource, DefaultResourceNames[i])) {
+		if (resource == DefaultResourceNames[i]) {
 			SpeedResourcesHarvest[i] = LuaToNumber(l, 2);
 			return 0;
 		}
@@ -2122,7 +2075,7 @@ static int CclSetSpeedResourcesReturn(lua_State *l)
 	LuaCheckArgs(l, 2);
 	resource = LuaToString(l, 1);
 	for (i = 0; i < MaxCosts; ++i) {
-		if (!strcmp(resource, DefaultResourceNames[i])) {
+		if (resource == DefaultResourceNames[i]) {
 			SpeedResourcesReturn[i] = LuaToNumber(l, 2);
 			return 0;
 		}
@@ -2160,34 +2113,6 @@ static int CclSetSpeedTrain(lua_State *l)
 	return 1;
 }
 
-/**
-**  For debug increase upgrading speed.
-**
-**  @param l  Lua state.
-*/
-static int CclSetSpeedUpgrade(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	SpeedUpgrade = LuaToNumber(l, 1);
-
-	lua_pushnumber(l, SpeedUpgrade);
-	return 1;
-}
-
-/**
-**  For debug increase researching speed.
-**
-**  @param l  Lua state.
-*/
-static int CclSetSpeedResearch(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	SpeedResearch = LuaToNumber(l, 1);
-
-	lua_pushnumber(l, SpeedResearch);
-	return 1;
-}
-
 /**
 **  For debug increase all speeds.
 **
@@ -2204,50 +2129,12 @@ static int CclSetSpeeds(lua_State *l)
 		SpeedResourcesHarvest[i] = s;
 		SpeedResourcesReturn[i] = s;
 	}
-	SpeedBuild = SpeedTrain = SpeedUpgrade = SpeedResearch = s;
+	SpeedBuild = SpeedTrain = s;
 
 	lua_pushnumber(l, s);
 	return 1;
 }
 
-/**
-**  Define default incomes for a new player.
-**
-**  @param l  Lua state.
-*/
-static int CclDefineDefaultIncomes(lua_State *l)
-{
-	int i;
-	int args;
-
-	args = lua_gettop(l);
-	for (i = 0; i < MaxCosts && i < args; ++i) {
-		DefaultIncomes[i] = LuaToNumber(l, i + 1);
-	}
-	return 0;
-}
-
-/**
-**  Define default action for the resources.
-**
-**  @param l  Lua state.
-*/
-static int CclDefineDefaultActions(lua_State *l)
-{
-	int i;
-	int args;
-
-	for (i = 0; i < MaxCosts; ++i) {
-		delete[] DefaultActions[i];
-		DefaultActions[i] = NULL;
-	}
-	args = lua_gettop(l);
-	for (i = 0; i < MaxCosts && i < args; ++i) {
-		DefaultActions[i] = new_strdup(LuaToString(l, i + 1));
-	}
-	return 0;
-}
-
 /**
 **  Define default names for the resources.
 **
@@ -2259,12 +2146,11 @@ static int CclDefineDefaultResourceNames(lua_State *l)
 	int args;
 
 	for (i = 0; i < MaxCosts; ++i) {
-		delete[] DefaultResourceNames[i];
-		DefaultResourceNames[i] = NULL;
+		DefaultResourceNames[i].clear();
 	}
 	args = lua_gettop(l);
 	for (i = 0; i < MaxCosts && i < args; ++i) {
-		DefaultResourceNames[i] = new_strdup(LuaToString(l, i + 1));
+		DefaultResourceNames[i] = LuaToString(l, i + 1);
 	}
 	return 0;
 }
@@ -2288,7 +2174,7 @@ static int CclDefineDefaultResourceAmounts(lua_State *l)
 	for (j = 0; j < args; ++j) {
 		value = LuaToString(l, j + 1);
 		for (i = 0; i < MaxCosts; ++i) {
-			if (!strcmp(value, DefaultResourceNames[i])) {
+			if (value == DefaultResourceNames[i]) {
 				++j;
 				DefaultResourceAmounts[i] = LuaToNumber(l, j + 1);
 				break;
@@ -2378,7 +2264,7 @@ int CclCommand(const char *command)
 	if (!(status = luaL_loadbuffer(Lua, command, strlen(command), command))) {
 		LuaCall(0, 1);
 	} else {
-		report(status);
+		report(status, false);
 	}
 	return status;
 }
@@ -2408,8 +2294,6 @@ void InitCcl(void)
 	lua_register(Lua, "ListDirectory", CclListDirectory);
 	lua_register(Lua, "ListFilesInDirectory", CclListFilesInDirectory);
 	lua_register(Lua, "ListDirsInDirectory", CclListDirsInDirectory);
-	lua_register(Lua, "GameCycle", CclGameCycle);
-	lua_register(Lua, "SetGameCycle", CclSetGameCycle);
 	lua_register(Lua, "SetGamePaused", CclSetGamePaused);
 	lua_register(Lua, "SetVideoSyncSpeed", CclSetVideoSyncSpeed);
 	lua_register(Lua, "SetGameSpeed", CclSetGameSpeed);
@@ -2422,14 +2306,9 @@ void InitCcl(void)
 	lua_register(Lua, "SetSpeedResourcesReturn", CclSetSpeedResourcesReturn);
 	lua_register(Lua, "SetSpeedBuild", CclSetSpeedBuild);
 	lua_register(Lua, "SetSpeedTrain", CclSetSpeedTrain);
-	lua_register(Lua, "SetSpeedUpgrade", CclSetSpeedUpgrade);
-	lua_register(Lua, "SetSpeedResearch", CclSetSpeedResearch);
 	lua_register(Lua, "SetSpeeds", CclSetSpeeds);
-	lua_register(Lua, "SetUseHPForXp", ScriptSetUseHPForXp);
 	lua_register(Lua, "SetDamageFormula", CclSetDamageFormula);
 
-	lua_register(Lua, "DefineDefaultIncomes", CclDefineDefaultIncomes);
-	lua_register(Lua, "DefineDefaultActions", CclDefineDefaultActions);
 	lua_register(Lua, "DefineDefaultResourceNames", CclDefineDefaultResourceNames);
 	lua_register(Lua, "DefineDefaultResourceAmounts", CclDefineDefaultResourceAmounts);
 	lua_register(Lua, "NoRandomPlacementMultiplayer", CclNoRandomPlacementMultiplayer);
@@ -2447,12 +2326,10 @@ void InitCcl(void)
 	PlayerCclRegister();
 	TilesetCclRegister();
 	MapCclRegister();
-	PathfinderCclRegister();
 	ConstructionCclRegister();
 	DecorationCclRegister();
 	UnitTypeCclRegister();
 	UpgradesCclRegister();
-	DependenciesCclRegister();
 	SelectionCclRegister();
 	GroupCclRegister();
 	UnitCclRegister();
@@ -2534,7 +2411,7 @@ void LoadCcl(void)
 	//  Load and evaluate configuration file
 	//
 	CclInConfigFile = 1;
-	file = LibraryFileName(CclStartFile, buf, sizeof(buf));
+	file = LibraryFileName(CclStartFile.c_str(), buf, sizeof(buf));
 	if (access(buf, R_OK)) {
 		printf("Maybe you need to specify another gamepath with '-d /path/to/datadir'?\n");
 		ExitFatal(-1);
@@ -2553,20 +2430,16 @@ void LoadCcl(void)
 */
 void SaveCcl(CFile *file)
 {
-	int i;
-
 	file->printf("SetGodMode(%s)\n", GodMode ? "true" : "false");
 
-	for (i = 0; i < MaxCosts; ++i) {
+	for (int i = 0; i < MaxCosts; ++i) {
 		file->printf("SetSpeedResourcesHarvest(\"%s\", %d)\n",
-			DefaultResourceNames[i], SpeedResourcesHarvest[i]);
+			DefaultResourceNames[i].c_str(), SpeedResourcesHarvest[i]);
 		file->printf("SetSpeedResourcesReturn(\"%s\", %d)\n",
-			DefaultResourceNames[i], SpeedResourcesReturn[i]);
+			DefaultResourceNames[i].c_str(), SpeedResourcesReturn[i]);
 	}
 	file->printf("SetSpeedBuild(%d)\n", SpeedBuild);
 	file->printf("SetSpeedTrain(%d)\n", SpeedTrain);
-	file->printf("SetSpeedUpgrade(%d)\n", SpeedUpgrade);
-	file->printf("SetSpeedResearch(%d)\n", SpeedResearch);
 }
 
 //@}
