@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: script_player.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: script_player.cpp 8902 2007-06-20 05:51:48Z jim4 $
 
 //@{
 
@@ -53,21 +53,6 @@
 --  Functions
 ----------------------------------------------------------------------------*/
 
-/**
-**  Get a player pointer
-**
-**  @param l  Lua state.
-**
-**  @return   The player pointer
-*/
-static CPlayer *CclGetPlayer(lua_State *l)
-{
-	int i = (int)LuaToNumber(l, -1);
-	if ( (i < 0)  || (i > PlayerMax) )
-		return NULL;
-	return &Players[i];
-}
-
 /**
 **  Parse the player configuration.
 **
@@ -162,7 +147,7 @@ static int CclPlayer(lua_State *l)
 			lua_rawgeti(l, j + 1, 2);
 			player->StartY = LuaToNumber(l, -1);
 			lua_pop(l, 1);
-		} else if (!strcmp(value, "resources")) {
+		} else if (!strcmp(value, "production-rate")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
 			}
@@ -174,9 +159,9 @@ static int CclPlayer(lua_State *l)
 				++k;
 
 				for (i = 0; i < MaxCosts; ++i) {
-					if (!strcmp(value, DefaultResourceNames[i])) {
+					if (value == DefaultResourceNames[i]) {
 						lua_rawgeti(l, j + 1, k + 1);
-						player->Resources[i] = LuaToNumber(l, -1);
+						player->ProductionRate[i] = LuaToNumber(l, -1);
 						lua_pop(l, 1);
 						break;
 					}
@@ -185,7 +170,7 @@ static int CclPlayer(lua_State *l)
 					LuaError(l, "Unsupported tag: %s" _C_ value);
 				}
 			}
-		} else if (!strcmp(value, "last-resources")) {
+		} else if (!strcmp(value, "stored-resources")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
 			}
@@ -197,9 +182,9 @@ static int CclPlayer(lua_State *l)
 				++k;
 
 				for (i = 0; i < MaxCosts; ++i) {
-					if (!strcmp(value, DefaultResourceNames[i])) {
+					if (value == DefaultResourceNames[i]) {
 						lua_rawgeti(l, j + 1, k + 1);
-						player->LastResources[i] = LuaToNumber(l, -1);
+						player->StoredResources[i] = LuaToNumber(l, -1);
 						lua_pop(l, 1);
 						break;
 					}
@@ -208,7 +193,7 @@ static int CclPlayer(lua_State *l)
 					LuaError(l, "Unsupported tag: %s" _C_ value);
 				}
 			}
-		} else if (!strcmp(value, "incomes")) {
+		} else if (!strcmp(value, "storage-capacity")) {
 			if (!lua_istable(l, j + 1)) {
 				LuaError(l, "incorrect argument");
 			}
@@ -220,32 +205,9 @@ static int CclPlayer(lua_State *l)
 				++k;
 
 				for (i = 0; i < MaxCosts; ++i) {
-					if (!strcmp(value, DefaultResourceNames[i])) {
+					if (value == DefaultResourceNames[i]) {
 						lua_rawgeti(l, j + 1, k + 1);
-						player->Incomes[i] = LuaToNumber(l, -1);
-						lua_pop(l, 1);
-						break;
-					}
-				}
-				if (i == MaxCosts) {
-					LuaError(l, "Unsupported tag: %s" _C_ value);
-				}
-			}
-		} else if (!strcmp(value, "revenue")) {
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			for (k = 0; k < subargs; ++k) {
-				lua_rawgeti(l, j + 1, k + 1);
-				value = LuaToString(l, -1);
-				lua_pop(l, 1);
-				++k;
-
-				for (i = 0; i < MaxCosts; ++i) {
-					if (!strcmp(value, DefaultResourceNames[i])) {
-						lua_rawgeti(l, j + 1, k + 1);
-						player->Revenue[i] = LuaToNumber(l, -1);
+						player->StorageCapacity[i] = LuaToNumber(l, -1);
 						lua_pop(l, 1);
 						break;
 					}
@@ -260,10 +222,6 @@ static int CclPlayer(lua_State *l)
 		} else if (!strcmp(value, "ai-disabled")) {
 			player->AiEnabled = 0;
 			--j;
-		} else if (!strcmp(value, "supply")) {
-			player->Supply = LuaToNumber(l, j + 1);
-		} else if (!strcmp(value, "demand")) {
-			player->Demand = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "unit-limit")) {
 			player->UnitLimit = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "building-limit")) {
@@ -311,19 +269,6 @@ static int CclPlayer(lua_State *l)
 			b = LuaToNumber(l, -1);
 			lua_pop(l, 1);
 			player->Color = Video.MapRGB(TheScreen->format, r, g, b);
-		} else if (!strcmp(value, "timers")) {
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			if (subargs != UpgradeMax) {
-				LuaError(l, "Wrong upgrade timer length: %d" _C_ i);
-			}
-			for (k = 0; k < subargs; ++k) {
-				lua_rawgeti(l, j + 1, k + 1);
-				player->UpgradeTimers.Upgrades[k] = LuaToNumber(l, -1);
-				lua_pop(l, 1);
-			}
 		} else {
 		   LuaError(l, "Unsupported tag: %s" _C_ value);
 		}
@@ -385,39 +330,6 @@ static int CclChangeUnitsOwner(lua_State *l)
 	return 0;
 }
 
-/**
-**  Get ThisPlayer.
-**
-**  @param l  Lua state.
-*/
-static int CclGetThisPlayer(lua_State *l)
-{
-	LuaCheckArgs(l, 0);
-	if (ThisPlayer) {
-		lua_pushnumber(l, ThisPlayer - Players);
-	} else {
-		lua_pushnumber(l, 0);
-	}
-	return 1;
-}
-
-/**
-**  Set ThisPlayer.
-**
-**  @param l  Lua state.
-*/
-static int CclSetThisPlayer(lua_State *l)
-{
-	int plynr;
-
-	LuaCheckArgs(l, 1);
-	plynr = LuaToNumber(l, 1);
-	ThisPlayer = &Players[plynr];
-
-	lua_pushnumber(l, plynr);
-	return 1;
-}
-
 /**
 **  Set MaxSelectable
 **
@@ -653,235 +565,6 @@ static int CclDefinePlayerColorIndex(lua_State *l)
 
 // ----------------------------------------------------------------------------
 
-/**
-**  Get player data.
-**
-**  @param l  Lua state.
-*/
-static int CclGetPlayerData(lua_State *l)
-{
-	CPlayer *p;
-	const char *data;
-
-	if (lua_gettop(l) < 2) {
-		LuaError(l, "incorrect argument");
-	}
-	lua_pushvalue(l, 1);
-	p = CclGetPlayer(l);
-	lua_pop(l, 1);
-	data = LuaToString(l, 2);
-
-	if (!strcmp(data, "Name")) {
-		lua_pushstring(l, p->Name.c_str());
-		return 1;
-	} else if (!strcmp(data, "Resources")) {
-		const char *res;
-		int i;
-
-		LuaCheckArgs(l, 3);
-		res = LuaToString(l, 3);
-		for (i = 0; i < MaxCosts; ++i) {
-			if (!strcmp(res, DefaultResourceNames[i])) {
-				break;
-			}
-		}
-		if (i == MaxCosts) {
-			LuaError(l, "Invalid resource \"%s\"" _C_ res);
-		}
-		lua_pushnumber(l, p->Resources[i]);
-		return 1;
-	} else if (!strcmp(data, "UnitTypesCount")) {
-		CUnitType *type;
-
-		LuaCheckArgs(l, 3);
-		type = CclGetUnitType(l);
-		lua_pushnumber(l, p->UnitTypesCount[type->Slot]);
-		return 1;
-	} else if (!strcmp(data, "AiEnabled")) {
-		lua_pushboolean(l, p->AiEnabled);
-		return 1;
-	} else if (!strcmp(data, "TotalNumUnits")) {
-		lua_pushnumber(l, p->TotalNumUnits);
-		return 1;
-	} else if (!strcmp(data, "NumBuildings")) {
-		lua_pushnumber(l, p->NumBuildings);
-		return 1;
-	} else if (!strcmp(data, "Supply")) {
-		lua_pushnumber(l, p->Supply);
-		return 1;
-	} else if (!strcmp(data, "Demand")) {
-		lua_pushnumber(l, p->Demand);
-		return 1;
-	} else if (!strcmp(data, "UnitLimit")) {
-		lua_pushnumber(l, p->UnitLimit);
-		return 1;
-	} else if (!strcmp(data, "BuildingLimit")) {
-		lua_pushnumber(l, p->BuildingLimit);
-		return 1;
-	} else if (!strcmp(data, "TotalUnitLimit")) {
-		lua_pushnumber(l, p->TotalUnitLimit);
-		return 1;
-	} else if (!strcmp(data, "Score")) {
-		lua_pushnumber(l, p->Score);
-		return 1;
-	} else if (!strcmp(data, "TotalUnits")) {
-		lua_pushnumber(l, p->TotalUnits);
-		return 1;
-	} else if (!strcmp(data, "TotalBuildings")) {
-		lua_pushnumber(l, p->TotalBuildings);
-		return 1;
-	} else if (!strcmp(data, "TotalResources")) {
-		const char *res;
-		int i;
-
-		LuaCheckArgs(l, 3);
-		res = LuaToString(l, 3);
-		for (i = 0; i < MaxCosts; ++i) {
-			if (!strcmp(res, DefaultResourceNames[i])) {
-				break;
-			}
-		}
-		if (i == MaxCosts) {
-			LuaError(l, "Invalid resource \"%s\"" _C_ res);
-		}
-		lua_pushnumber(l, p->TotalResources[i]);
-		return 1;
-	} else if (!strcmp(data, "TotalRazings")) {
-		lua_pushnumber(l, p->TotalRazings);
-		return 1;
-	} else if (!strcmp(data, "TotalKills")) {
-		lua_pushnumber(l, p->TotalKills);
-		return 1;
-	} else {
-		LuaError(l, "Invalid field: %s" _C_ data);
-	}
-
-	return 0;
-}
-
-/**
-**  Set player data.
-**
-**  @param l  Lua state.
-*/
-static int CclSetPlayerData(lua_State *l)
-{
-	CPlayer *p;
-	const char *data;
-
-	if (lua_gettop(l) < 3) {
-		LuaError(l, "incorrect argument");
-	}
-	lua_pushvalue(l, 1);
-	p = CclGetPlayer(l);
-	lua_pop(l, 1);
-	data = LuaToString(l, 2);
-
-	if (!strcmp(data, "Name")) {
-		p->SetName(LuaToString(l, 3));
-	} else if (!strcmp(data, "Resources")) {
-		const char *res;
-		int i;
-
-		LuaCheckArgs(l, 4);
-		res = LuaToString(l, 3);
-		for (i = 0; i < MaxCosts; ++i) {
-			if (!strcmp(res, DefaultResourceNames[i])) {
-				break;
-			}
-		}
-		if (i == MaxCosts) {
-			LuaError(l, "Invalid resource \"%s\"" _C_ res);
-		}
-		p->Resources[i] = LuaToNumber(l, 4);
-// } else if (!strcmp(data, "UnitTypesCount")) {
-// } else if (!strcmp(data, "AiEnabled")) {
-// } else if (!strcmp(data, "TotalNumUnits")) {
-// } else if (!strcmp(data, "NumBuildings")) {
-// } else if (!strcmp(data, "Supply")) {
-// } else if (!strcmp(data, "Demand")) {
-	} else if (!strcmp(data, "UnitLimit")) {
-		p->UnitLimit = LuaToNumber(l, 3);
-	} else if (!strcmp(data, "BuildingLimit")) {
-		p->BuildingLimit = LuaToNumber(l, 3);
-	} else if (!strcmp(data, "TotalUnitLimit")) {
-		p->TotalUnitLimit = LuaToNumber(l, 3);
-	} else if (!strcmp(data, "Score")) {
-		p->Score = LuaToNumber(l, 3);
-	} else if (!strcmp(data, "TotalUnits")) {
-		p->TotalUnits = LuaToNumber(l, 3);
-	} else if (!strcmp(data, "TotalBuildings")) {
-		p->TotalBuildings = LuaToNumber(l, 3);
-	} else if (!strcmp(data, "TotalResources")) {
-		const char *res;
-		int i;
-
-		LuaCheckArgs(l, 3);
-		res = LuaToString(l, 3);
-		for (i = 0; i < MaxCosts; ++i) {
-			if (!strcmp(res, DefaultResourceNames[i])) {
-				break;
-			}
-		}
-		if (i == MaxCosts) {
-			LuaError(l, "Invalid resource \"%s\"" _C_ res);
-		}
-		p->TotalResources[i] = LuaToNumber(l, 4);
-	} else if (!strcmp(data, "TotalRazings")) {
-		p->TotalRazings = LuaToNumber(l, 3);
-	} else if (!strcmp(data, "TotalKills")) {
-		p->TotalKills = LuaToNumber(l, 3);
-	} else {
-		LuaError(l, "Invalid field: %s" _C_ data);
-	}
-
-	return 0;
-}
-
-/**
-**  Get ai player algo.
-**
-**  @param l  Lua state.
-*/
-static int CclGetAiType(lua_State *l)
-{
-	CPlayer *p;
-
-	if (lua_gettop(l) < 1) {
-		LuaError(l, "incorrect argument");
-	}
-	lua_pushvalue(l, 1);
-	p = CclGetPlayer(l);
-	lua_pop(l, 1);
-
-	lua_pushstring(l, p->AiName.c_str());
-
-	return 1;
-}
-
-/**
-**  Set ai player algo.
-**
-**  @param l  Lua state.
-*/
-static int CclSetAiType(lua_State *l)
-{
-	CPlayer *p;
-
-	if (lua_gettop(l) < 2) {
-		LuaError(l, "incorrect argument");
-	}
-	lua_pushvalue(l, 1);
-	p = CclGetPlayer(l);
-	lua_pop(l, 1);
-
-	p->AiName = LuaToString(l, 2);
-
-	return 0;
-}
-
-// ----------------------------------------------------------------------------
-
 /**
 **  Register CCL features for players.
 */
@@ -889,8 +572,6 @@ void PlayerCclRegister(void)
 {
 	lua_register(Lua, "Player", CclPlayer);
 	lua_register(Lua, "ChangeUnitsOwner", CclChangeUnitsOwner);
-	lua_register(Lua, "GetThisPlayer", CclGetThisPlayer);
-	lua_register(Lua, "SetThisPlayer", CclSetThisPlayer);
 
 	lua_register(Lua, "SetMaxSelectable", CclSetMaxSelectable);
 
@@ -910,12 +591,6 @@ void PlayerCclRegister(void)
 	lua_register(Lua, "DefinePlayerColorIndex", CclDefinePlayerColorIndex);
 
 	lua_register(Lua, "NewColors", CclNewPlayerColors);
-
-	// player member access functions
-	lua_register(Lua, "GetPlayerData", CclGetPlayerData);
-	lua_register(Lua, "SetPlayerData", CclSetPlayerData);
-	lua_register(Lua, "GetAiType", CclGetAiType);
-	lua_register(Lua, "SetAiType", CclSetAiType);
 }
 
 //@}
