@@ -9,7 +9,7 @@
 //
 /**@name spells.cpp - The spell cast action. */
 //
-//      (c) Copyright 1998-2006 by Vladi Belperchinov-Shabanski, Lutz Sammer,
+//      (c) Copyright 1998-2007 by Vladi Belperchinov-Shabanski, Lutz Sammer,
 //                                 Jimmy Salmon, and Joris DAUPHIN
 //
 //      This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-// $Id: spells.cpp 8550 2007-02-10 12:33:10Z feb $
+// $Id: spells.cpp 8750 2007-04-14 18:09:46Z jim4 $
 
 /*
 ** And when we cast our final spell
@@ -546,12 +546,6 @@ int Polymorph::Cast(CUnit *caster, const SpellType *spell,
 		} else {
 			caster->Player->TotalKills++;
 		}
-		if (UseHPForXp) {
-			caster->Variable[XP_INDEX].Max += target->Variable[HP_INDEX].Value;
-		} else {
-			caster->Variable[XP_INDEX].Max += target->Type->Points;
-		}
-		caster->Variable[XP_INDEX].Value = caster->Variable[XP_INDEX].Max;
 		caster->Variable[KILL_INDEX].Value++;
 		caster->Variable[KILL_INDEX].Max++;
 		caster->Variable[KILL_INDEX].Enable = 1;
@@ -618,12 +612,6 @@ int Capture::Cast(CUnit *caster, const SpellType *spell,
 		} else {
 			caster->Player->TotalKills++;
 		}
-		if (UseHPForXp) {
-			caster->Variable[XP_INDEX].Max += target->Variable[HP_INDEX].Value;
-		} else {
-			caster->Variable[XP_INDEX].Max += target->Type->Points;
-		}
-		caster->Variable[XP_INDEX].Value = caster->Variable[XP_INDEX].Max;
 		caster->Variable[KILL_INDEX].Value++;
 		caster->Variable[KILL_INDEX].Max++;
 		caster->Variable[KILL_INDEX].Enable = 1;
@@ -757,8 +745,8 @@ static Target *NewTargetPosition(int x, int y)
 **  @param caster      Pointer to caster unit.
 **  @param spell       Pointer to the spell to cast.
 **  @param target      Pointer to target unit, or 0 if it is a position spell.
-**  @param x           X position, or -1 if it is an unit spell.
-**  @param y           Y position, or -1 if it is an unit spell.
+**  @param x           X position, or -1 if it is a unit spell.
+**  @param y           Y position, or -1 if it is a unit spell.
 **  @param condition   Pointer to condition info.
 **
 **  @return            1 if passed, 0 otherwise.
@@ -771,7 +759,7 @@ static int PassCondition(const CUnit *caster, const SpellType *spell, const CUni
 	if (caster->Variable[MANA_INDEX].Value < spell->ManaCost) { // Check caster mana.
 		return 0;
 	}
-	if (spell->Target == TargetUnit) { // Casting an unit spell without a target.
+	if (spell->Target == TargetUnit) { // Casting a unit spell without a target.
 		if ((!target) || target->Destroyed || target->Orders[0]->Action == UnitActionDie) {
 			return 0;
 		}
@@ -1009,7 +997,7 @@ SpellType *SpellTypeByIdent(const std::string &ident)
 // ****************************************************************************
 
 /**
-**  Check if spell is research for player \p player.
+**  Check if spell is available for player \p player.
 **  @param player    player for who we want to know if he knows the spell.
 **  @param spellid   id of the spell to check.
 **
@@ -1020,7 +1008,7 @@ bool SpellIsAvailable(const CPlayer *player, int spellid)
 	int dependencyId;
 	dependencyId = SpellTypeTable[spellid]->DependencyId;
 
-	return dependencyId == -1 || UpgradeIdAllowed(player, dependencyId) == 'R';
+	return dependencyId == -1;
 }
 
 /**
@@ -1033,7 +1021,7 @@ bool SpellIsAvailable(const CPlayer *player, int spellid)
 **  @param y         Y coord of target spot when/if target does not exist
 **
 **  @return          =!0 if spell should/can casted, 0 if not
-**  @note caster must know the spell, and spell must be researched.
+**  @note caster must know the spell, and spell must be available.
 */
 int CanCastSpell(const CUnit *caster, const SpellType *spell,
 	const CUnit *target, int x, int y)
@@ -1091,7 +1079,6 @@ int SpellCast(CUnit *caster, const SpellType *spell, CUnit *target,
 	int cont;             // Should we recast the spell.
 	int mustSubtractMana; // false if action which have their own calculation is present.
 
-	caster->Variable[INVISIBLE_INDEX].Value = 0;// unit is invisible until attacks // FIXME: Must be configurable
 	if (target) {
 		x = target->X;
 		y = target->Y;
