@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: stratagus.cpp 8586 2007-03-03 12:24:04Z feb $
+//      $Id: stratagus.cpp 8902 2007-06-20 05:51:48Z jim4 $
 
 //@{
 
@@ -85,11 +85,6 @@
 ** background music support, voices and sound effects.
 ** Following low level backends are supported: OSS and SDL.
 **
-** @todo adpcm file format support for sound effects
-** @todo better separation of low and high level, assembler mixing
-** support.
-** @todo Streaming support of ogg/mp3 files.
-**
 ** @see sound.h @see sound.cpp
 ** @see script_sound.cpp @see sound_id.cpp @see sound_server.cpp
 ** @see unitsound.cpp
@@ -167,15 +162,6 @@
 #include <ctype.h>
 #include <sstream>
 
-#ifdef USE_BEOS
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
-extern void beos_init(int argc, char **argv);
-
-#endif
-
 #ifndef _MSC_VER
 #include <unistd.h>
 #endif
@@ -210,6 +196,7 @@ extern int getopt(int argc, char *const *argv, const char *opt);
 #include "network.h"
 #include "netconnect.h"
 #include "ai.h"
+#include "missile.h"
 #include "commands.h"
 #include "results.h"
 #include "editor.h"
@@ -228,15 +215,17 @@ extern void CreateUserDirectories(void);
 
 TitleScreen **TitleScreens;          /// Title screens to show at startup
 std::string StratagusLibPath;        /// Path for data directory
-char LocalPlayerName[16];            /// Name of local player
+std::string LocalPlayerName;         /// Name of local player
 
 	/// Name, Version, Copyright
-char NameLine[] =
+std::string NameLine =
 	"Bos Wars V" VERSION ", (c) 1998-2007 by the Bos Wars and Stratagus Project.";
 
 static char *MapName;                /// Filename of the map to load
 std::string CompileOptions;          /// Compile options.
 
+static std::vector<gcn::Container *> Containers;
+
 /*----------------------------------------------------------------------------
 --  Speedups FIXME: Move to some other more logic place
 ----------------------------------------------------------------------------*/
@@ -245,8 +234,6 @@ int SpeedResourcesHarvest[MaxCosts]; /// speed factor for harvesting resources
 int SpeedResourcesReturn[MaxCosts];  /// speed factor for returning resources
 int SpeedBuild = 1;                  /// speed factor for building
 int SpeedTrain = 1;                  /// speed factor for training
-int SpeedUpgrade = 1;                /// speed factor for upgrading
-int SpeedResearch = 1;               /// speed factor for researching
 
 /*============================================================================
 ==  DISPLAY
@@ -452,6 +439,7 @@ void PreMenuSetup(void)
 	UI.Load();
 }
 
+extern void InitDefinedVariables();
 /**
 **  Run the guichan main menus loop.
 **
@@ -478,16 +466,13 @@ static int MenuLoop(const char *filename, CMap *map)
 	// FIXME delete this when switching to full guichan GUI
 	LibraryFileName("scripts/guichan.lua", buf, sizeof(buf));
 	status = LuaLoadFile(buf);
-	if (status == 0) {
-		CleanModules();
-	}
 
-	freeGuichan();
+	// We clean up later in Exit
+
 	return status;
 }
 
 extern void CleanMissiles();
-extern void InitDefinedVariables();
 extern void CleanTriggers();
 /**
 **  Cleanup game.
@@ -512,6 +497,8 @@ void CleanGame(void)
 	CleanReplayLog();
 	FreeVisionTable();
 	FreeAStar();
+	CursorBuilding = NULL;
+	UnitUnderCursor = NULL;
 }
 
 static void ExpandPath(char *newpath, const char *path)
@@ -532,6 +519,7 @@ void StartMap(const std::string &filename, bool clean)
 
 	gcn::Widget *oldTop = Gui->getTop();
 	gcn::Container *container = new gcn::Container();
+	Containers.push_back(container);
 	container->setDimension(gcn::Rectangle(0, 0, Video.Width, Video.Height));
 	container->setOpaque(false);
 	Gui->setTop(container);
@@ -548,8 +536,6 @@ void StartMap(const std::string &filename, bool clean)
 
 	CreateGame(filename, &Map);
 
-	UI.StatusLine.Set(NameLine);
-
 	//  Play the game.
 	GameMainLoop();
 
@@ -562,6 +548,7 @@ void StartMap(const std::string &filename, bool clean)
 	SetDefaultTextColors(nc, rc);
 
 	Gui->setTop(oldTop);
+	Containers.erase(std::find(Containers.begin(), Containers.end(), container));
 	delete container;
 }
 
@@ -569,7 +556,7 @@ void StartSavedGame(const std::string &filename)
 {
 	char path[512];
 
-	SaveGameLoading = 1;
+	SaveGameLoading = true;
 	CleanPlayers();
 	ExpandPath(path, filename.c_str());
 	LoadGame(path);
@@ -656,7 +643,7 @@ static void PrintHeader(void)
 	fprintf(stdout, "%s\n  written by Lutz Sammer, Fabrice Rossi, Vladi Shabanski, Patrice Fortier,\n"
 		"Jon Gabrielson, Andreas Arens, Nehal Mistry, Jimmy Salmon, and others.\n"
 		"\t(http://www.boswars.org)"
-		"\nCompile options %s", NameLine, CompileOptions.c_str());
+		"\nCompile options %s", NameLine.c_str(), CompileOptions.c_str());
 }
 
 /**
@@ -728,6 +715,16 @@ void Exit(int err)
 
 	ExitNetwork1();
 #ifdef DEBUG
+	CleanModules();
+	FreeBurningBuildingFrames();
+	FreeSounds();
+	FreeGraphics();
+	FreePlayerColors();
+	FreeButtonStyles();
+	for (size_t i = 0; i < Containers.size(); ++i) {
+		delete Containers[i];
+	}
+	freeGuichan();
 	DebugPrint("Frames %lu, Slow frames %d = %ld%%\n" _C_
 		FrameCounter _C_ SlowFrameCounter _C_
 		(SlowFrameCounter * 100) / (FrameCounter ? FrameCounter : 1));
@@ -805,13 +802,6 @@ int main(int argc, char **argv)
 		""
 	;
 
-#ifdef USE_BEOS
-	//
-	//  Parse arguments for BeOS
-	//
-	beos_init(argc, argv);
-#endif
-
 	//
 	//  Setup some defaults.
 	//
@@ -835,14 +825,14 @@ int main(int argc, char **argv)
 	EditorStartFile = "scripts/editor.lua";
 
 	//  Default player name to username on unix systems.
-	memset(LocalPlayerName, 0, sizeof(LocalPlayerName));
+	LocalPlayerName.clear();
 #ifdef USE_WIN32
-	strcpy_s(LocalPlayerName, sizeof(LocalPlayerName), "Anonymous");
+	LocalPlayerName = "Anonymous";
 #else
 	if (getenv("USER")) {
-		strncpy(LocalPlayerName, getenv("USER"), sizeof(LocalPlayerName) - 1);
+		LocalPlayerName = getenv("USER");
 	} else {
-		strcpy_s(LocalPlayerName, sizeof(LocalPlayerName), "Anonymous");
+		LocalPlayerName = "Anonymous";
 	}
 #endif
 
@@ -884,7 +874,7 @@ int main(int argc, char **argv)
 				NetworkArg = optarg;
 				continue;
 			case 'N':
-				strncpy_s(LocalPlayerName, sizeof(LocalPlayerName), optarg, _TRUNCATE);
+				LocalPlayerName = optarg;
 				continue;
 			case 's':
 				AiSleepCycles = atoi(optarg);
@@ -988,6 +978,7 @@ int main(int argc, char **argv)
 
 	main1(argc, argv);
 
+	Exit(0);
 	return 0;
 }
 
