@@ -9,7 +9,7 @@
 //
 /**@name botpanel.cpp - The bottom panel. */
 //
-//      (c) Copyright 1999-2006 by Lutz Sammer, Vladi Belperchinov-Shabanski,
+//      (c) Copyright 1999-2007 by Lutz Sammer, Vladi Belperchinov-Shabanski,
 //                                 and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: botpanel.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: botpanel.cpp 8830 2007-05-27 21:36:22Z jim4 $
 
 //@{
 
@@ -49,7 +49,6 @@
 #include "ui.h"
 #include "player.h"
 #include "spells.h"
-#include "depend.h"
 #include "sound.h"
 #include "map.h"
 #include "commands.h"
@@ -82,6 +81,7 @@ void InitButtons(void)
 	for (int z = 0; z < (int)UnitButtonTable.size(); ++z) {
 		UnitButtonTable[z]->Icon.Load();
 	}
+	delete[] CurrentButtons;
 	CurrentButtons = NULL;
 }
 
@@ -96,7 +96,7 @@ int AddButton(int pos, int level, const std::string &icon_ident,
 	ButtonCmd action, const std::string &value, const ButtonCheckFunc func,
 	const std::string &allow, int key, const std::string &hint, const std::string &umask)
 {
-	char buf[2048];
+	std::string buf;
 	ButtonAction *ba;
 
 	ba = new ButtonAction;
@@ -123,12 +123,6 @@ int AddButton(int pos, int level, const std::string &icon_ident,
 			case ButtonTrain:
 				ba->Value = UnitTypeIdByIdent(value);
 				break;
-			case ButtonResearch:
-				ba->Value = UpgradeIdByIdent(value);
-				break;
-			case ButtonUpgradeTo:
-				ba->Value = UnitTypeIdByIdent(value);
-				break;
 			case ButtonBuild:
 				ba->Value = UnitTypeIdByIdent(value);
 				break;
@@ -147,14 +141,9 @@ int AddButton(int pos, int level, const std::string &icon_ident,
 	ba->Hint = hint;
 	// FIXME: here should be added costs to the hint
 	// FIXME: johns: show should be nice done?
-	if (umask[0] == '*') {
-		strcpy_s(buf, sizeof(buf), umask.c_str());
-	} else {
-		sprintf(buf, ",%s,", umask.c_str());
-	}
-	ba->UnitMask = buf;
+	ba->UnitMask = umask;
 	UnitButtonTable.push_back(ba);
-	// FIXME: check if already initited
+	// FIXME: check if already initialized
 	//Assert(ba->Icon.Icon != NULL);// just checks, that's why at the end
 	return 1;
 }
@@ -225,7 +214,6 @@ static int GetButtonStatus(const ButtonAction *button)
 			action = UnitActionPatrol;
 			break;
 		case ButtonHarvest:
-		case ButtonReturn:
 			action = UnitActionResource;
 			break;
 		default:
@@ -371,24 +359,16 @@ void CButtonPanel::Draw(void)
 */
 void UpdateStatusLineForButton(const ButtonAction *button)
 {
-	const CUnitStats *stats;
-
 	Assert(button);
 	UI.StatusLine.Set(button->Hint);
 
 	switch (button->Action) {
 		case ButtonBuild:
 		case ButtonTrain:
-		case ButtonUpgradeTo:
-			// FIXME: store pointer in button table!
-			stats = &UnitTypes[button->Value]->Stats[ThisPlayer->Index];
-			SetCosts(0, UnitTypes[button->Value]->Demand, stats->Costs);
-			break;
-		case ButtonResearch:
-			SetCosts(0, 0, AllUpgrades[button->Value]->Costs);
+			SetCosts(0, UnitTypes[button->Value]->ProductionCosts);
 			break;
 		case ButtonSpellCast:
-			SetCosts(SpellTypeTable[button->Value]->ManaCost, 0, NULL);
+			SetCosts(SpellTypeTable[button->Value]->ManaCost, NULL);
 			break;
 		default:
 			ClearCosts();
@@ -409,7 +389,6 @@ void UpdateStatusLineForButton(const ButtonAction *button)
 **  @return 1 if button is allowed, 0 else.
 **
 **  @todo FIXME: better check. (dependancy, resource, ...)
-**  @todo FIXME: make difference with impossible and not yet researched.
 */
 static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 {
@@ -431,6 +410,7 @@ static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 		case ButtonStandGround:
 		case ButtonButton:
 		case ButtonMove:
+		case ButtonHarvest:
 			res = true;
 			break;
 		case ButtonRepair:
@@ -439,22 +419,6 @@ static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 		case ButtonPatrol:
 			res = CanMove(unit);
 			break;
-		case ButtonHarvest:
-			if (!unit->CurrentResource ||
-					!(unit->ResourcesHeld > 0 && !unit->Type->ResInfo[unit->CurrentResource]->LoseResources) ||
-					(unit->ResourcesHeld != unit->Type->ResInfo[unit->CurrentResource]->ResourceCapacity &&
-						unit->Type->ResInfo[unit->CurrentResource]->LoseResources)) {
-				res = true;
-			}
-			break;
-		case ButtonReturn:
-			if (!(!unit->CurrentResource ||
-					!(unit->ResourcesHeld > 0 && !unit->Type->ResInfo[unit->CurrentResource]->LoseResources) ||
-					(unit->ResourcesHeld != unit->Type->ResInfo[unit->CurrentResource]->ResourceCapacity &&
-						unit->Type->ResInfo[unit->CurrentResource]->LoseResources))) {
-				res = true;
-			}
-			break;
 		case ButtonAttack:
 			res = ButtonCheckAttack(unit, buttonaction);
 			break;
@@ -464,19 +428,8 @@ static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 			}
 			break;
 		case ButtonTrain:
-			// Check if building queue is enabled
-			if (!EnableTrainingQueue &&
-					unit->Orders[0]->Action == UnitActionTrain) {
-				break;
-			}
-			// FALL THROUGH
-		case ButtonUpgradeTo:
-		case ButtonResearch:
 		case ButtonBuild:
-			res = CheckDependByIdent(unit->Player, buttonaction->ValueStr);
-			if (res && !strncmp(buttonaction->ValueStr.c_str(), "upgrade-", 8)) {
-				res = UpgradeIdentAllowed(unit->Player, buttonaction->ValueStr) == 'A';
-			}
+			res = true;
 			break;
 		case ButtonSpellCast:
 			res = SpellIsAvailable(unit->Player, buttonaction->Value);
@@ -487,10 +440,6 @@ static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 		case ButtonCancel:
 			res = true;
 			break;
-		case ButtonCancelUpgrade:
-			res = unit->Orders[0]->Action == UnitActionUpgradeTo ||
-				unit->Orders[0]->Action == UnitActionResearch;
-			break;
 		case ButtonCancelTrain:
 			res = unit->Orders[0]->Action == UnitActionTrain;
 			break;
@@ -517,7 +466,7 @@ static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 */
 static ButtonAction *UpdateButtonPanelMultipleUnits(void)
 {
-	char unit_ident[128];
+	std::string unit_ident;
 	int z;
 	ButtonAction *res;
 	bool allow;         // button is available for at least 1 unit.
@@ -528,7 +477,7 @@ static ButtonAction *UpdateButtonPanelMultipleUnits(void)
 	}
 
 	// FIXME: hardcoded race name
-	sprintf(unit_ident,	",elites-group,");
+	unit_ident = ",elites-group,";
 
 	for (z = 0; z < (int)UnitButtonTable.size(); ++z) {
 		if (UnitButtonTable[z]->Level != CurrentButtonLevel) {
@@ -537,7 +486,7 @@ static ButtonAction *UpdateButtonPanelMultipleUnits(void)
 
 		// any unit or unit in list
 		if (UnitButtonTable[z]->UnitMask[0] != '*' &&
-				!strstr(UnitButtonTable[z]->UnitMask.c_str(), unit_ident)) {
+				UnitButtonTable[z]->UnitMask.find(unit_ident) == std::string::npos) {
 			continue;
 		}
 		allow = true;
@@ -572,7 +521,7 @@ static ButtonAction *UpdateButtonPanelMultipleUnits(void)
 static ButtonAction *UpdateButtonPanelSingleUnit(const CUnit *unit)
 {
 	int allow;
-	char unit_ident[128];
+	std::string unit_ident;
 	ButtonAction *buttonaction;
 	ButtonAction *res;
 	int z;
@@ -589,15 +538,9 @@ static ButtonAction *UpdateButtonPanelSingleUnit(const CUnit *unit)
 	//
 	if (unit->Orders[0]->Action == UnitActionBuilt) {
 		// Trick 17 to get the cancel-build button
-		strcpy_s(unit_ident, sizeof(unit_ident), ",cancel-build,");
-	} else if (unit->Orders[0]->Action == UnitActionUpgradeTo) {
-		// Trick 17 to get the cancel-upgrade button
-		strcpy_s(unit_ident, sizeof(unit_ident), ",cancel-upgrade,");
-	} else if (unit->Orders[0]->Action == UnitActionResearch) {
-		// Trick 17 to get the cancel-upgrade button
-		strcpy_s(unit_ident, sizeof(unit_ident), ",cancel-upgrade,");
+		unit_ident = ",cancel-build,";
 	} else {
-		sprintf(unit_ident, ",%s,", unit->Type->Ident.c_str());
+		unit_ident = "," + unit->Type->Ident + ",";
 	}
 
 	for (z = 0; z < (int)UnitButtonTable.size(); ++z) {
@@ -613,7 +556,7 @@ static ButtonAction *UpdateButtonPanelSingleUnit(const CUnit *unit)
 
 		// any unit or unit in list
 		if (buttonaction->UnitMask[0] != '*' &&
-				!strstr(buttonaction->UnitMask.c_str(), unit_ident)) {
+				buttonaction->UnitMask.find(unit_ident) == std::string::npos) {
 			continue;
 		}
 		allow = IsButtonAllowed(unit, buttonaction);
@@ -787,12 +730,6 @@ void CButtonPanel::DoClicked(int button)
 			UI.ButtonPanel.Update();
 			UI.StatusLine.Set(_("Select Target"));
 			break;
-		case ButtonReturn:
-			for (i = 0; i < NumSelected; ++i) {
-				SendCommandReturnGoods(Selected[i], NoUnitP,
-					!(KeyModifiers & ModifierShift));
-			}
-			break;
 		case ButtonStop:
 			for (i = 0; i < NumSelected; ++i) {
 				SendCommandStopUnit(Selected[i]);
@@ -810,14 +747,6 @@ void CButtonPanel::DoClicked(int button)
 			break;
 
 		case ButtonCancel:
-		case ButtonCancelUpgrade:
-			if (NumSelected == 1) {
-				if (Selected[0]->Orders[0]->Action == UnitActionUpgradeTo) {
-					SendCommandCancelUpgradeTo(Selected[0]);
-				} else if (Selected[0]->Orders[0]->Action == UnitActionResearch) {
-					SendCommandCancelResearch(Selected[0]);
-				}
-			}
 			UI.StatusLine.Clear();
 			ClearCosts();
 			CurrentButtonLevel = 0;
@@ -847,14 +776,12 @@ void CButtonPanel::DoClicked(int button)
 		case ButtonBuild:
 			// FIXME: store pointer in button table!
 			type = UnitTypes[CurrentButtons[button].Value];
-			if (!Selected[0]->Player->CheckUnitType(type)) {
-				UI.StatusLine.Set(_("Select Location"));
-				ClearCosts();
-				CursorBuilding = type;
-				// FIXME: check is this =9 necessary?
-				CurrentButtonLevel = 9; // level 9 is cancel-only
-				UI.ButtonPanel.Update();
-			}
+			UI.StatusLine.Set(_("Select Location"));
+			ClearCosts();
+			CursorBuilding = type;
+			// FIXME: check is this =9 necessary?
+			CurrentButtonLevel = 9; // level 9 is cancel-only
+			UI.ButtonPanel.Update();
 			break;
 
 		case ButtonTrain:
@@ -863,42 +790,13 @@ void CButtonPanel::DoClicked(int button)
 			// FIXME: Johns: I want to place commands in queue, even if not
 			// FIXME:        enough resources are available.
 			// FIXME: training queue full check is not correct for network.
-			// FIXME: this can be correct written, with a little more code.
-			if (Selected[0]->Orders[0]->Action == UnitActionTrain &&
-					!EnableTrainingQueue) {
-				Selected[0]->Player->Notify(NotifyYellow, Selected[0]->X,
-					Selected[0]->Y, _("Unit training queue is full"));
-			} else if (Selected[0]->Player->CheckLimits(type) >= 0 &&
-					!Selected[0]->Player->CheckUnitType(type)) {
-				//PlayerSubUnitType(player,type);
+			if (Selected[0]->Player->CheckLimits(type) >= 0) {
 				SendCommandTrainUnit(Selected[0], type,
 					!(KeyModifiers & ModifierShift));
 				UI.StatusLine.Clear();
 				ClearCosts();
 			}
 			break;
-
-		case ButtonUpgradeTo:
-			// FIXME: store pointer in button table!
-			type = UnitTypes[CurrentButtons[button].Value];
-			if (!Selected[0]->Player->CheckUnitType(type)) {
-				//PlayerSubUnitType(player,type);
-				SendCommandUpgradeTo(Selected[0],type,
-					!(KeyModifiers & ModifierShift));
-				UI.StatusLine.Clear();
-				ClearCosts();
-			}
-			break;
-		case ButtonResearch:
-			i = CurrentButtons[button].Value;
-			if (!Selected[0]->Player->CheckCosts(AllUpgrades[i]->Costs)) {
-				//PlayerSubCosts(player,Upgrades[i].Costs);
-				SendCommandResearch(Selected[0], AllUpgrades[i],
-					!(KeyModifiers & ModifierShift));
-				UI.StatusLine.Clear();
-				ClearCosts();
-			}
-			break;
 	}
 }
 
