@@ -9,7 +9,7 @@
 //
 /**@name mainscr.cpp - The main screen. */
 //
-//      (c) Copyright 1998-2006 by Lutz Sammer, Valery Shchedrin, and
+//      (c) Copyright 1998-2007 by Lutz Sammer, Valery Shchedrin, and
 //                                 Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: mainscr.cpp 8559 2007-02-10 17:25:47Z jim4 $
+//      $Id: mainscr.cpp 8945 2007-06-26 04:44:11Z jim4 $
 
 //@{
 
@@ -267,11 +267,15 @@ UStrInt GetComponent(const CUnit *unit, int index, EnumVariable e, int t)
 			break;
 		case VariableName:
 			if (index == GIVERESOURCE_INDEX) {
+				int i;
+				for (i = 0; i < MaxCosts; ++i) {
+					if (unit->ResourcesHeld[i] != 0) {
+						break;
+					}
+				}
+				Assert(i != MaxCosts);
 				val.type = USTRINT_STR;
-				val.s = DefaultResourceNames[unit->Type->GivesResource];
-			} else if (index == CARRYRESOURCE_INDEX) {
-				val.type = USTRINT_STR;
-				val.s = DefaultResourceNames[unit->CurrentResource];
+				val.s = DefaultResourceNames[i].c_str();
 			} else {
 				val.type = USTRINT_STR;
 				val.s = UnitTypeVar.VariableName[index];
@@ -282,7 +286,7 @@ UStrInt GetComponent(const CUnit *unit, int index, EnumVariable e, int t)
 }
 
 /**
-**  Get unit from an unit depending of the relation.
+**  Get unit from a unit depending of the relation.
 **
 **  @param unit  unit reference.
 **  @param e     relation with unit.
@@ -647,36 +651,6 @@ static void DrawUnitInfo(CUnit *unit)
 			}
 			return;
 		}
-
-		//
-		//  Building upgrading to better type.
-		//
-		if (unit->Orders[0]->Action == UnitActionUpgradeTo) {
-			if (UI.UpgradingButton) {
-				unit->Orders[0]->Type->Icon.Icon->DrawUnitIcon(unit->Player,
-					UI.UpgradingButton->Style,
-					(ButtonAreaUnderCursor == ButtonAreaUpgrading &&
-						ButtonUnderCursor == 0) ?
-						(IconActive | (MouseButtons & LeftButton)) : 0,
-					UI.UpgradingButton->X, UI.UpgradingButton->Y, "");
-			}
-			return;
-		}
-
-		//
-		//  Building research new technology.
-		//
-		if (unit->Orders[0]->Action == UnitActionResearch) {
-			if (UI.ResearchingButton) {
-				unit->Data.Research.Upgrade->Icon->DrawUnitIcon(unit->Player,
-					UI.ResearchingButton->Style,
-					(ButtonAreaUnderCursor == ButtonAreaResearching &&
-						ButtonUnderCursor == 0) ?
-						(IconActive | (MouseButtons & LeftButton)) : 0,
-					UI.ResearchingButton->X, UI.ResearchingButton->Y, "");
-			}
-			return;
-		}
 	}
 
 	//
@@ -717,41 +691,30 @@ static void DrawUnitInfo(CUnit *unit)
 */
 void DrawResources(void)
 {
+	const char *names[]={_("Energy"), _("Magma")};
 	char tmp[128];
+	int totalproduction = 0;
+	int totalrequested = 0;
 	int i;
-	int v;
+	int p = 0;
 
-	// Draw all icons of resource.
-	for (i = 0; i <= ScoreCost; ++i) {
-		if (UI.Resources[i].G) {
-			UI.Resources[i].G->DrawFrameClip(UI.Resources[i].IconFrame,
-				UI.Resources[i].IconX, UI.Resources[i].IconY);
-		}
-	}
 	for (i = 0; i < MaxCosts; ++i) {
-		if (UI.Resources[i].TextX != -1) {
-			v = ThisPlayer->Resources[i];
-			VideoDrawNumber(UI.Resources[i].TextX,
-				UI.Resources[i].TextY + (v > 99999) * 3,
-				v > 99999 ? SmallFont : GameFont, v);
-		}
+		sprintf(tmp, "%s:+%d-%d %d/%d",
+			names[i],
+			ThisPlayer->ProductionRate[i],
+			ThisPlayer->RequestedUtilizationRate[i],
+			ThisPlayer->StoredResources[i] / CYCLES_PER_SECOND / 100,
+			ThisPlayer->StorageCapacity[i] / CYCLES_PER_SECOND / 100);
+		VideoDrawText(40 +  176 * i, 1, GameFont, tmp);
+		totalproduction += ThisPlayer->ProductionRate[i];
+		totalrequested += ThisPlayer->RequestedUtilizationRate[i];
 	}
-	if (UI.Resources[FoodCost].TextX != -1) {
-		sprintf(tmp, "%d/%d", ThisPlayer->Demand, ThisPlayer->Supply);
-		if (ThisPlayer->Supply < ThisPlayer->Demand) {
-			VideoDrawReverseText(UI.Resources[FoodCost].TextX,
-				UI.Resources[FoodCost].TextY, GameFont, tmp);
-		} else {
-			VideoDrawText(UI.Resources[FoodCost].TextX,
-				UI.Resources[FoodCost].TextY, GameFont, tmp);
-		}
-	}
-	if (UI.Resources[ScoreCost].TextX != -1) {
-		v = ThisPlayer->Score;
-		VideoDrawNumber(UI.Resources[ScoreCost].TextX,
-			UI.Resources[ScoreCost].TextY + (v > 99999) * 3,
-			v > 99999 ? SmallFont : GameFont, v);
+
+	if (totalproduction + totalrequested) {
+		p = 100 - abs(totalproduction - totalrequested) * 100 / (totalproduction + totalrequested);
 	}
+	sprintf(tmp, "%d%%", p);
+	VideoDrawText(400, 1, GameFont, tmp);
 }
 
 /*----------------------------------------------------------------------------
@@ -774,7 +737,7 @@ class MessagesDisplay
 	int  MessagesScrollY;
 	unsigned long MessagesFrameTimeout;       /// Frame to expire message
 
-	static const int MESSAGES_TIMEOUT = (FRAMES_PER_SECOND * 5); /// Message timeout 5 seconds
+	static const int MESSAGES_TIMEOUT = 5;    /// Message timeout 5 seconds
 protected:
 	void ShiftMessages();
 	void AddMessage(const char *msg);
@@ -811,10 +774,11 @@ void MessagesDisplay::UpdateMessages()
 	}
 
 	// Scroll/remove old message line
-	if (MessagesFrameTimeout < FrameCounter) {
+	unsigned long ticks = GetTicks();
+	if (MessagesFrameTimeout < ticks) {
 		++MessagesScrollY;
 		if (MessagesScrollY == GameFont->Height() + 1) {
-			MessagesFrameTimeout = FrameCounter + MESSAGES_TIMEOUT - MessagesScrollY;
+			MessagesFrameTimeout = ticks + MESSAGES_TIMEOUT * 1000;
 			MessagesScrollY = 0;
 			ShiftMessages();
 		}
@@ -828,6 +792,13 @@ void MessagesDisplay::UpdateMessages()
 */
 void MessagesDisplay::DrawMessages()
 {
+	// background so the text is easier to read
+	if (MessagesCount) {
+		Uint32 color = Video.MapRGB(TheScreen->format, 38, 38, 78);
+		Video.FillTransRectangleClip(color, UI.MapArea.X + 8, UI.MapArea.Y + 8,
+			UI.MapArea.EndX - UI.MapArea.X - 16, MessagesCount * (GameFont->Height() + 1) - MessagesScrollY, 0x80);
+	}
+
 	// Draw message line(s)
 	for (int z = 0; z < MessagesCount; ++z) {
 		if (z == 0) {
@@ -857,15 +828,16 @@ void MessagesDisplay::AddMessage(const char *msg)
 	char *ptr;
 	char *message;
 	char *next;
+	unsigned long ticks = GetTicks();
 
 	if (!MessagesCount) {
-		MessagesFrameTimeout = FrameCounter + MESSAGES_TIMEOUT;
+		MessagesFrameTimeout = ticks + MESSAGES_TIMEOUT * 1000;
 	}
 
 	if (MessagesCount == MESSAGES_MAX) {
 		// Out of space to store messages, can't scroll smoothly
 		ShiftMessages();
-		MessagesFrameTimeout = FrameCounter + MESSAGES_TIMEOUT;
+		MessagesFrameTimeout = ticks + MESSAGES_TIMEOUT * 1000;
 		MessagesScrollY = 0;
 	}
 
@@ -1119,36 +1091,33 @@ void CStatusLine::Clear(void)
 --  COSTS
 ----------------------------------------------------------------------------*/
 
-static int CostsMana;                        /// mana cost to display in status line
-static int Costs[MaxCosts + 1];              /// costs to display in status line
+static int CostsMana;                    /// mana cost to display in status line
+static int Costs[MaxCosts];              /// costs to display in status line
 
 /**
 **  Draw costs in status line.
 **
 **  @todo FIXME : make DrawCosts more configurable.
-**  @todo FIXME : 'time' resource should be shown too.
 **  @todo FIXME : remove hardcoded image for mana.
-**
-**  @internal MaxCost == FoodCost.
 */
 void DrawCosts(void)
 {
 	int x = UI.StatusLine.TextX + 268;
 	if (CostsMana) {
 		// FIXME: hardcoded image!!!
-		UI.Resources[GoldCost].G->DrawFrameClip(3, x, UI.StatusLine.TextY);
+		UI.Resources[EnergyCost].G->DrawFrameClip(3, x, UI.StatusLine.TextY);
 
 		VideoDrawNumber(x + 15, UI.StatusLine.TextY, GameFont, CostsMana);
 		x += 60;
 	}
 
-	for (int i = 1; i <= MaxCosts; ++i) {
+	for (int i = 0; i < MaxCosts; ++i) {
 		if (Costs[i]) {
 			if (UI.Resources[i].G) {
 				UI.Resources[i].G->DrawFrameClip(UI.Resources[i].IconFrame,
 					x, UI.StatusLine.TextY);
 			}
-			VideoDrawNumber(x + 15, UI.StatusLine.TextY, GameFont,Costs[i]);
+			VideoDrawNumber(x + 15, UI.StatusLine.TextY, GameFont, Costs[i] / CYCLES_PER_SECOND);
 			x += 60;
 			if (x > Video.Width - 60) {
 				break;
@@ -1161,18 +1130,16 @@ void DrawCosts(void)
 **  Set costs in status line.
 **
 **  @param mana   Mana costs.
-**  @param food   Food costs.
 **  @param costs  Resource costs, NULL pointer if all are zero.
 */
-void SetCosts(int mana, int food, const int *costs)
+void SetCosts(int mana, const int *costs)
 {
 	CostsMana = mana;
 	if (costs) {
 		memcpy(Costs, costs, MaxCosts * sizeof(*costs));
 	} else {
-		memset(Costs, 0, sizeof (Costs));
+		memset(Costs, 0, sizeof(Costs));
 	}
-	Costs[FoodCost] = food;
 }
 
 /**
@@ -1180,7 +1147,7 @@ void SetCosts(int mana, int food, const int *costs)
 */
 void ClearCosts(void)
 {
-	SetCosts(0, 0, NULL);
+	SetCosts(0, NULL);
 }
 
 /*----------------------------------------------------------------------------
@@ -1249,9 +1216,7 @@ void CInfoPanel::Draw(void)
 					ThisPlayer->IsAllied(Selected[0]) ||
 					ReplayRevealMap) {
 				if (Selected[0]->Orders[0]->Action == UnitActionBuilt ||
-					Selected[0]->Orders[0]->Action == UnitActionResearch ||
-					Selected[0]->Orders[0]->Action == UnitActionUpgradeTo ||
-					Selected[0]->Orders[0]->Action == UnitActionTrain) {
+						Selected[0]->Orders[0]->Action == UnitActionTrain) {
 					i = 3;
 				} else if (Selected[0]->Stats->Variables[MANA_INDEX].Max) {
 					i = 2;
