@@ -9,7 +9,7 @@
 //
 /**@name mouse.cpp - The mouse handling. */
 //
-//      (c) Copyright 1998-2006 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: mouse.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: mouse.cpp 8964 2007-06-30 20:17:02Z jim4 $
 
 //@{
 
@@ -113,7 +113,6 @@ void DoRightButton(int sx, int sy)
 	int action;             // default action for unit.
 	int acknowledged;       // to play sound
 	int flush;              // append command to old command.
-	int res;                // resource id for harvester.
 	unsigned int spellnum;  // spell id for spell cast
 
 	// No unit selected
@@ -143,15 +142,11 @@ void DoRightButton(int sx, int sy)
 	if (!CanSelectMultipleUnits(Selected[0]->Player)) {
 		unit = Selected[0];
 		if (unit->Player->Index != PlayerNumNeutral || dest == NULL ||
-			!(dest->Player == ThisPlayer || dest->IsTeamed(ThisPlayer))) {
-			return ;
+				!(dest->Player == ThisPlayer || dest->IsTeamed(ThisPlayer))) {
+			return;
 		}
 		// tell to go and harvest from a unit
-		if (dest->Type->Harvester &&
-				(res = unit->Type->GivesResource) &&
-				dest->Type->ResInfo[res] &&
-				dest->ResourcesHeld < dest->Type->ResInfo[res]->ResourceCapacity &&
-				unit->Type->CanHarvest) {
+		if (dest->Type->Harvester && unit->Type->CanHarvestFrom) {
 			unit->Blink = 4;
 			SendCommandResource(dest, unit, flush);
 			return;
@@ -244,19 +239,8 @@ void DoRightButton(int sx, int sy)
 		if (action == MouseActionHarvest) {
 			if (type->Harvester) {
 				if (dest) {
-					// Return a loaded harvester to deposit
-					if (unit->ResourcesHeld > 0 &&
-							dest->Type->CanStore[unit->CurrentResource] &&
-							dest->Player == unit->Player) {
-						dest->Blink = 4;
-						SendCommandReturnGoods(unit, dest, flush);
-						continue;
-					}
 					// Go and harvest from a unit
-					if ((res = dest->Type->GivesResource) &&
-							type->ResInfo[res] &&
-							unit->ResourcesHeld < type->ResInfo[res]->ResourceCapacity &&
-							dest->Type->CanHarvest &&
+					if (dest->Type->CanHarvestFrom && type->Harvester &&
 							(dest->Player == unit->Player ||
 								(dest->Player->Index == PlayerNumNeutral))) {
 						dest->Blink = 4;
@@ -348,19 +332,8 @@ void DoRightButton(int sx, int sy)
 
 		// Manage harvester from the destination side.
 		if (dest && dest->Type->Harvester) {
-			// tell to return a loaded harvester to deposit
-			if (dest->ResourcesHeld > 0 &&
-					type->CanStore[dest->CurrentResource] &&
-					dest->Player == unit->Player) {
-				dest->Blink = 4;
-				SendCommandReturnGoods(dest, unit, flush);
-				continue;
-			}
 			// tell to go and harvest from a building
-			if ((res = type->GivesResource) &&
-					dest->Type->ResInfo[res] &&
-					dest->ResourcesHeld < dest->Type->ResInfo[res]->ResourceCapacity &&
-					type->CanHarvest &&
+			if (type->CanHarvestFrom && dest->Type->Harvester &&
 					dest->Player == unit->Player) {
 				unit->Blink = 4;
 				SendCommandResource(dest, unit, flush);
@@ -371,7 +344,7 @@ void DoRightButton(int sx, int sy)
 		// Manage new order.
 		if (!CanMove(unit)) {
 			// Go and harvest from a unit
-			if (dest && dest->Type->GivesResource && dest->Type->CanHarvest &&
+			if (dest && dest->Type->CanHarvestFrom &&
 					(dest->Player == unit->Player || dest->Player->Index == PlayerNumNeutral)) {
 				dest->Blink = 4;
 				SendCommandResource(unit, dest, flush);
@@ -519,20 +492,6 @@ static void HandleMouseOn(int x, int y)
 					}
 				}
 			}
-		} else if (Selected[0]->Orders[0]->Action == UnitActionUpgradeTo) {
-			if (OnButton(x, y, UI.UpgradingButton)) {
-				ButtonAreaUnderCursor = ButtonAreaUpgrading;
-				ButtonUnderCursor = 0;
-				CursorOn = CursorOnButton;
-				return;
-			}
-		} else if (Selected[0]->Orders[0]->Action == UnitActionResearch) {
-			if (OnButton(x, y, UI.ResearchingButton)) {
-				ButtonAreaUnderCursor = ButtonAreaResearching;
-				ButtonUnderCursor = 0;
-				CursorOn = CursorOnButton;
-				return;
-			}
 		}
 	}
 	if (NumSelected == 1) {
@@ -779,7 +738,7 @@ void UIHandleMouseMove(int x, int y)
 		const CViewport *vp = UI.MouseViewport;
 		if (Map.IsFieldExplored(ThisPlayer, vp->Viewport2MapX(x), vp->Viewport2MapY(y)) ||
 				ReplayRevealMap) {
-			UnitUnderCursor = UnitOnScreen(NULL, x - vp->X + vp->MapX * TileSizeX + vp->OffsetX,
+			UnitUnderCursor = UnitOnScreen(x - vp->X + vp->MapX * TileSizeX + vp->OffsetX,
 				y - vp->Y + vp->MapY * TileSizeY + vp->OffsetY);
 		}
 	} else if (CursorOn == CursorOnMinimap) {
@@ -1039,7 +998,6 @@ static int SendResource(int sx, int sy)
 	int i;
 	int x;
 	int y;
-	int res;
 	CUnit *unit;
 	CUnit *dest;
 	int ret;
@@ -1052,11 +1010,7 @@ static int SendResource(int sx, int sy)
 	for (i = 0; i < NumSelected; ++i) {
 		unit = Selected[i];
 		if (unit->Type->Harvester) {
-			if (dest &&
-					(res = dest->Type->GivesResource) &&
-					unit->Type->ResInfo[res] &&
-					unit->ResourcesHeld < unit->Type->ResInfo[res]->ResourceCapacity &&
-					dest->Type->CanHarvest &&
+			if (dest && dest->Type->CanHarvestFrom && unit->Type->Harvester &&
 					(dest->Player == unit->Player ||
 						(dest->Player->Index == PlayerMax - 1))) {
 				dest->Blink = 4;
@@ -1066,7 +1020,7 @@ static int SendResource(int sx, int sy)
 			}
 		}
 		if (!CanMove(unit)) {
-			if (dest && dest->Type->GivesResource && dest->Type->CanHarvest) {
+			if (dest && dest->Type->CanHarvestFrom) {
 				dest->Blink = 4;
 				SendCommandResource(unit, dest, !(KeyModifiers & ModifierShift));
 				ret = 1;
@@ -1407,7 +1361,7 @@ void UIHandleButtonDown(unsigned button)
 		// to redraw the cursor immediately (and avoid up to 1 sec delay
 		if (CursorBuilding) {
 			// Possible Selected[0] was removed from map
-			// need to make sure there is an unit to build
+			// need to make sure there is a unit to build
 			if (Selected[0] && (MouseButtons & LeftButton) &&
 					UI.MouseViewport->IsInsideMapArea(CursorX, CursorY)) {// enter select mode
 				int explored;
@@ -1430,8 +1384,10 @@ void UIHandleButtonDown(unsigned button)
 						(explored || ReplayRevealMap)) {
 					PlayGameSound(GameSounds.PlacementSuccess.Sound,
 						MaxSampleVolume);
-					SendCommandBuildBuilding(Selected[0], x, y, CursorBuilding,
-						!(KeyModifiers & ModifierShift));
+					for (int i = 0; i < NumSelected; ++i) {
+						SendCommandBuildBuilding(Selected[i], x, y, CursorBuilding,
+							!(KeyModifiers & ModifierShift));
+					}
 					if (!(KeyModifiers & (ModifierAlt | ModifierShift))) {
 						CancelBuildingMode();
 					}
@@ -1568,30 +1524,6 @@ void UIHandleButtonDown(unsigned button)
 					}
 				}
 			//
-			//  clicked on upgrading button
-			//
-			} else if (ButtonAreaUnderCursor == ButtonAreaUpgrading) {
-				if (!GameObserve && !GamePaused &&
-						ThisPlayer->IsTeamed(Selected[0])) {
-					if (ButtonUnderCursor == 0 && NumSelected == 1) {
-						DebugPrint("Cancel upgrade %s\n" _C_
-							Selected[0]->Type->Ident.c_str());
-						SendCommandCancelUpgradeTo(Selected[0]);
-					}
-				}
-			//
-			//  clicked on researching button
-			//
-			} else if (ButtonAreaUnderCursor == ButtonAreaResearching) {
-				if (!GameObserve && !GamePaused &&
-						ThisPlayer->IsTeamed(Selected[0])) {
-					if (ButtonUnderCursor == 0 && NumSelected == 1) {
-						DebugPrint("Cancel research %s\n" _C_
-							Selected[0]->Type->Ident.c_str());
-						SendCommandCancelResearch(Selected[0]);
-					}
-				}
-			//
 			//  clicked on button panel
 			//
 			} else if (ButtonAreaUnderCursor == ButtonAreaTransporting) {
@@ -1711,7 +1643,7 @@ void UIHandleButtonUp(unsigned button)
 	// SHIFT toggles select/unselect a single unit and
 	// add the content of the rectangle to the selectection
 	// ALT takes group of unit
-	// CTRL takes all units of same type (st*rcr*ft)
+	// CTRL takes all units of same type
 	if (CursorState == CursorStateRectangle &&
 			!(MouseButtons & LeftButton)) { // leave select mode
 		int num;
@@ -1775,7 +1707,7 @@ void UIHandleButtonUp(unsigned button)
 			if (Map.IsFieldVisible(ThisPlayer,
 					UI.MouseViewport->Viewport2MapX(CursorX),
 					UI.MouseViewport->Viewport2MapY(CursorY)) || ReplayRevealMap) {
-				unit = UnitOnScreen(unit,
+				unit = UnitOnScreen(
 					CursorX - UI.MouseViewport->X + UI.MouseViewport->MapX * TileSizeX + UI.MouseViewport->OffsetX,
 					CursorY - UI.MouseViewport->Y + UI.MouseViewport->MapY * TileSizeY + UI.MouseViewport->OffsetY);
 			}
