@@ -9,7 +9,7 @@
 //
 /**@name script_ui.cpp - The ui ccl functions. */
 //
-//      (c) Copyright 1999-2006 by Lutz Sammer, Jimmy Salmon, Martin Renold
+//      (c) Copyright 1999-2007 by Lutz Sammer, Jimmy Salmon, Martin Renold
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: script_ui.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: script_ui.cpp 8839 2007-06-02 17:05:34Z jim4 $
 
 //@{
 
@@ -1132,7 +1132,6 @@ static int CclDefineButton(lua_State *l)
 {
 	char buf[64];
 	const char *value;
-	char *s1;
 	const char *s2;
 	ButtonAction ba;
 
@@ -1177,20 +1176,12 @@ static int CclDefineButton(lua_State *l)
 				ba.Action = ButtonStandGround;
 			} else if (!strcmp(value, "attack-ground")) {
 				ba.Action = ButtonAttackGround;
-			} else if (!strcmp(value, "return-goods")) {
-				ba.Action = ButtonReturn;
 			} else if (!strcmp(value, "cast-spell")) {
 				ba.Action = ButtonSpellCast;
-			} else if (!strcmp(value, "research")) {
-				ba.Action = ButtonResearch;
-			} else if (!strcmp(value, "upgrade-to")) {
-				ba.Action = ButtonUpgradeTo;
 			} else if (!strcmp(value, "unload")) {
 				ba.Action = ButtonUnload;
 			} else if (!strcmp(value, "cancel")) {
 				ba.Action = ButtonCancel;
-			} else if (!strcmp(value, "cancel-upgrade")) {
-				ba.Action = ButtonCancelUpgrade;
 			} else if (!strcmp(value, "cancel-train-unit")) {
 				ba.Action = ButtonCancelTrain;
 			} else if (!strcmp(value, "cancel-build")) {
@@ -1204,19 +1195,17 @@ static int CclDefineButton(lua_State *l)
 			}
 			if (lua_isnumber(l, -1)) {
 				sprintf(buf, "%ld", (long int)lua_tonumber(l, -1));
-				s1 = new_strdup(buf);
+				s2 = buf;
 			} else {
-				s1 = new_strdup(lua_tostring(l, -1));
+				s2 = lua_tostring(l, -1);
 			}
-			ba.ValueStr = s1;
+			ba.ValueStr = s2;
 		} else if (!strcmp(value, "Allowed")) {
 			value = LuaToString(l, -1);
 			if (!strcmp(value, "check-true")) {
 				ba.Allowed = ButtonCheckTrue;
 			} else if (!strcmp(value, "check-false")) {
 				ba.Allowed = ButtonCheckFalse;
-			} else if (!strcmp(value, "check-upgrade")) {
-				ba.Allowed = ButtonCheckUpgrade;
 			} else if (!strcmp(value, "check-units-or")) {
 				ba.Allowed = ButtonCheckUnitsOr;
 			} else if (!strcmp(value, "check-units-and")) {
@@ -1227,44 +1216,30 @@ static int CclDefineButton(lua_State *l)
 				ba.Allowed = ButtonCheckNoNetwork;
 			} else if (!strcmp(value, "check-no-work")) {
 				ba.Allowed = ButtonCheckNoWork;
-			} else if (!strcmp(value, "check-no-research")) {
-				ba.Allowed = ButtonCheckNoResearch;
 			} else if (!strcmp(value, "check-attack")) {
 				ba.Allowed = ButtonCheckAttack;
-			} else if (!strcmp(value, "check-upgrade-to")) {
-				ba.Allowed = ButtonCheckUpgradeTo;
-			} else if (!strcmp(value, "check-research")) {
-				ba.Allowed = ButtonCheckResearch;
-			} else if (!strcmp(value, "check-single-research")) {
-				ba.Allowed = ButtonCheckSingleResearch;
 			} else {
 				LuaError(l, "Unsupported action: %s" _C_ value);
 			}
 		} else if (!strcmp(value, "AllowArg")) {
 			int subargs;
 			int k;
+			std::string allowstr;
 
 			if (!lua_istable(l, -1)) {
 				LuaError(l, "incorrect argument");
 			}
-			s1 = new_strdup("");
 			subargs = luaL_getn(l, -1);
 			for (k = 0; k < subargs; ++k) {
 				lua_rawgeti(l, -1, k + 1);
 				s2 = LuaToString(l, -1);
 				lua_pop(l, 1);
-				int news1len = strlen(s1) + strlen(s2) + 2;
-				char *news1 = new char[news1len];
-				strcpy_s(news1, news1len, s1);
-				strcat_s(news1, news1len, s2);
-				delete[] s1;
-				s1 = news1;
+				allowstr += s2;
 				if (k != subargs - 1) {
-					strcat_s(s1, news1len, ",");
+					allowstr += ",";
 				}
 			}
-			ba.AllowStr = s1;
-			delete[] s1;
+			ba.AllowStr = allowstr;
 		} else if (!strcmp(value, "Key")) {
 			ba.Key = *LuaToString(l, -1);
 		} else if (!strcmp(value, "Hint")) {
@@ -1272,28 +1247,23 @@ static int CclDefineButton(lua_State *l)
 		} else if (!strcmp(value, "ForUnit")) {
 			int subargs;
 			int k;
+			std::string umask;
 
 			if (!lua_istable(l, -1)) {
 				LuaError(l, "incorrect argument");
 			}
 			// FIXME: ba.UnitMask shouldn't be a string
-			s1 = new_strdup(",");
+			umask = ",";
 			subargs = luaL_getn(l, -1);
 			for (k = 0; k < subargs; ++k) {
 				lua_rawgeti(l, -1, k + 1);
 				s2 = LuaToString(l, -1);
 				lua_pop(l, 1);
-				int news1len = strlen(s1) + strlen(s2) + 2;
-				char *news1 = new char[news1len];
-				strcpy_s(news1, news1len, s1);
-				strcat_s(news1, news1len, s2);
-				strcat_s(news1, news1len, ",");
-				delete[] s1;
-				s1 = news1;
+				umask += s2;
+				umask += ",";
 			}
-			ba.UnitMask = s1;
-			delete[] s1;
-			if (!strncmp(ba.UnitMask.c_str(), ",*,", 3)) {
+			ba.UnitMask = umask;
+			if (ba.UnitMask.compare(0, 3, ",*,") == 0) {
 				ba.UnitMask = "*";
 			}
 		} else {
@@ -1331,35 +1301,6 @@ void SelectedUnitChanged(void)
 	UI.ButtonPanel.Update();
 }
 
-/**
-**  Set selection style.
-**
-**  @param l  Lua state.
-*/
-static int CclSetSelectionStyle(lua_State *l)
-{
-	const char *style;
-
-	LuaCheckArgs(l, 1);
-
-	style = LuaToString(l, 1);
-	if (!strcmp(style, "rectangle")) {
-		DrawSelection = DrawSelectionRectangle;
-	} else if (!strcmp(style, "alpha-rectangle")) {
-		DrawSelection = DrawSelectionRectangleWithTrans;
-	} else if (!strcmp(style, "circle")) {
-		DrawSelection = DrawSelectionCircle;
-	} else if (!strcmp(style, "alpha-circle")) {
-		DrawSelection = DrawSelectionCircleWithTrans;
-	} else if (!strcmp(style, "corners")) {
-		DrawSelection = DrawSelectionCorners;
-	} else {
-		LuaError(l, "Unsupported selection style");
-	}
-
-	return 0;
-}
-
 /**
 **  Add a new message.
 **
@@ -1380,10 +1321,7 @@ static int CclAddMessage(lua_State *l)
 static int CclSetGroupKeys(lua_State *l)
 {
 	LuaCheckArgs(l, 1);
-	if (UiGroupKeys != DefaultGroupKeys) {
-		delete[] UiGroupKeys;
-	}
-	UiGroupKeys = new_strdup(LuaToString(l, 1));
+	UiGroupKeys = LuaToString(l, 1);
 	return 0;
 }
 
@@ -1453,11 +1391,6 @@ void UserInterfaceCclRegister(void)
 	lua_register(Lua, "PresentMap", CclPresentMap);
 	lua_register(Lua, "DefineMapSetup", CclDefineMapSetup);
 
-	//
-	// Look and feel of units
-	//
-	lua_register(Lua, "SetSelectionStyle", CclSetSelectionStyle);
-
 	lua_register(Lua, "SetGroupKeys", CclSetGroupKeys);
 }
 
