@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: ui.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: ui.cpp 8839 2007-06-02 17:05:34Z jim4 $
 
 //@{
 
@@ -74,7 +74,6 @@ CUserInterface::CUserInterface() :
 	SingleTrainingButton(NULL),
 	SingleTrainingFont(0), SingleTrainingTextX(0), SingleTrainingTextY(0),
 	TrainingFont(0), TrainingTextX(0), TrainingTextY(0),
-	UpgradingButton(NULL), ResearchingButton(NULL),
 	CompletedBarColor(0), CompletedBarShadow(0),
 	ViewportMode(VIEWPORT_SINGLE), MouseViewport(NULL),
 	SelectedViewport(NULL), NumViewports(0),
@@ -267,8 +266,6 @@ void CleanUserInterface(void)
 	UI.SingleTrainingText.clear();
 	UI.TrainingButtons.clear();
 	UI.TrainingText.clear();
-	delete UI.UpgradingButton;
-	delete UI.ResearchingButton;
 	UI.TransportingButtons.clear();
 
 	// Button Panel
@@ -291,6 +288,17 @@ void CleanUserInterface(void)
 	}
 }
 
+#ifdef DEBUG
+void FreeButtonStyles()
+{
+	std::map<std::string, ButtonStyle *>::iterator i;
+	for (i = ButtonStyleHash.begin(); i != ButtonStyleHash.end(); ++i) {
+		delete (*i).second;
+	}
+	ButtonStyleHash.clear();
+}
+#endif
+
 /**
 **  Takes coordinates of a pixel in stratagus's window and computes
 **  the map viewport which contains this pixel.
