@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: widgets.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: widgets.cpp 9008 2007-07-08 03:49:07Z jim4 $
 
 //@{
 
@@ -272,18 +272,46 @@ void MyOpenGLGraphics::drawLine(int x1, int y1, int x2, int y2)
 
 void MyOpenGLGraphics::drawRectangle(const gcn::Rectangle& rectangle)
 {
+	gcn::Rectangle area = rectangle;
+	gcn::ClipRectangle top = mClipStack.top();
 	gcn::Color c = this->getColor();
-	Video.DrawRectangle(Video.MapRGBA(0, c.r, c.g, c.b, c.a),
-		rectangle.x + mClipStack.top().xOffset, rectangle.y + mClipStack.top().yOffset,
-		rectangle.width, rectangle.height);
+
+	area.x += top.xOffset;
+	area.y += top.yOffset;
+
+	if (!area.intersect(top) || c.a == 0) {
+		return;
+	}
+
+	int x1 = area.x > top.x ? area.x : top.x;
+	int y1 = area.y > top.y ? area.y : top.y;
+	int x2 = area.x + area.width < top.x + top.width ? area.x + area.width : top.x + top.width;
+	int y2 = area.y + area.height < top.y + top.height ? area.y + area.height : top.y + top.height;
+
+	Video.DrawTransRectangle(Video.MapRGB(0, c.r, c.g, c.b),
+		x1, y1, x2 - x1, y2 - y1, mColor.a);
 }
 
 void MyOpenGLGraphics::fillRectangle(const gcn::Rectangle& rectangle)
 {
+	gcn::Rectangle area = rectangle;
+	gcn::ClipRectangle top = mClipStack.top();
 	gcn::Color c = this->getColor();
-	Video.FillRectangle(Video.MapRGBA(0, c.r, c.g, c.b, c.a),
-		rectangle.x + mClipStack.top().xOffset, rectangle.y + mClipStack.top().yOffset,
-		rectangle.width, rectangle.height);
+
+	area.x += top.xOffset;
+	area.y += top.yOffset;
+
+	if (!area.intersect(top) || c.a == 0) {
+		return;
+	}
+
+	int x1 = area.x > top.x ? area.x : top.x;
+	int y1 = area.y > top.y ? area.y : top.y;
+	int x2 = area.x + area.width < top.x + top.width ? area.x + area.width : top.x + top.width;
+	int y2 = area.y + area.height < top.y + top.height ? area.y + area.height : top.y + top.height;
+
+	Video.FillTransRectangle(Video.MapRGB(0, c.r, c.g, c.b),
+		x1, y1, x2 - x1, y2 - y1, c.a);
 }
 #endif
 
@@ -1475,6 +1503,7 @@ int MenuScreen::run(bool loop)
 
 	CursorState = CursorStatePoint;
 	GameCursor = UI.Point.Cursor;
+	CursorOn = CursorOnUnknown;
 
 	if (loop) {
 		const EventCallback *old_callbacks = GetCallbacks();
