@@ -9,7 +9,7 @@
 //
 /**@name script_unit.cpp - The unit ccl functions. */
 //
-//      (c) Copyright 2001-2005 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 2001-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: script_unit.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: script_unit.cpp 9030 2007-08-04 17:09:35Z jim4 $
 
 //@{
 
@@ -61,34 +61,6 @@
 	/// Get resource by name
 extern unsigned CclGetResourceByName(lua_State *l);
 
-/**
-**  Set xp damage
-**
-**  @param l  Lua state.
-**
-**  @return   The old state of the xp damage
-*/
-static int CclSetXpDamage(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	XpDamage = LuaToBoolean(l, 1);
-	return 0;
-}
-
-/**
-**  Set training queue
-**
-**  @param l  Lua state.
-**
-**  @return  The old state of the training queue
-*/
-static int CclSetTrainingQueue(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	EnableTrainingQueue = LuaToBoolean(l, 1);
-	return 0;
-}
-
 /**
 **  Set capture buildings
 **
@@ -103,20 +75,6 @@ static int CclSetBuildingCapture(lua_State *l)
 	return 0;
 }
 
-/**
-**  Set reveal attacker
-**
-**  @param l  Lua state.
-**
-**  @return   The old state of the flag
-*/
-static int CclSetRevealAttacker(lua_State *l)
-{
-	LuaCheckArgs(l, 1);
-	RevealAttacker = LuaToBoolean(l, 1);
-	return 0;
-}
-
 /**
 **  Get a unit pointer
 **
@@ -169,10 +127,6 @@ void CclParseOrder(lua_State *l, COrder *order)
 			order->Action = UnitActionSpellCast;
 		} else if (!strcmp(value, "action-train")) {
 			order->Action = UnitActionTrain;
-		} else if (!strcmp(value, "action-upgrade-to")) {
-			order->Action = UnitActionUpgradeTo;
-		} else if (!strcmp(value, "action-research")) {
-			order->Action = UnitActionResearch;
 		} else if (!strcmp(value, "action-built")) {
 			order->Action = UnitActionBuilt;
 		} else if (!strcmp(value, "action-board")) {
@@ -187,10 +141,6 @@ void CclParseOrder(lua_State *l, COrder *order)
 			order->Action = UnitActionRepair;
 		} else if (!strcmp(value, "action-resource")) {
 			order->Action = UnitActionResource;
-		} else if (!strcmp(value, "action-return-goods")) {
-			order->Action = UnitActionReturnGoods;
-		} else if (!strcmp(value, "action-transform-into")) {
-			order->Action = UnitActionTransformInto;
 		} else if (!strcmp(value, "range")) {
 			++j;
 			lua_rawgeti(l, -1, j + 1);
@@ -266,18 +216,6 @@ void CclParseOrder(lua_State *l, COrder *order)
 			order->Arg1.Spell = SpellTypeByIdent(LuaToString(l, -1));
 			lua_pop(l, 1);
 
-		} else if (!strcmp(value, "upgrade")) {
-			++j;
-			lua_rawgeti(l, -1, j + 1);
-			order->Arg1.Upgrade = CUpgrade::Get(LuaToString(l, -1));
-			lua_pop(l, 1);
-
-		} else if (!strcmp(value, "mine")) {
-			++j;
-			lua_rawgeti(l, -1, j + 1);
-			order->Arg1.ResourcePos = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-
 		} else {
 		   // This leaves a half initialized unit
 		   LuaError(l, "Unsupported tag: %s" _C_ value);
@@ -357,97 +295,8 @@ static void CclParseBuilt(lua_State *l, CUnit *unit)
 				cframe = cframe->Next;
 			}
 			unit->Data.Built.Frame = cframe;
-		}
-	}
-}
-
-/**
-**  Parse res worker data
-**
-**  @param l     Lua state.
-**  @param unit  Unit pointer which should be filled with the data.
-*/
-static void CclParseResWorker(lua_State *l, CUnit *unit)
-{
-	const char *value;
-	int args;
-	int j;
-
-	if (!lua_istable(l, -1)) {
-		LuaError(l, "incorrect argument");
-	}
-	args = luaL_getn(l, -1);
-	for (j = 0; j < args; ++j) {
-		lua_rawgeti(l, -1, j + 1);
-		value = LuaToString(l, -1);
-		lua_pop(l, 1);
-		++j;
-		if (!strcmp(value, "time-to-harvest")) {
-			lua_rawgeti(l, -1, j + 1);
-			unit->Data.ResWorker.TimeToHarvest = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-		} else if (!strcmp(value, "done-harvesting")) {
-			unit->Data.ResWorker.DoneHarvesting = 1;
-			--j;
-		}
-	}
-}
-
-/**
-**  Parse research
-**
-**  @param l     Lua state.
-**  @param unit  Unit pointer which should be filled with the data.
-*/
-static void CclParseResearch(lua_State *l, CUnit *unit)
-{
-	const char *value;
-	int args;
-	int j;
-
-	if (!lua_istable(l, -1)) {
-		LuaError(l, "incorrect argument");
-	}
-	args = luaL_getn(l, -1);
-	for (j = 0; j < args; ++j) {
-		lua_rawgeti(l, -1, j + 1);
-		value = LuaToString(l, -1);
-		lua_pop(l, 1);
-		++j;
-		if (!strcmp(value, "ident")) {
-			lua_rawgeti(l, -1, j + 1);
-			value = LuaToString(l, -1);
-			lua_pop(l, 1);
-			unit->Data.Research.Upgrade = CUpgrade::Get(value);
-		}
-	}
-}
-
-/**
-**  Parse upgrade to
-**
-**  @param l     Lua state.
-**  @param unit  Unit pointer which should be filled with the data.
-*/
-static void CclParseUpgradeTo(lua_State *l, CUnit *unit)
-{
-	const char *value;
-	int args;
-	int j;
-
-	if (!lua_istable(l, -1)) {
-		LuaError(l, "incorrect argument");
-	}
-	args = luaL_getn(l, -1);
-	for (j = 0; j < args; ++j) {
-		lua_rawgeti(l, -1, j + 1);
-		value = LuaToString(l, -1);
-		lua_pop(l, 1);
-		++j;
-		if (!strcmp(value, "ticks")) {
-			lua_rawgeti(l, -1, j + 1);
-			unit->Data.UpgradeTo.Ticks = LuaToNumber(l, -1);
-			lua_pop(l, 1);
+		} else {
+			LuaError(l, "Unsupported tag: %s" _C_ value);
 		}
 	}
 }
@@ -477,6 +326,46 @@ static void CclParseTrain(lua_State *l, CUnit *unit)
 			lua_rawgeti(l, -1, j + 1);
 			unit->Data.Train.Ticks = LuaToNumber(l, -1);
 			lua_pop(l, 1);
+		} else {
+			LuaError(l, "Unsupported tag: %s" _C_ value);
+		}
+	}
+}
+
+/**
+**  Parse stored data for harvest order
+**
+**  @param l     Lua state.
+**  @param unit  Unit pointer which should be filled with the data.
+*/
+static void CclParseHarvest(lua_State *l, CUnit *unit)
+{
+	const char *value;
+	int args;
+	int j;
+
+	if (!lua_istable(l, -1)) {
+		LuaError(l, "incorrect argument");
+	}
+	args = luaL_getn(l, -1);
+	for (j = 0; j < args; ++j) {
+		lua_rawgeti(l, -1, j + 1);
+		value = LuaToString(l, -1);
+		lua_pop(l, 1);
+		++j;
+		if (!strcmp(value, "current-production")) {
+			lua_rawgeti(l, -1, j + 1);
+			if (!lua_istable(l, -1) || luaL_getn(l, -1) != MaxCosts) {
+				LuaError(l, "incorrect argument");
+			}
+			for (int i = 0; i < MaxCosts; ++i) {
+				lua_rawgeti(l, -1, i + 1);
+				unit->Data.Harvest.CurrentProduction[i] = LuaToNumber(l, -1);
+				lua_pop(l, 1);
+			}
+			lua_pop(l, 1);
+		} else {
+			LuaError(l, "Unsupported tag: %s" _C_ value);
 		}
 	}
 }
@@ -521,6 +410,8 @@ static void CclParseMove(lua_State *l, CUnit *unit)
 			}
 			unit->Data.Move.Length = subargs;
 			lua_pop(l, 1);
+		} else {
+			LuaError(l, "Unsupported tag: %s" _C_ value);
 		}
 	}
 }
@@ -586,7 +477,6 @@ static int CclUnit(lua_State *l)
 			unit = UnitSlots[slot];
 			unit->Init(type);
 			unit->Seen.Type = seentype;
-			unit->Active = 0;
 			unit->Removed = 0;
 			Assert(unit->Slot == slot);
 		} else if (!strcmp(value, "next")) {
@@ -707,9 +597,6 @@ static int CclUnit(lua_State *l)
 			--j;
 		} else if (!strcmp(value, "seen-state")) {
 			unit->Seen.State = LuaToNumber(l, j + 1);
-		} else if (!strcmp(value, "active")) {
-			unit->Active = 1;
-			--j;
 		} else if (!strcmp(value, "resource-active")) {
 			unit->Data.Resource.Active = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "ttl")) {
@@ -720,11 +607,16 @@ static int CclUnit(lua_State *l)
 		} else if (!strcmp(value, "last-group")) {
 			unit->LastGroup = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "resources-held")) {
-			unit->ResourcesHeld = LuaToNumber(l, j + 1);
-		} else if (!strcmp(value, "current-resource")) {
-			lua_pushvalue(l, j + 1);
-			unit->CurrentResource = CclGetResourceByName(l);
-			lua_pop(l, 1);
+			if (!lua_istable(l, j + 1) || luaL_getn(l, j + 1) != MaxCosts) {
+				LuaError(l, "incorrect argument");
+			}
+			for (i = 0; i < MaxCosts; ++i) {
+				lua_rawgeti(l, j + 1, i + 1);
+				unit->ResourcesHeld[i] = LuaToNumber(l, -1);
+				lua_pop(l, 1);
+			}
+		} else if (!strcmp(value, "production-efficiency")) {
+			unit->ProductionEfficiency = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "sub-action")) {
 			unit->SubAction = LuaToNumber(l, j + 1);
 		} else if (!strcmp(value, "wait")) {
@@ -785,10 +677,6 @@ static int CclUnit(lua_State *l)
 				// HACK: the building is not ready yet
 				unit->Player->UnitTypesCount[type->Slot]--;
 			}
-		} else if (!strcmp(value, "critical-order")) {
-			lua_pushvalue(l, j + 1);
-			CclParseOrder(l, &unit->CriticalOrder);
-			lua_pop(l, 1);
 		} else if (!strcmp(value, "saved-order")) {
 			lua_pushvalue(l, j + 1);
 			CclParseOrder(l, &unit->SavedOrder);
@@ -801,22 +689,14 @@ static int CclUnit(lua_State *l)
 			lua_pushvalue(l, j + 1);
 			CclParseBuilt(l, unit);
 			lua_pop(l, 1);
-		} else if (!strcmp(value, "data-res-worker")) {
-			lua_pushvalue(l, j + 1);
-			CclParseResWorker(l, unit);
-			lua_pop(l, 1);
-		} else if (!strcmp(value, "data-research")) {
-			lua_pushvalue(l, j + 1);
-			CclParseResearch(l, unit);
-			lua_pop(l, 1);
-		} else if (!strcmp(value, "data-upgrade-to")) {
-			lua_pushvalue(l, j + 1);
-			CclParseUpgradeTo(l, unit);
-			lua_pop(l, 1);
 		} else if (!strcmp(value, "data-train")) {
 			lua_pushvalue(l, j + 1);
 			CclParseTrain(l, unit);
 			lua_pop(l, 1);
+		} else if (!strcmp(value, "data-harvest")) {
+			lua_pushvalue(l, j + 1);
+			CclParseHarvest(l, unit);
+			lua_pop(l, 1);
 		} else if (!strcmp(value, "data-move")) {
 			lua_pushvalue(l, j + 1);
 			CclParseMove(l, unit);
@@ -837,7 +717,7 @@ static int CclUnit(lua_State *l)
 				DefineVariableField(l, unit->Variable + i, j + 1);
 				continue;
 			}
-		   LuaError(l, "Unsupported tag: %s" _C_ value);
+			LuaError(l, "Unsupported tag: %s" _C_ value);
 		}
 	}
 
@@ -856,12 +736,6 @@ static int CclUnit(lua_State *l)
 		MapMarkUnitSight(unit);
 	}
 
-	// Place on map
-	if (!unit->Removed) {
-		unit->Removed = 1;
-		unit->Place(unit->X, unit->Y);
-	}
-
 	// Fix Colors for rescued units.
 	if (unit->RescuedFrom) {
 		unit->Colors = &unit->RescuedFrom->UnitColors;
@@ -947,7 +821,7 @@ static int CclCreateUnit(lua_State *l)
 	playerno = TriggerGetPlayer(l);
 	lua_pop(l, 1);
 	if (playerno == -1) {
-		printf("CreateUnit: You cannot use 'any in create-unit, specify a player\n");
+		printf("CreateUnit: You cannot use \"any\" in create-unit, specify a player\n");
 		LuaError(l, "bad player");
 		return 0;
 	}
@@ -962,7 +836,8 @@ static int CclCreateUnit(lua_State *l)
 		DebugPrint("Unable to allocate unit");
 		return 0;
 	} else {
-		if (UnitCanBeAt(unit, ix, iy)) {
+		if (UnitCanBeAt(unit, ix, iy) ||
+				(unit->Type->Building && CanBuildUnitType(NULL, unit->Type, ix, iy, 0))) {
 			unit->Place(ix, iy);
 		} else {
 			unit->X = ix;
@@ -984,19 +859,25 @@ static int CclCreateUnit(lua_State *l)
 static int CclSetResourcesHeld(lua_State *l)
 {
 	CUnit *unit;
-	int value;
 
 	LuaCheckArgs(l, 2);
 
 	if (lua_isnil(l, 1)) {
 		return 0;
 	}
+	if (!lua_istable(l, 2) || luaL_getn(l, 2) != MaxCosts) {
+		LuaError(l, "incorrect argument");
+	}
 
 	lua_pushvalue(l, 1);
 	unit = CclGetUnit(l);
 	lua_pop(l, 1);
-	value = LuaToNumber(l, 2);
-	unit->ResourcesHeld = value;
+
+	for (int i = 0; i < MaxCosts; ++i) {
+		lua_rawgeti(l, 2, i + 1);
+		unit->ResourcesHeld[i] = LuaToNumber(l, -1) * CYCLES_PER_SECOND;
+		lua_pop(l, 1);
+	}
 
 	return 0;
 }
@@ -1353,10 +1234,7 @@ static int CclSlotUsage(lua_State *l)
 */
 void UnitCclRegister(void)
 {
-	lua_register(Lua, "SetXPDamage", CclSetXpDamage);
-	lua_register(Lua, "SetTrainingQueue", CclSetTrainingQueue);
 	lua_register(Lua, "SetBuildingCapture", CclSetBuildingCapture);
-	lua_register(Lua, "SetRevealAttacker", CclSetRevealAttacker);
 
 	lua_register(Lua, "Unit", CclUnit);
 
