@@ -9,7 +9,7 @@
 //
 /**@name script_unittype.cpp - The unit-type ccl functions. */
 //
-//      (c) Copyright 1999-2005 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1999-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: script_unittype.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: script_unittype.cpp 8902 2007-06-20 05:51:48Z jim4 $
 
 //@{
 
@@ -99,7 +99,7 @@ unsigned CclGetResourceByName(lua_State *l)
 
 	value = LuaToString(l, -1);
 	for (i = 0; i < MaxCosts; ++i) {
-		if (!strcmp(value, DefaultResourceNames[i])) {
+		if (value == DefaultResourceNames[i]) {
 			return i;
 		}
 	}
@@ -215,7 +215,6 @@ static int CclDefineUnitType(lua_State *l)
 {
 	const char *value;
 	CUnitType *type;
-	ResourceInfo *res;
 	const char *str;
 	int i;
 	int redefine;
@@ -341,38 +340,46 @@ static int CclDefineUnitType(lua_State *l)
 		} else if (!strcmp(value, "Icon")) {
 			type->Icon.Name = LuaToString(l, -1);
 			type->Icon.Icon = NULL;
-		} else if (!strcmp(value, "Costs")) {
-			if (!lua_istable(l, -1)) {
+		} else if (!strcmp(value, "EnergyValue")) {
+			if (!lua_isnumber(l, -1)) {
 				LuaError(l, "incorrect argument");
 			}
-			subargs = luaL_getn(l, -1);
-			for (k = 0; k < subargs; ++k) {
-				int res;
-
-				lua_rawgeti(l, -1, k + 1);
-				res = CclGetResourceByName(l);
-				lua_pop(l, 1);
-				++k;
-				lua_rawgeti(l, -1, k + 1);
-				type->_Costs[res] = LuaToNumber(l, -1);
-				lua_pop(l, 1);
-			}
-		} else if (!strcmp(value, "ImproveProduction")) {
-			if (!lua_istable(l, -1)) {
+			type->SetEnergyValue(LuaToNumber(l, -1));
+		} else if (!strcmp(value, "MagmaValue")) {
+			if (!lua_isnumber(l, -1)) {
 				LuaError(l, "incorrect argument");
 			}
-			subargs = luaL_getn(l, -1);
-			for (k = 0; k < subargs; ++k) {
-				int res;
-
-				lua_rawgeti(l, -1, k + 1);
-				res = CclGetResourceByName(l);
-				lua_pop(l, 1);
-				++k;
-				lua_rawgeti(l, -1, k + 1);
-				type->ImproveIncomes[res] = DefaultIncomes[res] + LuaToNumber(l, -1);
-				lua_pop(l, 1);
+			type->SetMagmaValue(LuaToNumber(l, -1));
+		} else if (!strcmp(value, "MaxEnergyUtilizationRate")) {
+			if (!lua_isnumber(l, -1)) {
+				LuaError(l, "incorrect argument");
 			}
+			type->SetMaxEnergyUtilizationRate(LuaToNumber(l, -1));
+		} else if (!strcmp(value, "MaxMagmaUtilizationRate")) {
+			if (!lua_isnumber(l, -1)) {
+				LuaError(l, "incorrect argument");
+			}
+			type->SetMaxMagmaUtilizationRate(LuaToNumber(l, -1));
+		} else if (!strcmp(value, "EnergyProductionRate")) {
+			if (!lua_isnumber(l, -1)) {
+				LuaError(l, "incorrect argument");
+			}
+			type->SetEnergyProductionRate(LuaToNumber(l, -1));
+		} else if (!strcmp(value, "MagmaProductionRate")) {
+			if (!lua_isnumber(l, -1)) {
+				LuaError(l, "incorrect argument");
+			}
+			type->SetMagmaProductionRate(LuaToNumber(l, -1));
+		} else if (!strcmp(value, "EnergyStorageCapacity")) {
+			if (!lua_isnumber(l, -1)) {
+				LuaError(l, "incorrect argument");
+			}
+			type->SetEnergyStorageCapacity(LuaToNumber(l, -1));
+		} else if (!strcmp(value, "MagmaStorageCapacity")) {
+			if (!lua_isnumber(l, -1)) {
+				LuaError(l, "incorrect argument");
+			}
+			type->SetMagmaStorageCapacity(LuaToNumber(l, -1));
 		} else if (!strcmp(value, "Construction")) {
 			// FIXME: What if constructions aren't yet loaded?
 			type->Construction = ConstructionByIdent(LuaToString(l, -1));
@@ -380,8 +387,6 @@ static int CclDefineUnitType(lua_State *l)
 			type->DrawLevel = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "MaxOnBoard")) {
 			type->MaxOnBoard = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "StartingResources")) {
-			type->StartingResources = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "RegenerationRate")) {
 			type->Variable[HP_INDEX].Increase = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "BurnPercent")) {
@@ -437,7 +442,7 @@ static int CclDefineUnitType(lua_State *l)
 		} else if (!strcmp(value, "PersonReactionRange")) {
 			type->ReactRangePerson = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Missile")) {
-			type->Missile.Name = new_strdup(LuaToString(l, -1));
+			type->Missile.Name = LuaToString(l, -1);
 			type->Missile.Missile = NULL;
 		} else if (!strcmp(value, "MinAttackRange")) {
 			type->MinAttackRange = LuaToNumber(l, -1);
@@ -452,19 +457,12 @@ static int CclDefineUnitType(lua_State *l)
 			type->DecayRate = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Points")) {
 			type->Points = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "Demand")) {
-			type->Demand = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "Supply")) {
-			type->Supply = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Corpse")) {
-			if (redefine) {
-				delete[] type->CorpseName;
-			}
-			type->CorpseName = new_strdup(LuaToString(l, -1));
+			type->CorpseName = LuaToString(l, -1);
 			type->CorpseType = NULL;
 		} else if (!strcmp(value, "ExplodeWhenKilled")) {
 			type->ExplodeWhenKilled = 1;
-			type->Explosion.Name = new_strdup(LuaToString(l, -1));
+			type->Explosion.Name = LuaToString(l, -1);
 			type->Explosion.Missile = NULL;
 		} else if (!strcmp(value, "Type")) {
 			value = LuaToString(l, -1);
@@ -504,22 +502,6 @@ static int CclDefineUnitType(lua_State *l)
 			type->RepairRange = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "RepairHp")) {
 			type->RepairHP = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "RepairCosts")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, -1);
-			for (k = 0; k < subargs; ++k) {
-				int res;
-
-				lua_rawgeti(l, -1, k + 1);
-				res = CclGetResourceByName(l);
-				lua_pop(l, 1);
-				++k;
-				lua_rawgeti(l, -1, k + 1);
-				type->RepairCosts[res] = LuaToNumber(l, -1);
-				lua_pop(l, 1);
-			}
 		} else if (!strcmp(value, "CanTargetLand")) {
 			if (LuaToBoolean(l, -1)) {
 				type->CanTarget |= CanTargetLand;
@@ -566,8 +548,6 @@ static int CclDefineUnitType(lua_State *l)
 			type->BuilderOutside = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "BuilderLost")) {
 			type->BuilderLost = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "AutoBuildRate")) {
-			type->AutoBuildRate = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "ShoreBuilding")) {
 			type->ShoreBuilding = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "LandUnit")) {
@@ -576,10 +556,6 @@ static int CclDefineUnitType(lua_State *l)
 			type->AirUnit = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "SeaUnit")) {
 			type->SeaUnit = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "RandomMovementProbability")) {
-			type->RandomMovementProbability = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "ClicksToExplode")) {
-			type->ClicksToExplode = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Indestructible")) {
 			type->Indestructible = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "CanTransport")) {
@@ -620,90 +596,12 @@ static int CclDefineUnitType(lua_State *l)
 			type->AttackFromTransporter = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "Coward")) {
 			type->Coward = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "CanGatherResources")) {
-			int args;
-			int j;
-
-			args = luaL_getn(l, -1);
-			for (j = 0; j < args; ++j) {
-				lua_rawgeti(l, -1, j + 1);
-				res = new ResourceInfo;
-				if (!lua_istable(l, -1)) {
-					LuaError(l, "incorrect argument");
-				}
-				subargs = luaL_getn(l, -1);
-				for (k = 0; k < subargs; ++k) {
-					lua_rawgeti(l, -1, k + 1);
-					value = LuaToString(l, -1);
-					lua_pop(l, 1);
-					++k;
-					if (!strcmp(value, "resource-id")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->ResourceId = CclGetResourceByName(l);
-						lua_pop(l, 1);
-						type->ResInfo[res->ResourceId] = res;
-					} else if (!strcmp(value, "resource-step")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->ResourceStep = LuaToNumber(l, -1);
-						lua_pop(l, 1);
-					} else if (!strcmp(value, "final-resource")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->FinalResource = CclGetResourceByName(l);
-						lua_pop(l, 1);
-					} else if (!strcmp(value, "wait-at-resource")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->WaitAtResource = LuaToNumber(l, -1);
-						lua_pop(l, 1);
-					} else if (!strcmp(value, "wait-at-depot")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->WaitAtDepot = LuaToNumber(l, -1);
-						lua_pop(l, 1);
-					} else if (!strcmp(value, "resource-capacity")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->ResourceCapacity = LuaToNumber(l, -1);
-						lua_pop(l, 1);
-					} else if (!strcmp(value, "lose-resources")) {
-						res->LoseResources = 1;
-						--k;
-					} else if (!strcmp(value, "harvest-from-outside")) {
-						res->HarvestFromOutside = 1;
-						--k;
-					} else if (!strcmp(value, "file-when-empty")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->FileWhenEmpty = LuaToString(l, -1);
-						lua_pop(l, 1);
-					} else if (!strcmp(value, "file-when-loaded")) {
-						lua_rawgeti(l, -1, k + 1);
-						res->FileWhenLoaded = LuaToString(l, -1);
-						lua_pop(l, 1);
-					} else {
-					   printf("\n%s\n", type->Name.c_str());
-					   LuaError(l, "Unsupported tag: %s" _C_ value);
-					}
-				}
-				if (!res->FinalResource) {
-					res->FinalResource = res->ResourceId;
-				}
-				Assert(res->ResourceId);
-				lua_pop(l, 1);
-			}
-			type->Harvester = 1;
-		} else if (!strcmp(value, "GivesResource")) {
-			lua_pushvalue(l, -1);
-			type->GivesResource = CclGetResourceByName(l);
-			lua_pop(l, 1);
-		} else if (!strcmp(value, "CanHarvest")) {
-			type->CanHarvest = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "CanStore")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, -1);
-			for (k = 0; k < subargs; ++k) {
-				lua_rawgeti(l, -1, k + 1);
-				type->CanStore[CclGetResourceByName(l)] = 1;
-				lua_pop(l, 1);
-			}
+		} else if (!strcmp(value, "Harvester")) {
+			type->Harvester = LuaToBoolean(l, -1);
+		} else if (!strcmp(value, "Neutral")) {
+			type->Neutral = LuaToBoolean(l, -1);
+		} else if (!strcmp(value, "CanHarvestFrom")) {
+			type->CanHarvestFrom = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "Vanishes")) {
 			type->Vanishes = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "CanCastSpell")) {
@@ -797,6 +695,8 @@ static int CclDefineUnitType(lua_State *l)
 				}
 				LuaError(l, "Unsupported flag tag for can-target-flag: %s" _C_ value);
 			}
+		} else if (!strcmp(value, "ProductionEfficiency")) {
+			type->ProductionEfficiency = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "IsNotSelectable")) {
 			type->IsNotSelectable = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "SelectableByRectangle")) {
@@ -831,23 +731,8 @@ static int CclDefineUnitType(lua_State *l)
 					type->Sound.Repair.Name = LuaToString(l, -1);
 					lua_pop(l, 1);
 				} else if (!strcmp(value, "harvest")) {
-					int res;
-					const char *name;
-
 					lua_rawgeti(l, -1, k + 1);
-					name = LuaToString(l, -1 );
-					lua_pop(l, 1);
-					++k;
-					for (res = 0; res < MaxCosts; ++res) {
-						if (!strcmp(name, DefaultResourceNames[res])) {
-							break;
-						}
-					}
-					if (res == MaxCosts) {
-						LuaError(l, "Resource not found: %s" _C_ value);
-					}
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Harvest[res].Name = LuaToString(l, -1);
+					type->Sound.Harvest.Name = LuaToString(l, -1);
 					lua_pop(l, 1);
 				} else if (!strcmp(value, "help")) {
 					lua_rawgeti(l, -1, k + 1);
@@ -906,95 +791,6 @@ static int CclDefineUnitType(lua_State *l)
 	return 0;
 }
 
-/**
-**  Parse unit-stats.
-**
-**  @param l  Lua state.
-*/
-static int CclDefineUnitStats(lua_State *l)
-{
-	const char *value;
-	CUnitType *type;
-	CUnitStats *stats;
-	int i;
-	int args;
-	int j;
-
-	args = lua_gettop(l);
-	j = 0;
-
-	type = UnitTypeByIdent(LuaToString(l, j + 1));
-	Assert(type);
-	++j;
-
-	i = LuaToNumber(l, j + 1);
-	Assert(i < PlayerMax);
-	++j;
-
-	stats = &type->Stats[i];
-	if (!stats->Variables) {
-		stats->Variables = new CVariable[UnitTypeVar.NumberVariable];
-	}
-
-	//
-	// Parse the list: (still everything could be changed!)
-	//
-	for (; j < args; ++j) {
-
-		value = LuaToString(l, j + 1);
-		++j;
-
-		if (!strcmp(value, "costs")) {
-			int subargs;
-			int k;
-
-			if (!lua_istable(l, j + 1)) {
-				LuaError(l, "incorrect argument");
-			}
-			subargs = luaL_getn(l, j + 1);
-			for (k = 0; k < subargs; ++k) {
-
-				lua_rawgeti(l, j + 1, k + 1);
-				value = LuaToString(l, -1);
-				lua_pop(l, 1);
-				++k;
-
-				for (i = 0; i < MaxCosts; ++i) {
-					if (!strcmp(value, DefaultResourceNames[i])) {
-						lua_rawgeti(l, j + 1, k + 1);
-						stats->Costs[i] = LuaToNumber(l, -1);
-						lua_pop(l, 1);
-						break;
-					}
-				}
-				if (i == MaxCosts) {
-				   // This leaves half initialized stats
-				   LuaError(l, "Unsupported tag: %s" _C_ value);
-				}
-			}
-		} else {
-			i = GetVariableIndex(value);
-			if (i != -1) { // valid index
-				if (lua_istable(l, j + 1)) {
-					DefineVariableField(l, stats->Variables + i, j + 1);
-				} else if (lua_isnumber(l, -1)) {
-					stats->Variables[i].Enable = 1;
-					stats->Variables[i].Value = LuaToNumber(l, j + 1);
-					stats->Variables[i].Max = LuaToNumber(l, j + 1);
-				} else { // Error
-					LuaError(l, "incorrect argument for the variable in unittype");
-				}
-				continue;
-			}
-
-
-		   // This leaves a half initialized unit
-		   LuaError(l, "Unsupported tag: %s" _C_ value);
-		}
-	}
-	return 0;
-}
-
 // ----------------------------------------------------------------------------
 
 /**
@@ -1021,113 +817,6 @@ CUnitType *CclGetUnitType(lua_State *l)
 	return NULL;
 }
 
-/**
-**  Get unit-type structure.
-**
-**  @param l  Lua state.
-**
-**  @return   Unit-type structure.
-*/
-static int CclUnitType(lua_State *l)
-{
-	const char *str;
-	CUnitType *type;
-	LuaUserData *data;
-
-	LuaCheckArgs(l, 1);
-
-	str = LuaToString(l, 1);
-	type = UnitTypeByIdent(str);
-	data = (LuaUserData *)lua_newuserdata(l, sizeof(LuaUserData));
-	data->Type = LuaUnitType;
-	data->Data = type;
-	return 1;
-}
-
-/**
-**  Get all unit-type structures.
-**
-**  @param l  Lua state.
-**
-**  @return   An array of all unit-type structures.
-*/
-static int CclUnitTypeArray(lua_State *l)
-{
-	LuaCheckArgs(l, 0);
-
-	lua_newtable(l);
-
-	for (std::vector<CUnitType *>::size_type i = 0; i < UnitTypes.size(); ++i) {
-		LuaUserData *data = (LuaUserData *)lua_newuserdata(l, sizeof(LuaUserData));
-		data->Type = LuaUnitType;
-		data->Data = UnitTypes[i];
-		lua_rawseti(l, 1, i + 1);
-	}
-	return 1;
-}
-
-/**
-**  Get the ident of the unit-type structure.
-**
-**  @param l  Lua state.
-**
-**  @return   The identifier of the unit-type.
-*/
-static int CclGetUnitTypeIdent(lua_State *l)
-{
-	const CUnitType *type;
-
-	LuaCheckArgs(l, 1);
-
-	type = CclGetUnitType(l);
-	if (type) {
-		lua_pushstring(l, type->Ident.c_str());
-	} else {
-		LuaError(l, "unit '%s' not defined" _C_ LuaToString(l, -1));
-	}
-	return 1;
-}
-
-/**
-**  Get the name of the unit-type structure.
-**
-**  @param l  Lua state.
-**
-**  @return   The name of the unit-type.
-*/
-static int CclGetUnitTypeName(lua_State *l)
-{
-	const CUnitType *type;
-
-	LuaCheckArgs(l, 1);
-
-	type = CclGetUnitType(l);
-	lua_pushstring(l, type->Name.c_str());
-	return 1;
-}
-
-/**
-**  Set the name of the unit-type structure.
-**
-**  @param l  Lua state.
-**
-**  @return   The name of the unit-type.
-*/
-static int CclSetUnitTypeName(lua_State *l)
-{
-	CUnitType *type;
-
-	LuaCheckArgs(l, 2);
-
-	lua_pushvalue(l, 1);
-	type = CclGetUnitType(l);
-	lua_pop(l, 1);
-	type->Name = LuaToString(l, 2);
-
-	lua_pushvalue(l, 2);
-	return 1;
-}
-
 // ----------------------------------------------------------------------------
 
 /**
@@ -1328,20 +1017,6 @@ static CAnimation *ParseAnimation(lua_State *l, int idx)
 	return anim;
 }
 
-/**
-**  Find the index of a resource
-*/
-static int ResourceIndex(lua_State *l, const char *resource)
-{
-	for (int res = 0; res < MaxCosts; ++res) {
-		if (!strcmp(resource, DefaultResourceNames[res])) {
-			return res;
-		}
-	}
-	LuaError(l, "Resource not found: %s" _C_ resource);
-	return 0;
-}
-
 /**
 **  Define a unit-type animation set.
 **
@@ -1352,7 +1027,6 @@ static int CclDefineAnimations(lua_State *l)
 	const char *name;
 	const char *value;
 	CAnimations *anims;
-	int res;
 
 	LuaCheckArgs(l, 2);
 	if (!lua_istable(l, 2)) {
@@ -1384,15 +1058,8 @@ static int CclDefineAnimations(lua_State *l)
 			anims->Repair = ParseAnimation(l, -1);
 		} else if (!strcmp(value, "Train")) {
 			anims->Train = ParseAnimation(l, -1);
-		} else if (!strcmp(value, "Research")) {
-			anims->Research = ParseAnimation(l, -1);
-		} else if (!strcmp(value, "Upgrade")) {
-			anims->Upgrade = ParseAnimation(l, -1);
-		} else if (!strcmp(value, "Build")) {
-			anims->Build = ParseAnimation(l, -1);
-		} else if (!strncmp(value, "Harvest_", 8)) {
-			res = ResourceIndex(l, value + 8);
-			anims->Harvest[res] = ParseAnimation(l, -1);
+		} else if (!strcmp(value, "Harvest")) {
+			anims->Harvest = ParseAnimation(l, -1);
 		} else {
 			LuaError(l, "Unsupported animation: %s" _C_ value);
 		}
@@ -1571,12 +1238,10 @@ static int CclDefineDecorations(lua_State *l)
 		int OffsetY;
 		int OffsetXPercent;
 		int OffsetYPercent;
-		char IsCenteredInX;
-		char IsCenteredInY;
-		char ShowIfNotEnable;
-		char ShowWhenNull;
-		char HideHalf;
-		char ShowWhenMax;
+		bool IsCenteredInX;
+		bool IsCenteredInY;
+		bool ShowIfNotEnable;
+		bool ShowWhenMax;
 		bool ShowOnlySelected;
 		bool HideNeutral;
 		bool HideAllied;
@@ -1593,6 +1258,7 @@ static int CclDefineDecorations(lua_State *l)
 			key = LuaToString(l, -2);
 			if (!strcmp(key, "Index")) {
 				tmp.Index = GetVariableIndex(LuaToString(l, -1));
+				Assert(tmp.Index != -1);
 			} else if (!strcmp(key, "Offset")) {
 				Assert(lua_istable(l, -1));
 				lua_rawgeti(l, -1, 1); // X
@@ -1613,10 +1279,6 @@ static int CclDefineDecorations(lua_State *l)
 				tmp.IsCenteredInY = LuaToBoolean(l, -1);
 			} else if (!strcmp(key, "ShowIfNotEnable")) {
 				tmp.ShowIfNotEnable = LuaToBoolean(l, -1);
-			} else if (!strcmp(key, "ShowWhenNull")) {
-				tmp.ShowWhenNull = LuaToBoolean(l, -1);
-			} else if (!strcmp(key, "HideHalf")) {
-				tmp.HideHalf = LuaToBoolean(l, -1);
 			} else if (!strcmp(key, "ShowWhenMax")) {
 				tmp.ShowWhenMax = LuaToBoolean(l, -1);
 			} else if (!strcmp(key, "ShowOnlySelected")) {
@@ -1633,51 +1295,7 @@ static int CclDefineDecorations(lua_State *l)
 				lua_rawgeti(l, -2, 2); // Data
 				Assert(lua_istable(l, -1));
 				key = LuaToString(l, -2);
-				if (!strcmp(key, "bar")) {
-					CDecoVarBar *decovarbar = new CDecoVarBar;
-					lua_pushnil(l);
-					while (lua_next(l, -2)) {
-						key = LuaToString(l, -2);
-						if (!strcmp(key, "Height")) {
-							decovarbar->Height = LuaToNumber(l, -1);
-						} else if (!strcmp(key, "Width")) {
-							decovarbar->Width = LuaToNumber(l, -1);
-						} else if (!strcmp(key, "Orientation")) {
-							key = LuaToString(l, -1);;
-							if (!strcmp(key, "horizontal")) {
-								decovarbar->IsVertical = 0;
-							} else if (!strcmp(key, "vertical")) {
-								decovarbar->IsVertical = 1;
-							} else { // Error
-								LuaError(l, "invalid Orientation '%s' for bar in DefineDecorations" _C_ key);
-							}
-						} else if (!strcmp(key, "SEToNW")) {
-							decovarbar->SEToNW = LuaToBoolean(l, -1);
-						} else if (!strcmp(key, "BorderSize")) {
-							decovarbar->BorderSize = LuaToNumber(l, -1);
-						} else if (!strcmp(key, "ShowFullBackground")) {
-							decovarbar->ShowFullBackground = LuaToBoolean(l, -1);
-#if 0 // FIXME Color configuration
-						} else if (!strcmp(key, "Color")) {
-							decovar->Color = // FIXME
-						} else if (!strcmp(key, "BColor")) {
-							decovar->BColor = // FIXME
-#endif
-						} else {
-							LuaError(l, "'%s' invalid for Method bar" _C_ key);
-						}
-						lua_pop(l, 1); // Pop value
-					}
-					decovar = decovarbar;
-				} else if (!strcmp(key, "text")) {
-					CDecoVarText *decovartext = new CDecoVarText;
-					lua_rawgeti(l, -1, 1);
-
-					decovartext->Font = CFont::Get(LuaToString(l, -1));
-					lua_pop(l, 1);
-// FIXME : More arguments ? color...
-					decovar = decovartext;
-				} else if (!strcmp(key, "sprite")) {
+				if (!strcmp(key, "sprite")) {
 					CDecoVarSpriteBar *decovarspritebar = new CDecoVarSpriteBar;
 					lua_rawgeti(l, -1, 1);
 					decovarspritebar->NSprite = GetSpriteIndex(LuaToString(l, -1));
@@ -1688,14 +1306,6 @@ static int CclDefineDecorations(lua_State *l)
 					lua_pop(l, 1);
 					// FIXME : More arguments ?
 					decovar = decovarspritebar;
-				} else if (!strcmp(key, "static-sprite")) {
-					CDecoVarStaticSprite *decovarstaticsprite = new CDecoVarStaticSprite;
-					lua_rawgeti(l, -1, 1); // sprite
-					lua_rawgeti(l, -2, 2); // frame
-					decovarstaticsprite->NSprite = GetSpriteIndex(LuaToString(l, -2));
-					decovarstaticsprite->n = LuaToNumber(l, -1);
-					lua_pop(l, 2);
-					decovar = decovarstaticsprite;
 				} else { // Error
 					LuaError(l, "invalid method '%s' for Method in DefineDecorations" _C_ key);
 				}
@@ -1713,8 +1323,6 @@ static int CclDefineDecorations(lua_State *l)
 		decovar->IsCenteredInX = tmp.IsCenteredInX;
 		decovar->IsCenteredInY = tmp.IsCenteredInY;
 		decovar->ShowIfNotEnable = tmp.ShowIfNotEnable;
-		decovar->ShowWhenNull = tmp.ShowWhenNull;
-		decovar->HideHalf = tmp.HideHalf;
 		decovar->ShowWhenMax = tmp.ShowWhenMax;
 		decovar->ShowOnlySelected = tmp.ShowOnlySelected;
 		decovar->HideNeutral = tmp.HideNeutral;
@@ -1740,10 +1348,9 @@ void UpdateUnitVariables(const CUnit *unit)
 
 	type = unit->Type;
 	for (i = 0; i < NVARALREADYDEFINED; i++) { // default values
-		if (i == ARMOR_INDEX || i == PIERCINGDAMAGE_INDEX || i == BASICDAMAGE_INDEX
-			|| i == MANA_INDEX || i == KILL_INDEX || i == XP_INDEX
-			|| i == BLOODLUST_INDEX || i == HASTE_INDEX || i == SLOW_INDEX
-			|| i == INVISIBLE_INDEX || i == UNHOLYARMOR_INDEX || i == HP_INDEX) {
+		if (i == ARMOR_INDEX || i == PIERCINGDAMAGE_INDEX ||
+				i == BASICDAMAGE_INDEX || i == MANA_INDEX ||
+				i == KILL_INDEX || i == HP_INDEX) {
 			continue;
 		}
 		unit->Variable[i].Value = 0;
@@ -1753,8 +1360,10 @@ void UpdateUnitVariables(const CUnit *unit)
 
 	// Build
 	if (unit->Orders[0]->Action == UnitActionBuilt) {
+		const int *pcosts = type->ProductionCosts;
+		int pcost = CYCLES_PER_SECOND * (pcosts[EnergyCost] ? pcosts[EnergyCost] : pcosts[MagmaCost]);
 		unit->Variable[BUILD_INDEX].Value = unit->Data.Built.Progress;
-		unit->Variable[BUILD_INDEX].Max = type->Stats[unit->Player->Index].Costs[TimeCost] * 600;
+		unit->Variable[BUILD_INDEX].Max = pcost;
 
 		// This should happen when building unit with several peons
 		// Maybe also with only one.
@@ -1769,49 +1378,26 @@ void UpdateUnitVariables(const CUnit *unit)
 	unit->Variable[TRANSPORT_INDEX].Value = unit->BoardCount;
 	unit->Variable[TRANSPORT_INDEX].Max = unit->Type->MaxOnBoard;
 
-	// Research.
-	if (unit->Orders[0]->Action == UnitActionResearch) {
-		unit->Variable[RESEARCH_INDEX].Value =
-			unit->Player->UpgradeTimers.Upgrades[unit->Data.Research.Upgrade->ID];
-		unit->Variable[RESEARCH_INDEX].Max = unit->Data.Research.Upgrade->Costs[TimeCost];
-	}
-
 	// Training
 	if (unit->Orders[0]->Action == UnitActionTrain) {
 		unit->Variable[TRAINING_INDEX].Value = unit->Data.Train.Ticks;
-		unit->Variable[TRAINING_INDEX].Max =
-			unit->Orders[0]->Type->Stats[unit->Player->Index].Costs[TimeCost];
-	}
-
-	// UpgradeTo
-	if (unit->Orders[0]->Action == UnitActionUpgradeTo) {
-		unit->Variable[UPGRADINGTO_INDEX].Value = unit->Data.UpgradeTo.Ticks;
-		unit->Variable[UPGRADINGTO_INDEX].Max =
-			unit->Orders[0]->Type->Stats[unit->Player->Index].Costs[TimeCost];
+		int *costs = unit->Orders[0]->Type->ProductionCosts;
+		int cost = costs[EnergyCost] ? costs[EnergyCost] : costs[MagmaCost];
+		unit->Variable[TRAINING_INDEX].Max = cost;
 	}
 
 	// Resources.
-	if (unit->Type->GivesResource) {
-		unit->Variable[GIVERESOURCE_INDEX].Value = unit->ResourcesHeld;
+	if (unit->Type->CanHarvestFrom) {
+		int i;
+		for (i = 0; i < MaxCosts; ++i) {
+			if (unit->ResourcesHeld[i] != 0) {
+				break;
+			}
+		}
+		Assert(i != MaxCosts);
+		unit->Variable[GIVERESOURCE_INDEX].Value = unit->ResourcesHeld[i] / CYCLES_PER_SECOND;
 		unit->Variable[GIVERESOURCE_INDEX].Max = 0x7FFFFFFF;
 	}
-	if (unit->Type->Harvester && unit->CurrentResource) {
-		unit->Variable[CARRYRESOURCE_INDEX].Value = unit->ResourcesHeld;
-		unit->Variable[CARRYRESOURCE_INDEX].Max = unit->Type->ResInfo[unit->CurrentResource]->ResourceCapacity;
-	}
-
-	// Supply
-	unit->Variable[SUPPLY_INDEX].Value = unit->Type->Supply;
-	unit->Variable[SUPPLY_INDEX].Max = unit->Player->Supply;
-	if (unit->Player->Supply < unit->Type->Supply) { // Come with 1st supply building.
-		unit->Variable[SUPPLY_INDEX].Value = unit->Variable[SUPPLY_INDEX].Max;
-	}
-	unit->Variable[SUPPLY_INDEX].Enable = unit->Type->Supply > 0;
-
-	// Demand
-	unit->Variable[DEMAND_INDEX].Value = unit->Type->Demand;
-	unit->Variable[DEMAND_INDEX].Max = unit->Player->Demand;
-	unit->Variable[DEMAND_INDEX].Enable = unit->Type->Demand > 0;
 
 	// SightRange
 	unit->Variable[SIGHTRANGE_INDEX].Value = type->Variable[SIGHTRANGE_INDEX].Value;
@@ -1870,7 +1456,7 @@ void UpdateUnitVariables(const CUnit *unit)
 	type->BoolFlag[CANATTACK_INDEX]             = type->CanAttack;
 	type->BoolFlag[BUILDEROUTSIDE_INDEX]        = type->BuilderOutside;
 	type->BoolFlag[BUILDERLOST_INDEX]           = type->BuilderLost;
-	type->BoolFlag[CANHARVEST_INDEX]            = type->CanHarvest;
+	type->BoolFlag[CANHARVESTFROM_INDEX]            = type->CanHarvestFrom;
 	type->BoolFlag[HARVESTER_INDEX]             = type->Harvester;
 	type->BoolFlag[SELECTABLEBYRECTANGLE_INDEX] = type->SelectableByRectangle;
 	type->BoolFlag[ISNOTSELECTABLE_INDEX]       = type->IsNotSelectable;
@@ -1885,17 +1471,15 @@ void UpdateUnitVariables(const CUnit *unit)
 void InitDefinedVariables()
 {
 	const char *var[NVARALREADYDEFINED] = {"HitPoints", "Build", "Mana", "Transport",
-		"Research", "Training", "UpgradeTo", "GiveResource", "CarryResource",
-		"Xp", "Kill", "Supply", "Demand", "Armor", "SightRange",
+		"Training", "GiveResource", "Kill", "Armor", "SightRange",
 		"AttackRange", "PiercingDamage", "BasicDamage", "PosX", "PosY", "RadarRange",
-		"RadarJammerRange", "AutoRepairRange", "Bloodlust", "Haste", "Slow", "Invisible",
-		"UnholyArmor", "Slot"
+		"RadarJammerRange", "AutoRepairRange", "Slot"
 		}; // names of the variable.
 	const char *boolflag = "DefineBoolFlags(\"Coward\", \"Building\", \"Flip\","
 		"\"Revealer\", \"LandUnit\", \"AirUnit\", \"SeaUnit\", \"ExplodeWhenKilled\","
 		"\"VisibleUnderFog\", \"AttackFromTransporter\","
 		"\"Vanishes\", \"GroundAttack\", \"ShoreBuilding\", \"CanAttack\","
-		"\"BuilderOutside\", \"BuilderLost\", \"CanHarvest\", \"Harvester\","
+		"\"BuilderOutside\", \"BuilderLost\", \"CanHarvestFrom\", \"Harvester\","
 		"\"SelectableByRectangle\", \"IsNotSelectable\", \"Decoration\","
 		"\"Indestructible\", \"Teleporter\")";
 	int i;
@@ -1918,20 +1502,12 @@ void InitDefinedVariables()
 void UnitTypeCclRegister(void)
 {
 	lua_register(Lua, "DefineUnitType", CclDefineUnitType);
-	lua_register(Lua, "DefineUnitStats", CclDefineUnitStats);
 	lua_register(Lua, "DefineBoolFlags", CclDefineBoolFlags);
 	lua_register(Lua, "DefineVariables", CclDefineVariables);
 	lua_register(Lua, "DefineDecorations", CclDefineDecorations);
 
 	InitDefinedVariables();
 
-	lua_register(Lua, "UnitType", CclUnitType);
-	lua_register(Lua, "UnitTypeArray", CclUnitTypeArray);
-	// unit type structure access
-	lua_register(Lua, "GetUnitTypeIdent", CclGetUnitTypeIdent);
-	lua_register(Lua, "GetUnitTypeName", CclGetUnitTypeName);
-	lua_register(Lua, "SetUnitTypeName", CclSetUnitTypeName);
-
 	lua_register(Lua, "DefineAnimations", CclDefineAnimations);
 }
 
