@@ -9,7 +9,7 @@
 //
 /**@name unit.cpp - The units. */
 //
-//      (c) Copyright 1998-2005 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: unit.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: unit.cpp 9029 2007-08-04 15:17:16Z feb $
 
 //@{
 
@@ -78,15 +78,13 @@ CUnit *ReleasedTail;                      /// List tail of released units.
 CUnit *Units[MAX_UNIT_SLOTS];             /// Array of used slots
 int NumUnits;                             /// Number of slots used
 
-int XpDamage;                             /// Hit point regeneration for all units
-bool EnableTrainingQueue;                 /// Config: training queues enabled
 bool EnableBuildingCapture;               /// Config: capture buildings enabled
-bool RevealAttacker;                      /// Config: reveal attacker enabled
 
 static unsigned long HelpMeLastCycle;     /// Last cycle HelpMe sound played
 static int HelpMeLastX;                   /// Last X coordinate HelpMe sound played
 static int HelpMeLastY;                   /// Last Y coordinate HelpMe sound played
 
+
 /*----------------------------------------------------------------------------
 --  Functions
 ----------------------------------------------------------------------------*/
@@ -135,7 +133,7 @@ void CUnit::RefsDecrease()
 }
 
 /**
-**  Release an unit.
+**  Release a unit.
 **
 **  The unit is only released, if all references are dropped.
 */
@@ -198,6 +196,8 @@ void CUnit::Release()
 	Refs = GameCycle + (NetworkMaxLag << 1); // could be reuse after this time
 	Type = 0;  // for debugging.
 
+	delete[] AutoCastSpell;
+	delete[] Variable;
 	for (std::vector<COrder *>::iterator order = Orders.begin(); order != Orders.end(); ++order) {
 		delete *order;
 	}
@@ -302,10 +302,13 @@ void CUnit::Init(CUnitType *type)
 			memset(AutoCastSpell, 0, SpellTypeTable.size());
 		}
 	}
-	Active = 1;
 
 	Removed = 1;
 
+	for (int i = 0; i < MaxCosts; ++i) {
+		ResourcesHeld[i] = type->ProductionCosts[i];
+	}
+
 	Assert(Orders.empty());
 
 	Orders.push_back(new COrder);
@@ -320,9 +323,6 @@ void CUnit::Init(CUnitType *type)
 	SavedOrder.Action = UnitActionStill;
 	SavedOrder.X = SavedOrder.Y = -1;
 	Assert(!SavedOrder.Goal);
-	CriticalOrder.Action = UnitActionStill;
-	CriticalOrder.X = CriticalOrder.Y = -1;
-	Assert(!CriticalOrder.Goal);
 }
 
 /**
@@ -343,7 +343,7 @@ void CUnit::AssignToPlayer(CPlayer *player)
 	if (!type->Vanishes && Orders[0]->Action != UnitActionDie) {
 		PlayerSlot = player->Units + player->TotalNumUnits++;
 		if (!SaveGameLoading) {
-			// If unit is dieing, it's already been lost by all players
+			// If unit is dying, it's already been lost by all players
 			// don't count again
 			if (type->Building) {
 				player->TotalBuildings++;
@@ -354,7 +354,6 @@ void CUnit::AssignToPlayer(CPlayer *player)
 		*PlayerSlot = this;
 
 		player->UnitTypesCount[type->Slot]++;
-		player->Demand += type->Demand; // food needed
 	}
 
 
@@ -442,7 +441,7 @@ static CUnit *GetFirstContainer(const CUnit *unit)
 	while (unit->Container) {
 		unit = unit->Container;
 	}
-	return (CUnit *)unit;
+	return const_cast<CUnit *>(unit);
 }
 
 /**
@@ -571,7 +570,7 @@ void MarkUnitFieldFlags(const CUnit *unit)
 	flags = type->FieldFlags;
 	for (h = type->TileHeight; h--;) {
 		for (w = type->TileWidth; w--;) {
-			Map.Fields[x + w + (y + h) * Map.Info.MapWidth].Flags |= flags;
+			Map.Field(x + w, y + h)->Flags |= flags;
 		}
 	}
 }
@@ -598,13 +597,12 @@ void UnmarkUnitFieldFlags(const CUnit *unit)
 	flags = type->FieldFlags;
 	for (h = type->TileHeight; h--;) {
 		for (w = type->TileWidth; w--;) {
-			Map.Fields[x + w + (y + h) * Map.Info.MapWidth].Flags &= ~flags;
+			Map.Field(x + w, y + h)->Flags &= ~flags;
 
 			int n = UnitCacheOnTile(x + w, y + h, table);
 			while (n--) {
 				if (table[n] != unit && table[n]->Orders[0]->Action != UnitActionDie) {
-					Map.Fields[x + w + (y + h) * Map.Info.MapWidth].Flags |=
-						table[n]->Type->FieldFlags;
+					Map.Field(x + w, y + h)->Flags |= table[n]->Type->FieldFlags;
 				}
 			}
 		}
@@ -659,7 +657,7 @@ static void RemoveUnitFromContainer(CUnit *unit)
 
 
 /**
-**  Affect Tile coord of an unit (with units inside) to tile (x, y).
+**  Affect Tile coord of a unit (with units inside) to tile (x, y).
 **
 **  @param unit    unit to move.
 **  @param x       X map tile position.
@@ -811,7 +809,7 @@ void CUnit::Remove(CUnit *host)
 /**
 **  Update information for lost units.
 **
-**  @param unit    Pointer to unit.
+**  @param unit  Pointer to unit.
 **
 **  @note Also called by ChangeUnitOwner
 */
@@ -821,7 +819,6 @@ void UnitLost(CUnit *unit)
 	CBuildRestrictionOnTop *b;
 	const CUnitType *type;
 	CPlayer *player;
-	int i;
 
 	Assert(unit);
 
@@ -862,56 +859,16 @@ void UnitLost(CUnit *unit)
 		}
 	}
 
-
-	//
-	//  Handle unit demand. (Currently only food supported.)
-	//
-	player->Demand -= type->Demand;
-
-	//
-	//  Update information.
-	//
-	if (unit->Orders[0]->Action != UnitActionBuilt) {
-		player->Supply -= type->Supply;
-
-		//
-		//  Handle income improvements, look if a player loses a building
-		//  which have given him a better income, find the next best
-		//  income.
-		//
-		for (i = 1; i < MaxCosts; ++i) {
-			if (player->Incomes[i] && type->ImproveIncomes[i] == player->Incomes[i]) {
-				int m;
-				int j;
-
-				m = DefaultIncomes[i];
-				for (j = 0; j < player->TotalNumUnits; ++j) {
-					if (m < player->Units[j]->Type->ImproveIncomes[i]) {
-						m = player->Units[j]->Type->ImproveIncomes[i];
-					}
-				}
-				player->Incomes[i] = m;
-			}
-		}
-	}
-
-	//
-	//  Handle research cancels.
-	//
-	if (unit->Orders[0]->Action == UnitActionResearch) {
-		unit->Player->UpgradeTimers.Upgrades[unit->Data.Research.Upgrade->ID] = 0;
-	}
-
 	DebugPrint("Lost %s(%d)\n" _C_ unit->Type->Ident.c_str() _C_ UnitNumber(unit));
 
 	// Destroy resource-platform, must re-make resource patch.
 	if ((b = OnTopDetails(unit, NULL)) != NULL) {
-		if (b->ReplaceOnDie && (unit->Type->GivesResource && unit->ResourcesHeld != 0)) {
+		if (b->ReplaceOnDie && (unit->Type->CanHarvestFrom && UnitHoldsResources(unit))) {
 			temp = MakeUnitAndPlace(unit->X, unit->Y, b->Parent, &Players[PlayerNumNeutral]);
 			if (temp == NoUnitP) {
 				DebugPrint("Unable to allocate Unit");
 			} else {
-				temp->ResourcesHeld = unit->ResourcesHeld;
+				memcpy(temp->ResourcesHeld, unit->ResourcesHeld, sizeof(temp->ResourcesHeld));
 			}
 		}
 	}
@@ -962,28 +919,15 @@ void UnitClearOrders(CUnit *unit)
 */
 void UpdateForNewUnit(const CUnit *unit, int upgrade)
 {
-	const CUnitType *type;
-	CPlayer *player;
-	int u;
-
-	player = unit->Player;
-	type = unit->Type;
-
-	//
-	// Handle unit supply. (Currently only food supported.)
-	// Note an upgraded unit can't give more supply.
-	//
-	if (!upgrade) {
-		player->Supply += type->Supply;
-	}
+	const CUnitType *type = unit->Type;
+	CPlayer *player = unit->Player;
 
 	//
 	// Update resources
 	//
-	for (u = 1; u < MaxCosts; ++u) {
-		if (player->Incomes[u] < unit->Type->ImproveIncomes[u]) {
-			player->Incomes[u] = unit->Type->ImproveIncomes[u];
-		}
+	for (int u = 0; u < MaxCosts; ++u) {
+		player->ProductionRate[u] += type->ProductionRate[u] * unit->ProductionEfficiency / 100;
+		player->StorageCapacity[u] += type->StorageCapacity[u];
 	}
 }
 
@@ -1034,8 +978,7 @@ static void UnitFillSeenValues(CUnit *unit)
 	unit->Seen.IY = unit->IY;
 	unit->Seen.IX = unit->IX;
 	unit->Seen.Frame = unit->Frame;
-	unit->Seen.State = (unit->Orders[0]->Action == UnitActionBuilt) |
-			((unit->Orders[0]->Action == UnitActionUpgradeTo) << 1);
+	unit->Seen.State = (unit->Orders[0]->Action == UnitActionBuilt);
 	if (unit->Orders[0]->Action == UnitActionDie) {
 		unit->Seen.State = 3;
 	}
@@ -1059,7 +1002,7 @@ void UnitGoesUnderFog(CUnit *unit, const CPlayer *player)
 		//
 		// Icky yucky icky Seen.Destroyed trickery.
 		// We track for each player if he's seen the unit as destroyed.
-		// Remember, an unit is marked Destroyed when it's gone as in
+		// Remember, a unit is marked Destroyed when it's gone as in
 		// completely gone, the corpses vanishes. In that case the unit
 		// only survives since some players did NOT see the unit destroyed.
 		// Keeping trackof that is hard, mostly due to complex shared vision
@@ -1079,7 +1022,7 @@ void UnitGoesUnderFog(CUnit *unit, const CPlayer *player)
 }
 
 /**
-**  This function should get called when an unit goes out of fog of war.
+**  This function should get called when a unit goes out of fog of war.
 **
 **  @param unit    The unit that goes out of fog.
 **  @param player  The player the unit goes out of fog for.
@@ -1180,8 +1123,8 @@ void UnitsOnTileUnmarkSeen(const CPlayer *player, int x, int y)
 }
 
 /**
-**  Recalculates an units visiblity count. This happens really often,
-**  Like every time an unit moves. It's really fast though, since we
+**  Recalculates a units visiblity count. This happens really often,
+**  Like every time a unit moves. It's really fast though, since we
 **  have per-tile counts.
 **
 **  @param unit    pointer to the unit to check if seen
@@ -1216,7 +1159,7 @@ void UnitCountSeen(CUnit *unit)
 			for (x = 0; x < unit->Type->TileWidth; ++x) {
 				for (y = 0; y < unit->Type->TileHeight; ++y) {
 					//  Icky ugly code trick. With NoFogOfWar we have to be > 0;
-					if (Map.Fields[(unit->Y + y) * Map.Info.MapWidth + unit->X + x].Visible[p] >
+					if (Map.Field(unit->X + x, unit->Y + y)->Visible[p] >
 							1 - (Map.NoFogOfWar ? 1 : 0)) {
 						newv++;
 					}
@@ -1276,13 +1219,6 @@ bool CUnit::IsVisible(const CPlayer *player) const
 */
 bool CUnit::IsVisibleAsGoal(const CPlayer *player) const
 {
-	//
-	// Invisibility
-	//
-	if (Variable[INVISIBLE_INDEX].Value && (player != Player) &&
-			(!player->IsBothSharedVision(Player))) {
-		return false;
-	}
 	if (IsVisible(player) || player->Type == PlayerComputer ||
 			UnitVisibleOnRadar(player, this)) {
 		return !Removed && !Destroyed && Orders[0]->Action != UnitActionDie;
@@ -1303,14 +1239,6 @@ bool CUnit::IsVisibleAsGoal(const CPlayer *player) const
 */
 bool CUnit::IsVisibleOnMap(const CPlayer *player) const
 {
-	//
-	// Invisible units.
-	//
-	if (Variable[INVISIBLE_INDEX].Value && player != Player &&
-			!player->IsBothSharedVision(Player)) {
-		return false;
-	}
-
 	return !Removed && !Destroyed &&
 		Orders[0]->Action != UnitActionDie && IsVisible(player);
 }
@@ -1325,13 +1253,6 @@ bool CUnit::IsVisibleOnMap(const CPlayer *player) const
 */
 bool CUnit::IsVisibleOnMinimap() const
 {
-	//
-	// Invisible units.
-	//
-	if (Variable[INVISIBLE_INDEX].Value && (ThisPlayer != Player) &&
-			(!ThisPlayer->IsBothSharedVision(Player))) {
-		return false;
-	}
 	if (IsVisible(ThisPlayer) || ReplayRevealMap ||
 			UnitVisibleOnRadar(ThisPlayer, this)) {
 		return !Removed && !Destroyed && (Orders[0]->Action != UnitActionDie);
@@ -1373,12 +1294,6 @@ bool CUnit::IsVisibleInViewport(const CViewport *vp) const
 		return false;
 	}
 
-	// Those are never ever visible.
-	if (Variable[INVISIBLE_INDEX].Value && ThisPlayer != Player &&
-			!ThisPlayer->IsBothSharedVision(Player)) {
-		return false;
-	}
-
 	if (IsVisible(ThisPlayer) || ReplayRevealMap) {
 		return !Destroyed;
 	} else {
@@ -1479,11 +1394,9 @@ void CUnit::ChangeOwner(CPlayer *newplayer)
 	//
 	//  Must change food/gold and other.
 	//
-	if (Type->GivesResource) {
+	if (Type->CanHarvestFrom) {
 		DebugPrint("Resource transfer not supported\n");
 	}
-	newplayer->Demand += Type->Demand;
-	newplayer->Supply += Type->Supply;
 	if (Type->Building) {
 		newplayer->NumBuildings++;
 	}
@@ -1492,6 +1405,7 @@ void CUnit::ChangeOwner(CPlayer *newplayer)
 	UpdateForNewUnit(this, 1);
 }
 
+#if 0
 /**
 **  Change the owner of all units of a player.
 **
@@ -1519,6 +1433,7 @@ static void ChangePlayerOwner(CPlayer *oldplayer, CPlayer *newplayer)
 		unit->RescuedFrom = oldplayer;
 	}
 }
+#endif
 
 /**
 **  Rescue units.
@@ -1576,16 +1491,17 @@ void RescueUnits(void)
 				//  Look if ally near the unit.
 				//
 				for (i = 0; i < n; ++i) {
-					if (around[i]->Type->CanAttack &&
-							unit->IsAllied(around[i])) {
+					if (unit->IsAllied(around[i])) {
+#if 0
 						//
 						//  City center converts complete player
-						//  NOTE: I use a trick here, centers could
-						//        store gold. FIXME!!!
-						if (unit->Type->CanStore[GoldCost]) {
+						//  FIXME: what unit should we use here?
+						//
+						if (0) {
 							ChangePlayerOwner(p, around[i]->Player);
 							break;
 						}
+#endif
 						unit->RescuedFrom = unit->Player;
 						unit->ChangeOwner(around[i]->Player);
 						unit->Blink = 5;
@@ -1936,7 +1852,9 @@ CBuildRestrictionOnTop *OnTopDetails(const CUnit *unit, const CUnitType *parent)
 	return NULL;
 }
 
-// Run Distance Checking
+/**
+**  Check And Restriction
+*/
 bool CBuildRestrictionAnd::Check(const CUnitType *type, int x, int y, CUnit *&ontoptarget) const
 {
 	for (std::vector<CBuildRestriction*>::const_iterator i = _or_list.begin();
@@ -1948,6 +1866,9 @@ bool CBuildRestrictionAnd::Check(const CUnitType *type, int x, int y, CUnit *&on
 	return true;
 }
 
+/**
+**  Check Distance Restriction
+*/
 bool CBuildRestrictionDistance::Check(const CUnitType *type, int x, int y, CUnit *&ontoptarget) const
 {
 	CUnit *table[UnitMax];
@@ -1960,9 +1881,9 @@ bool CBuildRestrictionDistance::Check(const CUnitType *type, int x, int y, CUnit
 	int distance = 0;
 
 	if (this->DistanceType == LessThanEqual ||
-		this->DistanceType == GreaterThan ||
-		this->DistanceType == Equal ||
-		this->DistanceType == NotEqual) {
+			this->DistanceType == GreaterThan ||
+			this->DistanceType == Equal ||
+			this->DistanceType == NotEqual) {
 		x1 = x - this->Distance > 0 ? x - this->Distance : 0;
 		y1 = y - this->Distance > 0 ? y - this->Distance : 0;
 		x2 = x + type->TileWidth + this->Distance < Map.Info.MapWidth ?
@@ -2021,7 +1942,9 @@ bool CBuildRestrictionDistance::Check(const CUnitType *type, int x, int y, CUnit
 	return false;
 }
 
-// Check AddOn Restriction
+/**
+**  Check AddOn Restriction
+*/
 bool CBuildRestrictionAddOn::Check(const CUnitType *type, int x, int y, CUnit *&ontoptarget) const
 {
 	CUnit *table[UnitMax];
@@ -2038,7 +1961,7 @@ bool CBuildRestrictionAddOn::Check(const CUnitType *type, int x, int y, CUnit *&
 		n = UnitCacheOnTile(x1, y1, table);
 		for (i = 0; i < n; ++i) {
 			if (table[i]->Type == this->Parent &&
-				table[i]->X == x1 && table[i]->Y == y1) {
+					table[i]->X == x1 && table[i]->Y == y1) {
 				return true;
 			}
 		}
@@ -2046,33 +1969,37 @@ bool CBuildRestrictionAddOn::Check(const CUnitType *type, int x, int y, CUnit *&
 	return false;
 }
 
-
-// Check AddOn Restriction
+/**
+**  Check OnTop Restriction
+*/
 bool CBuildRestrictionOnTop::Check(const CUnitType *type, int x, int y, CUnit *&ontoptarget) const
 {
 	CUnit *table[UnitMax];
 	int n;
 	int i;
 
+	ontoptarget = NULL;
 	n = UnitCacheOnTile(x, y, table);
 	for (i = 0; i < n; ++i) {
-		if (table[i]->Type == this->Parent &&
-			table[i]->X == x && table[i]->Y == y &&
-			table[i]->Orders[0]->Action != UnitActionBuilt &&
-			!table[i]->Destroyed &&
-			table[i]->Orders[0]->Action != UnitActionDie) {
-			ontoptarget = table[i];
-			return true;
+		if (table[i]->X == x && table[i]->Y == y && !table[i]->Destroyed &&
+				table[i]->Orders[0]->Action != UnitActionDie) {
+			if (table[i]->Type == this->Parent &&
+					table[i]->Orders[0]->Action != UnitActionBuilt) {
+				// Available to build on
+				ontoptarget = table[i];
+			} else {
+				// Something else is built on this already
+				ontoptarget = NULL;
+				return false;
+			}
 		}
 	}
-	return false;
+	return ontoptarget != NULL;
 }
 
 
 /**
 **  Can build unit here.
-**  Hall to near to goldmine.
-**  Refinery or shipyard to near to oil patch.
 **
 **  @param unit  Unit doing the building
 **  @param type  unit-type to be checked.
@@ -2084,9 +2011,6 @@ bool CBuildRestrictionOnTop::Check(const CUnitType *type, int x, int y, CUnit *&
 CUnit *CanBuildHere(const CUnit *unit, const CUnitType *type, int x, int y)
 {
 	CUnit *ontoptarget;
-	int w;
-	int h;
-	int success;
 
 	//
 	//  Can't build outside the map
@@ -2100,15 +2024,15 @@ CUnit *CanBuildHere(const CUnit *unit, const CUnitType *type, int x, int y)
 
 	// Must be checked before oil!
 	if (type->ShoreBuilding) {
-		success = 0;
+		bool success = false;
 
 		// Need at least one coast tile
-		for (h = type->TileHeight; h--;) {
-			for (w = type->TileWidth; w--;) {
-				if (Map.Fields[x + w + (y + h) * Map.Info.MapWidth].Flags &
-						MapFieldCoastAllowed) {
+		for (int h = type->TileHeight; h--;) {
+			for (int w = type->TileWidth; w--;) {
+				if (Map.Field(x + w, y + h)->Flags & MapFieldCoastAllowed) {
 					h = w = 0;
-					success = 1;
+					success = true;
+					break;
 				}
 			}
 		}
@@ -2120,13 +2044,12 @@ CUnit *CanBuildHere(const CUnit *unit, const CUnitType *type, int x, int y)
 	if (type->BuildingRules.empty()) {
 		return (unit == NULL) ? (CUnit *)1 : const_cast<CUnit *>(unit);
 	}
+
 	ontoptarget = NULL;
 	for (std::vector<CBuildRestriction *>::const_iterator ib = type->BuildingRules.begin();
-		ib != type->BuildingRules.end(); ++ib) {
-		const CBuildRestriction *b = *ib;
-
+			ib != type->BuildingRules.end(); ++ib) {
 		// All checks processed, did we really have success
-		if (b->Check(type, x, y, ontoptarget)) {
+		if ((*ib)->Check(type, x, y, ontoptarget)) {
 			// We passed a full ruleset return
 			if (unit == NULL) {
 				return ontoptarget ? ontoptarget : (CUnit *)1;
@@ -2152,11 +2075,11 @@ bool CanBuildOn(int x, int y, int mask)
 	if (x < 0 || y < 0 || x >= Map.Info.MapWidth || y >= Map.Info.MapHeight) {
 		return false;
 	}
-	return (Map.Fields[x + y * Map.Info.MapWidth].Flags & mask) ? false : true;
+	return (Map.Field(x, y)->Flags & mask) ? false : true;
 }
 
 /**
-**  Can build unit-type on this point.
+**  Can build unit-type at this point.
 **
 **  @param unit  Worker that want to build the building or NULL.
 **  @param type  Building unit-type.
@@ -2165,12 +2088,9 @@ bool CanBuildOn(int x, int y, int mask)
 **  @param real  Really build, or just placement
 **
 **  @return      OnTop, parent unit, builder on true, NULL false.
-**
 */
 CUnit *CanBuildUnitType(const CUnit *unit, const CUnitType *type, int x, int y, int real)
 {
-	int w;
-	int h;
 	int j;
 	int testmask;
 	CPlayer *player;
@@ -2181,7 +2101,7 @@ CUnit *CanBuildUnitType(const CUnit *unit, const CUnitType *type, int x, int y,
 	if (ontop == NULL) {
 		return NULL;
 	}
-	if (ontop != (CUnit *) 1 && ontop != unit) {
+	if (ontop != (CUnit *)1 && ontop != unit) {
 		return ontop;
 	}
 
@@ -2199,8 +2119,8 @@ CUnit *CanBuildUnitType(const CUnit *unit, const CUnitType *type, int x, int y,
 		player = unit->Player;
 	}
 
-	for (h = type->TileHeight; h--;) {
-		for (w = type->TileWidth; w--;) {
+	for (int h = type->TileHeight; h--;) {
+		for (int w = type->TileWidth; w--;) {
 			if (player && !real) {
 				testmask = MapFogFilterFlags(player, x + w, y + h, type->MovementMask);
 			} else {
@@ -2252,7 +2172,7 @@ CUnit *CanBuildUnitType(const CUnit *unit, const CUnitType *type, int x, int y,
 **  Range is not circular, but square.
 **  Player is ignored if nil(search the entire map)
 **  Use rvresult if you search for a tile that doesn't
-**  match resmask. Like for a tile where an unit can go
+**  match resmask. Like for a tile where a unit can go
 **  with it's movement mask.
 **
 **  @return            True if wood was found.
@@ -2354,6 +2274,23 @@ int FindTerrainType(int movemask, int resmask, int rvresult, int range,
 	return 0;
 }
 
+/**
+**  Check if a unit holds any resources
+**
+**  @param unit  Unit to check
+**
+**  @return      True if the unit holds resources, false if it doesn't
+*/
+bool UnitHoldsResources(const CUnit *unit)
+{
+	for (int i = 0; i < MaxCosts; ++i) {
+		if (unit->ResourcesHeld[i] != 0) {
+			return true;
+		}
+	}
+	return false;
+}
+
 /**
 **  Find Resource.
 **
@@ -2385,12 +2322,10 @@ CUnit *UnitFindResource(const CUnit *unit, int x, int y, int range, int resource
 	int ep;
 	int i;
 	int w;
-	int n;
 	unsigned char *m;
 	unsigned char *matrix;
-	const CUnit *destu;
-	CUnit *mine;
-	CUnit *bestmine;
+	CUnit *res;
+	CUnit *bestres;
 	int destx;
 	int desty;
 	int bestd;
@@ -2402,19 +2337,15 @@ CUnit *UnitFindResource(const CUnit *unit, int x, int y, int range, int resource
 		Map.Info.MapWidth * Map.Info.MapHeight / 4 : range * range * 5;
 	points = new p[size];
 
-	// Find the nearest gold depot
-	if ((destu = FindDeposit(unit, x, y,range,resource))) {
-		NearestOfUnit(destu, x, y, &destx, &desty);
-	}
 	bestd = 99999;
 	// Make movement matrix. FIXME: can create smaller matrix.
 	matrix = CreateMatrix();
 	w = Map.Info.MapWidth + 2;
 	matrix += w + w + 2;
-	//  Unit movement mask
+	// Unit movement mask
 	mask = unit->Type->MovementMask;
-	//  Ignore all units along the way. Might seem wierd, but otherwise
-	//  peasants would lock at a mine with a lot of workers.
+	// Ignore all units along the way. Might seem weird, but otherwise
+	// peasants would lock at a resource with a lot of workers.
 	mask &= ~(MapFieldLandUnit | MapFieldSeaUnit | MapFieldAirUnit);
 	points[0].X = x;
 	points[0].Y = y;
@@ -2422,7 +2353,7 @@ CUnit *UnitFindResource(const CUnit *unit, int x, int y, int range, int resource
 	matrix[x + y * w] = 1; // mark start point
 	ep = wp = 1; // start with one point
 	cdist = 0; // current distance is 0
-	bestmine = NoUnitP;
+	bestres = NoUnitP;
 
 	//
 	// Pop a point from stack, push all neighbors which could be entered.
@@ -2444,24 +2375,15 @@ CUnit *UnitFindResource(const CUnit *unit, int x, int y, int range, int resource
 				}
 
 				//
-				// Look if there is a mine
+				// Look if there is a resource
 				//
-				if ((mine = ResourceOnMap(x, y, resource)) &&
-						mine->Type->CanHarvest &&
-						(mine->Player->Index == PlayerMax - 1 ||
-							mine->Player == unit->Player ||
-							(unit->IsAllied(mine) && mine->IsAllied(unit)))) {
-					if (destu) {
-						n = (abs(destx - x) > abs(desty - y)) ? abs(destx - x) : abs(desty - y);
-						if (n < bestd) {
-							bestd = n;
-							bestmine = mine;
-						}
-						*m = 99;
-					} else { // no goal take the first
-						delete[] points;
-						return mine;
-					}
+				if ((res = ResourceOnMap(x, y, resource)) &&
+						res->Type->CanHarvestFrom &&
+						(res->Player->Index == PlayerMax - 1 ||
+							res->Player == unit->Player ||
+							(unit->IsAllied(res) && res->IsAllied(unit)))) {
+					delete[] points;
+					return res;
 				}
 
 				if (CanMoveToMask(x, y, mask)) { // reachable
@@ -2486,129 +2408,7 @@ CUnit *UnitFindResource(const CUnit *unit, int x, int y, int range, int resource
 		// Take best of this frame, if any.
 		if (bestd != 99999) {
 			delete[] points;
-			return bestmine;
-		}
-		++cdist;
-		if (rp == wp || cdist >= range) { // unreachable, no more points available
-			break;
-		}
-		// Continue with next set.
-		ep = wp;
-	}
-	delete[] points;
-	return NoUnitP;
-}
-
-/**
-**  Find deposit. This will find a deposit for a resource
-**
-**  @param unit        The unit that wants to find a resource.
-**  @param x           Closest to x
-**  @param y           Closest to y
-**  @param range       Maximum distance to the deposit.
-**  @param resource    Resource to find deposit from.
-**
-**  @note This will return a reachable allied depot.
-**
-**  @return            NoUnitP or deposit unit
-*/
-CUnit *FindDeposit(const CUnit *unit, int x, int y, int range, int resource)
-{
-	static const int xoffset[] = {  0,-1,+1, 0, -1,+1,-1,+1 };
-	static const int yoffset[] = { -1, 0, 0,+1, -1,-1,+1,+1 };
-	struct p {
-		unsigned short X;
-		unsigned short Y;
-	} *points;
-	int size;
-	int rx;
-	int ry;
-	int mask;
-	int wp;
-	int rp;
-	int ep;
-	int i;
-	int w;
-	int nodes_searched;
-	unsigned char *m;
-	unsigned char *matrix;
-	CUnit *depot;
-	int destx;
-	int desty;
-	int cdist;
-
-	nodes_searched = 0;
-
-	destx = x;
-	desty = y;
-	size = (Map.Info.MapWidth * Map.Info.MapHeight / 4 < range * range * 5) ?
-		Map.Info.MapWidth * Map.Info.MapHeight / 4 : range * range * 5;
-	points = new p[size];
-
-	// Make movement matrix. FIXME: can create smaller matrix.
-	matrix = CreateMatrix();
-	w = Map.Info.MapWidth + 2;
-	matrix += w + w + 2;
-	//  Unit movement mask
-	mask = unit->Type->MovementMask;
-	//  Ignore all units along the way. Might seem wierd, but otherwise
-	//  peasants would lock at a mine with a lot of workers.
-	mask &= ~(MapFieldLandUnit | MapFieldSeaUnit | MapFieldAirUnit | MapFieldBuilding);
-	points[0].X = x;
-	points[0].Y = y;
-	rp = 0;
-	matrix[x + y * w] = 1; // mark start point
-	ep = wp = 1; // start with one point
-	cdist = 0; // current distance is 0
-
-	//
-	// Pop a point from stack, push all neighbors which could be entered.
-	//
-	for (;;) {
-		while (rp != ep) {
-			rx = points[rp].X;
-			ry = points[rp].Y;
-			for (i = 0; i < 8; ++i) { // mark all neighbors
-				x = rx + xoffset[i];
-				y = ry + yoffset[i];
-				++nodes_searched;
-				//  Make sure we don't leave the map.
-				if (x < 0 || y < 0 || x >= Map.Info.MapWidth || y >= Map.Info.MapHeight) {
-					continue;
-				}
-				m = matrix + x + y * w;
-				//  Check if visited or unexplored
-				if (*m || !Map.IsFieldExplored(unit->Player, x, y)) {
-					continue;
-				}
-				//
-				// Look if there is a deposit
-				//
-				if ((depot = ResourceDepositOnMap(x, y, resource)) &&
-						((unit->IsAllied(depot) && depot->IsAllied(unit)) ||
-							(unit->Player == depot->Player))) {
-					delete[] points;
-					return depot;
-				}
-				if (CanMoveToMask(x, y, mask)) { // reachable
-					*m = 1;
-					points[wp].X = x; // push the point
-					points[wp].Y = y;
-					if (++wp >= size) { // round about
-						wp = 0;
-					}
-					if (wp == ep) {
-						//  We are out of points, give up!
-						DebugPrint("Ran out of points the hard way, beware.\n");
-						break;
-					}
-				} else { // unreachable
-					*m = 99;
-				}
-			}
-			if (++rp >= size) { // round about
-				rp = 0;
-			}
+			return bestres;
 		}
 		++cdist;
 		if (rp == wp || cdist >= range) { // unreachable, no more points available
@@ -2648,7 +2448,7 @@ CUnit *FindIdleWorker(const CPlayer *player, const CUnit *last)
 
 	for (i = 0; i < nunits; ++i) {
 		unit = player->Units[i];
-		if (unit->Type->Harvester && unit->Type->ResInfo && !unit->Removed) {
+		if (unit->Type->Harvester && !unit->Removed) {
 			if (unit->Orders[0]->Action == UnitActionStill) {
 				if (SelectNextUnit && !IsOnlySelected(unit)) {
 					return unit;
@@ -2675,48 +2475,28 @@ CUnit *FindIdleWorker(const CPlayer *player, const CUnit *last)
   ----------------------------------------------------------------------------*/
 
 /**
-**  Unit on map screen.
+**  Select unit on screen. (x, y are in pixels relative to map 0,0).
 **
-**  Select units on screen. (x, y are in pixels relative to map 0,0).
-**  Not GAMEPLAY safe, uses ReplayRevealMap
+**  @param x  X pixel position.
+**  @param y  Y pixel position.
 **
-**  More units on same position.
-**    Cycle through units. ounit is the old one.
-**    First take highest unit.
-**
-**  @param ounit  Old selected unit.
-**  @param x      X pixel position.
-**  @param y      Y pixel position.
-**
-**  @return       An unit on x, y position.
+**  @return   Unit on x, y position.
 */
-CUnit *UnitOnScreen(CUnit *ounit, int x, int y)
+CUnit *UnitOnScreen(int x, int y)
 {
-	CUnit **table;
-	CUnit *unit;
-	CUnit *nunit;
-	CUnit *funit; // first possible unit
-	CUnitType *type;
-	int flag; // flag take next unit
-	int gx;
-	int gy;
+	CUnit *bestUnit = NULL;
 
-	funit = NULL;
-	nunit = NULL;
-	flag = 0;
-	if (!ounit) { // no old on this position
-		flag = 1;
-	}
-	for (table = Units; table < Units + NumUnits; ++table) {
-		unit = *table;
+	for (int i = 0; i < NumUnits; ++i) {
+		CUnit *unit = Units[i];
+		CUnitType *type = unit->Type;
+		int gx, gy;
+
+		// Must be visible
 		if (!unit->IsVisibleAsGoal(ThisPlayer) && !ReplayRevealMap) {
 			continue;
 		}
-		type = unit->Type;
 
-		//
-		// Check if mouse is over the unit.
-		//
+		// Mouse is in the unit's box
 		gx = unit->X * TileSizeX + unit->IX;
 		if (x + (type->BoxWidth - type->TileWidth * TileSizeX) / 2 < gx) {
 			continue;
@@ -2733,28 +2513,34 @@ CUnit *UnitOnScreen(CUnit *ounit, int x, int y)
 			continue;
 		}
 
-		//
 		// This could be taken.
-		//
-		if (flag) {
-			return unit;
+		if (!bestUnit || unit->Type->DrawLevel > bestUnit->Type->DrawLevel) {
+			bestUnit = unit;
 		}
-		if (unit == ounit) {
-			flag = 1;
-		} else if (!funit) {
-			funit = unit;
-		}
-		nunit=unit;
 	}
 
-	if (flag && funit) {
-		return funit;
-	}
-	return nunit;
+	return bestUnit;
 }
 
 /**
-**  Let an unit die.
+**  Check if a unit should be removed from UnitsConsumingResources
+*/
+void UnitRemoveConsumingResources(CUnit *unit)
+{
+	if ((unit->Orders[0]->Action == UnitActionBuild && !unit->Type->BuilderOutside && unit->SubAction == 40) ||
+			(unit->Orders[0]->Action == UnitActionRepair && unit->SubAction == 20)) {
+		unit->Player->RemoveFromUnitsConsumingResources(unit);
+	} else if (unit->Orders[0]->Action == UnitActionResource && unit->SubAction >= 55) {
+		for (int u = 0; u < MaxCosts; ++u) {
+			unit->Player->ProductionRate[u] -= unit->Data.Harvest.CurrentProduction[u];
+		}
+	} else if (unit->Orders[0]->Action == UnitActionTrain && unit->SubAction != 0) {
+		unit->Player->RemoveFromUnitsConsumingResources(unit);
+	}
+}
+
+/**
+**  Let a unit die.
 **
 **  @param unit    Unit to be destroyed.
 */
@@ -2798,14 +2584,25 @@ void LetUnitDie(CUnit *unit)
 		unit->Goal = NULL;
 	}
 
+	UnitRemoveConsumingResources(unit);
+	if (unit->Orders[0]->Action != UnitActionBuilt) {
+		for (int u = 0; u < MaxCosts; ++u) {
+			unit->Player->ProductionRate[u] -= unit->Type->ProductionRate[u] * unit->ProductionEfficiency / 100;
+			unit->Player->StorageCapacity[u] -= unit->Type->StorageCapacity[u];
+			if (unit->Player->StoredResources[u] > unit->Player->StorageCapacity[u]) {
+				unit->Player->StoredResources[u] = unit->Player->StorageCapacity[u];
+			}
+		}
+	}
+
 	// During resource build, the worker holds the resource amount,
 	// but if canceling building the platform, the worker is already
 	// outside.
-	if (type->GivesResource &&
+	if (type->CanHarvestFrom &&
 			unit->Orders[0]->Action == UnitActionBuilt &&
 			unit->Data.Built.Worker) {
 		// Restore value for oil-patch
-		unit->ResourcesHeld = unit->Data.Built.Worker->ResourcesHeld;
+		memcpy(unit->ResourcesHeld, unit->Data.Built.Worker->ResourcesHeld, sizeof(unit->ResourcesHeld));
 	}
 
 	// Transporters lose their units and building their workers
@@ -2897,8 +2694,7 @@ void HitUnit(CUnit *attacker, CUnit *target, int damage)
 
 	Assert(damage != 0 && target->Orders[0]->Action != UnitActionDie && !target->Type->Vanishes);
 
-	if (target->Variable[UNHOLYARMOR_INDEX].Value > 0 || target->Type->Indestructible) {
-		// vladi: units with active UnholyArmour are invulnerable
+	if (target->Type->Indestructible) {
 		return;
 	}
 
@@ -2961,12 +2757,6 @@ void HitUnit(CUnit *attacker, CUnit *target, int damage)
 			} else {
 				attacker->Player->TotalKills++;
 			}
-			if (UseHPForXp) {
-				attacker->Variable[XP_INDEX].Max += target->Variable[HP_INDEX].Value;
-			} else {
-				attacker->Variable[XP_INDEX].Max += target->Type->Points;
-			}
-			attacker->Variable[XP_INDEX].Value = attacker->Variable[XP_INDEX].Max;
 			attacker->Variable[KILL_INDEX].Value++;
 			attacker->Variable[KILL_INDEX].Max++;
 			attacker->Variable[KILL_INDEX].Enable = 1;
@@ -2975,10 +2765,6 @@ void HitUnit(CUnit *attacker, CUnit *target, int damage)
 		return;
 	}
 	target->Variable[HP_INDEX].Value -= damage;
-	if (UseHPForXp && attacker && target->IsEnemy(attacker)) {
-		attacker->Variable[XP_INDEX].Value += damage;
-		attacker->Variable[XP_INDEX].Max += damage;
-	}
 
 	// FIXME: this is dumb. I made repairers capture. crap.
 	// david: capture enemy buildings
@@ -3045,10 +2831,11 @@ void HitUnit(CUnit *attacker, CUnit *target, int damage)
 	//
 	if (attacker && !type->Coward) {
 		if (type->CanAttack) {
-			if (RevealAttacker && CanTarget(target->Type, attacker->Type)) {
-				// Reveal Unit that is attacking
+			if (CanTarget(target->Type, attacker->Type)) {
+				// Attack unit that is attacking
 				goal = attacker;
 			} else {
+				// Check for any other units in range
 				goal = AttackUnitsInReactRange(target);
 			}
 			if (goal) {
@@ -3447,6 +3234,7 @@ bool CUnit::IsUnusable() const
 		this->Orders[0]->Action == UnitActionBuilt || this->Destroyed;
 }
 
+
 /*----------------------------------------------------------------------------
 --  SAVE/LOAD
 ----------------------------------------------------------------------------*/
@@ -3505,12 +3293,6 @@ void SaveOrder(const COrder *order, CFile *file)
 		case UnitActionTrain:
 			file->printf("\"action-train\",");
 			break;
-		case UnitActionUpgradeTo:
-			file->printf("\"action-upgrade-to\",");
-			break;
-		case UnitActionResearch:
-			file->printf("\"action-research\",");
-			break;
 		case UnitActionBuilt:
 			file->printf("\"action-built\",");
 			break;
@@ -3534,12 +3316,6 @@ void SaveOrder(const COrder *order, CFile *file)
 		case UnitActionResource:
 			file->printf("\"action-resource\",");
 			break;
-		case UnitActionReturnGoods:
-			file->printf("\"action-return-goods\",");
-			break;
-		case UnitActionTransformInto:
-			file->printf("\"action-transform-into\",");
-			break;
 		default:
 			DebugPrint("Unknown action in order\n");
 	}
@@ -3570,15 +3346,6 @@ void SaveOrder(const COrder *order, CFile *file)
 				file->printf(" \"spell\", \"%s\",", order->Arg1.Spell->Ident.c_str());
 			}
 			break;
-		case UnitActionResearch:
-			if (order->Arg1.Upgrade) {
-				file->printf(" \"upgrade\", \"%s\",", order->Arg1.Upgrade->Ident.c_str());
-			}
-			break;
-		case UnitActionResource :
-		case UnitActionReturnGoods :
-			file->printf(" \"mine\", %d,", order->Arg1.ResourcePos);
-			break;
 		default:
 			break;
 	}
@@ -3586,7 +3353,7 @@ void SaveOrder(const COrder *order, CFile *file)
 }
 
 /**
-**  Save the state of an unit to file.
+**  Save the state of a unit to file.
 **
 **  @param unit    Unit pointer to be saved.
 **  @param file    Output file.
@@ -3681,9 +3448,6 @@ void SaveUnit(const CUnit *unit, CFile *file)
 		file->printf(" \"seen-constructed\",");
 	}
 	file->printf(" \"seen-state\", %d, ", unit->Seen.State);
-	if (unit->Active) {
-		file->printf(" \"active\",");
-	}
 	file->printf("\"ttl\", %lu, ", unit->TTL);
 
 	for (i = 0; i < UnitTypeVar.NumberVariable; i++) {
@@ -3695,11 +3459,13 @@ void SaveUnit(const CUnit *unit, CFile *file)
 	file->printf("\"group-id\", %d,\n  ", unit->GroupId);
 	file->printf("\"last-group\", %d,\n  ", unit->LastGroup);
 
-	file->printf("\"resources-held\", %d,\n  ", unit->ResourcesHeld);
-	if (unit->CurrentResource) {
-		file->printf("\"current-resource\", \"%s\",\n  ",
-			DefaultResourceNames[unit->CurrentResource]);
+	file->printf("\"resources-held\", {");
+	for (i = 0; i < MaxCosts; ++i) {
+		file->printf("%s%d", (i ? ", " : ""), unit->ResourcesHeld[i]);
 	}
+	file->printf("},\n  ");
+
+	file->printf("\"production-efficiency\", %d, ", unit->ProductionEfficiency);
 
 	file->printf("\"sub-action\", %d, ", unit->SubAction);
 	file->printf("\"wait\", %d, ", unit->Wait);
@@ -3753,8 +3519,6 @@ void SaveUnit(const CUnit *unit, CFile *file)
 	}
 	file->printf("},\n  \"saved-order\", ");
 	SaveOrder(&unit->SavedOrder, file);
-	file->printf(",\n  \"critical-order\", ");
-	SaveOrder(&unit->CriticalOrder, file);
 	file->printf(",\n  \"new-order\", ");
 	SaveOrder(&unit->NewOrder, file);
 
@@ -3763,18 +3527,10 @@ void SaveUnit(const CUnit *unit, CFile *file)
 	//
 	switch (unit->Orders[0]->Action) {
 		case UnitActionStill:
-			// FIXME: support other resource types
-			if (unit->Type->GivesResource) {
+			if (unit->Type->CanHarvestFrom) {
 				file->printf(", \"resource-active\", %d", unit->Data.Resource.Active);
 			}
 			break;
-		case UnitActionResource:
-			file->printf(", \"data-res-worker\", {\"time-to-harvest\", %d,", unit->Data.ResWorker.TimeToHarvest);
-			if (unit->Data.ResWorker.DoneHarvesting) {
-				file->printf(" \"done-harvesting\",");
-			}
-			file->printf("}");
-			break;
 		case UnitActionBuilt:
 			{
 				CConstructionFrame *cframe;
@@ -3800,21 +3556,19 @@ void SaveUnit(const CUnit *unit, CFile *file)
 				file->printf("}");
 				break;
 			}
-		case UnitActionResearch:
-			file->printf(",\n  \"data-research\", {");
-			file->printf("\"ident\", \"%s\",", unit->Data.Research.Upgrade->Ident.c_str());
-			file->printf("}");
-			break;
-		case UnitActionUpgradeTo:
-			file->printf(",\n  \"data-upgrade-to\", {");
-			file->printf("\"ticks\", %d,", unit->Data.UpgradeTo.Ticks);
-			file->printf("}");
-			break;
 		case UnitActionTrain:
 			file->printf(",\n  \"data-train\", {");
 			file->printf("\"ticks\", %d, ", unit->Data.Train.Ticks);
 			file->printf("}");
 			break;
+		case UnitActionResource:
+			file->printf(",\n  \"data-harvest\", {");
+			file->printf("\"current-production\", {");
+			for (i = 0; i < MaxCosts; ++i) {
+				file->printf("%s%d", (i ? ", " : ""), unit->Data.Harvest.CurrentProduction[i]);
+			}
+			file->printf("}}");
+			break;
 		default:
 			file->printf(",\n  \"data-move\", {");
 			if (unit->Data.Move.Fast) {
@@ -3856,7 +3610,7 @@ void SaveUnits(CFile *file)
 	CUnit *unit;
 
 	file->printf("\n--- -----------------------------------------\n");
-	file->printf("--- MODULE: units $Id: unit.cpp 8550 2007-02-10 12:33:10Z feb $\n\n");
+	file->printf("--- MODULE: units $Id: unit.cpp 9029 2007-08-04 15:17:16Z feb $\n\n");
 
 	file->printf("-- Unit slot usage bitmap\n");
 	file->printf("SlotUsage(%d", UnitSlotFree);
@@ -3870,7 +3624,6 @@ void SaveUnits(CFile *file)
 	for (table = Units; table < &Units[NumUnits]; ++table) {
 		SaveUnit(*table, file);
 	}
-
 }
 
 /*----------------------------------------------------------------------------
@@ -3916,7 +3669,6 @@ void CleanUnits(void)
 
 	InitUnitsMemory();
 
-	XpDamage = 0;
 	HelpMeLastCycle = 0;
 }
 
