@@ -9,7 +9,7 @@
 //
 /**@name unit_draw.cpp - The draw routines for units. */
 //
-//      (c) Copyright 1998-2006 by Lutz Sammer, Jimmy Salmon, Nehal Mistry
+//      (c) Copyright 1998-2007 by Lutz Sammer, Jimmy Salmon, Nehal Mistry
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: unit_draw.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: unit_draw.cpp 8758 2007-04-15 02:12:24Z jim4 $
 
 //@{
 
@@ -104,16 +104,6 @@ unsigned long ShowOrdersCount;    /// Show orders for some time
 // FIXME: not all variables of this file are here
 // FIXME: perhaps split this file into two or three parts?
 
-/**
-**  Show that units are selected.
-**
-**  @param color    FIXME
-**  @param x1,y1    Coordinates of the top left corner.
-**  @param x2,y2    Coordinates of the bottom right corner.
-*/
-void (*DrawSelection)(Uint32 color, int x1, int y1,
-	int x2, int y2) = DrawSelectionNone;
-
 /*----------------------------------------------------------------------------
 --  Functions
 ----------------------------------------------------------------------------*/
@@ -126,7 +116,7 @@ const CViewport *CurrentViewport;  /// FIXME: quick hack for split screen
 // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
 /**
-**  Show selection marker around an unit.
+**  Show selection marker around a unit.
 **
 **  @param unit  Pointer to unit.
 */
@@ -185,76 +175,6 @@ void DrawUnitSelection(const CUnit *unit)
 	DrawSelection(color, x, y, x + type->BoxWidth, y + type->BoxHeight);
 }
 
-/**
-**  Don't show selected units.
-**
-**  @param color  Color to draw, nothing in this case.
-**  @param x1,y1  Coordinates of the top left corner.
-**  @param x2,y2  Coordinates of the bottom right corner.
-*/
-void DrawSelectionNone(Uint32 color, int x1, int y1,
-	int x2, int y2)
-{
-}
-
-/**
-**  Show selected units with circle.
-**
-**  @param color  Color to draw circle
-**  @param x1,y1  Coordinates of the top left corner.
-**  @param x2,y2  Coordinates of the bottom right corner.
-*/
-void DrawSelectionCircle(Uint32 color, int x1, int y1,
-	int x2, int y2)
-{
-	Video.DrawCircleClip(color, (x1 + x2) / 2, (y1 + y2) / 2,
-		s_min((x2 - x1) / 2, (y2 - y1) / 2) + 2);
-}
-
-/**
-**  Show selected units with circle.
-**
-**  @param color  Color to draw and fill circle.
-**  @param x1,y1  Coordinates of the top left corner.
-**  @param x2,y2  Coordinates of the bottom right corner.
-*/
-void DrawSelectionCircleWithTrans(Uint32 color, int x1, int y1,
-	int x2, int y2)
-{
-	Video.FillTransCircleClip(color, (x1 + x2) / 2, (y1 + y2) / 2,
-		s_min((x2 - x1) / 2, (y2 - y1) / 2), 95);
-	Video.DrawCircleClip(color, (x1 + x2) / 2, (y1 + y2) / 2,
-		s_min((x2 - x1) / 2, (y2 - y1) / 2));
-}
-
-/**
-**  Draw selected rectangle around the unit.
-**
-**  @param color  Color to draw rectangle.
-**  @param x1,y1  Coordinates of the top left corner.
-**  @param x2,y2  Coordinates of the bottom right corner.
-*/
-void DrawSelectionRectangle(Uint32 color, int x1, int y1,
-	int x2, int y2)
-{
-	Video.DrawRectangleClip(color, x1, y1, x2 - x1, y2 - y1);
-}
-
-/**
-**  Draw selected rectangle around the unit.
-**
-**  @param color  Color to draw and fill rectangle.
-**  @param x1,y1  Coordinates of the top left corner.
-**  @param x2,y2  Coordinates of the bottom right corner.
-*/
-void DrawSelectionRectangleWithTrans(Uint32 color, int x1, int y1,
-	int x2, int y2)
-{
-	Video.DrawRectangleClip(color, x1, y1, x2 - x1, y2 - y1);
-	Video.FillTransRectangleClip(color, x1 + 1, y1 + 1,
-		x2 - x1 - 2, y2 - y1 - 2, 75);
-}
-
 /**
 **  Draw selected corners around the unit.
 **
@@ -262,11 +182,9 @@ void DrawSelectionRectangleWithTrans(Uint32 color, int x1, int y1,
 **  @param x1,y1  Coordinates of the top left corner.
 **  @param x2,y2  Coordinates of the bottom right corner.
 */
-void DrawSelectionCorners(Uint32 color, int x1, int y1,
-	int x2, int y2)
+void DrawSelection(Uint32 color, int x1, int y1, int x2, int y2)
 {
 #define CORNER_PIXELS 6
-
 	Video.DrawVLineClip(color, x1, y1, CORNER_PIXELS);
 	Video.DrawHLineClip(color, x1 + 1, y1, CORNER_PIXELS - 1);
 
@@ -278,6 +196,7 @@ void DrawSelectionCorners(Uint32 color, int x1, int y1,
 
 	Video.DrawVLineClip(color, x2, y2 - CORNER_PIXELS + 1, CORNER_PIXELS);
 	Video.DrawHLineClip(color, x2 - CORNER_PIXELS + 1, y2, CORNER_PIXELS - 1);
+#undef CORNER_PIXELS
 }
 
 
@@ -368,319 +287,6 @@ static int CclDefineSprites(lua_State *l)
 	return 0;
 }
 
-/**
-**  Define mana sprite.
-**  Equivalent of
-**  DefineSprites({Name = "sprite-mana",
-**    File = file, Offset = {hotx, hoty}, Size = {w, h}})
-**
-**  @param l  Lua state
-*/
-static int CclManaSprite(lua_State *l)
-{
-	char buffer[1024]; // lua equivalent.
-
-	LuaCheckArgs(l, 5);
-
-	sprintf(buffer, "DefineSprites({Name = \"%s\", File = \"%s\", "
-		"Offset = {%i, %i}, Size = {%i, %i}})",
-		"sprite-mana", LuaToString(l, 1),
-		(int) LuaToNumber(l, 2), (int) LuaToNumber(l, 3),
-		(int) LuaToNumber(l, 4), (int) LuaToNumber(l, 5));
-
-	DebugPrint("instead of ManaSprite, you should write instead :\n%s\n" _C_ buffer);
-
-	CclCommand(buffer);
-	return 0;
-}
-
-/**
-**  Define health sprite.
-**
-**  @param l  Lua state
-*/
-static int CclHealthSprite(lua_State *l)
-{
-	char buffer[1024]; // lua equivalent.
-
-	LuaCheckArgs(l, 5);
-
-	sprintf(buffer, "DefineSprites({Name = \"%s\", File = \"%s\", "
-		"Offset = {%i, %i}, Size = {%i, %i}})",
-		"sprite-health", LuaToString(l, 1),
-		(int) LuaToNumber(l, 2), (int) LuaToNumber(l, 3),
-		(int )LuaToNumber(l, 4), (int) LuaToNumber(l, 5));
-
-	DebugPrint("instead of HealthSprite, you should write instead :\n%s\n" _C_ buffer);
-
-	CclCommand(buffer);
-	return 0;
-}
-
-#if 1 // To be deleted ? DefineDecoration do that.
-
-/**
-**  Enable display health as health-sprite.
-**  Equivalent of
-**  DefineDecorations({Index = "HitPoints", HideNeutral = true, CenterX = true,
-**    OffsetPercent = {50, 100}, Offset = {0, -7},
-**    Method = {"sprite", {1}}})
-**
-**  @param l  Lua state
-*/
-static int CclShowHealthDot(lua_State *l)
-{
-	const char *lua_equiv = "DefineDecorations({Index = \"HitPoints\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"sprite\", {\"sprite-health\"}}})";
-
-	LuaCheckArgs(l, 0);
-
-	DebugPrint("instead of ShowHealthDot, you should write instead :\n%s\n" _C_ lua_equiv);
-
-	CclCommand(lua_equiv);
-	return 0;
-}
-
-/**
-**  Enable display health as health-bar.
-**  Equivalent of
-**  DefineDecorations({Index = "HitPoints", HideNeutral = true, CenterX = true,
-**    OffsetPercent = {50, 100}, Offset = {0, -7},
-**    Method = {"bar", {Width = 3, BorderSize = 1}}})
-**
-**  @param l  Lua state
-*/
-static int CclShowHealthHorizontal(lua_State *l)
-{
-	const char *lua_equiv = "DefineDecorations({Index = \"HitPoints\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100}, Offset = {0, -7},"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1}}})";
-
-	LuaCheckArgs(l, 0);
-
-	DebugPrint("instead of ShowHealthHorizontal, you should write instead :\n%s\n" _C_ lua_equiv);
-
-	CclCommand(lua_equiv);
-	return 0;
-}
-
-/**
-**  Enable display health as health-bar.
-**  Equivalent of
-**  DefineDecorations({Index = "HitPoints", HideNeutral = true,
-**    Offset = {-7, 0},
-**    Method = {"bar", {Width = 3, BorderSize = 1, Orientation = "vertical"}}})
-**
-**  @param l  Lua state
-*/
-static int CclShowHealthVertical(lua_State *l)
-{
-	const char *lua_equiv = "DefineDecorations({Index = \"HitPoints\", HideNeutral = true,"
-		"Offset = {-7, 0},"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})";
-
-	LuaCheckArgs(l, 0);
-
-	DebugPrint("instead of ShowHealthVertical, you should write instead :\n%s\n" _C_ lua_equiv);
-
-	CclCommand(lua_equiv);
-	return 0;
-}
-
-/**
-**  Enable display health as health-bar.
-**
-**  Equivalent of ShowHealthVertical()
-**
-**  @param l  Lua state
-*/
-static int CclShowHealthBar(lua_State *l)
-{
-	return CclShowHealthVertical(l);
-}
-
-/**
-**  Enable display mana as mana-sprite.
-**
-**  Equivalent of
-**  DefineDecorations({Index = "Mana", HideNeutral = true, CenterX = true,
-**    OffsetPercent = {50, 100},
-**    Method = {"sprite", {0}}})
-**  For index Mana, Transport, Research, Training, UpgradeTo, Resource.
-**  Except for ressource which have HideNeutral = false.
-**
-**  @param l  Lua state
-*/
-static int CclShowManaDot(lua_State *l)
-{
-	char lua_equiv[] = "DefineDecorations({Index = \"Mana\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100},Method = {\"sprite\", {\"sprite-mana\"}}})\n"
-		"DefineDecorations({Index = \"Transport\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100},Method = {\"sprite\", {\"sprite-mana\"}}})\n"
-		"DefineDecorations({Index = \"Research\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100},Method = {\"sprite\", {\"sprite-mana\"}}})\n"
-		"DefineDecorations({Index = \"Training\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100},Method = {\"sprite\", {\"sprite-mana\"}}})\n"
-		"DefineDecorations({Index = \"UpgradeTo\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100},Method = {\"sprite\", {\"sprite-mana\"}}})\n"
-		"DefineDecorations({Index = \"GiveResource\", HideNeutral = false, CenterX = true,"
-		"OffsetPercent = {50, 100},Method = {\"sprite\", {\"sprite-mana\"}}})\n"
-		"DefineDecorations({Index = \"CarryResource\", HideNeutral = false, CenterX = true,"
-		"OffsetPercent = {50, 100},Method = {\"sprite\", {\"sprite-mana\"}}})\n";
-
-
-	LuaCheckArgs(l, 0);
-
-	DebugPrint("instead of ShowManaDot, you should write instead :\n%s\n" _C_ lua_equiv);
-
-	CclCommand(lua_equiv);
-	return 0;
-}
-
-/**
-**  Enable display mana as horizontal bar.
-**
-**  Equivalent of
-**  DefineDecorations({Index = "Mana", HideNeutral = true, CenterX = true,
-**    OffsetPercent = {50, 100},
-**    Method = {"bar", {Width = 3, BorderSize = 1}}})
-**  For index Mana, Transport, Research, Training, UpgradeTo, Resource.
-**
-**  @param l  Lua state
-*/
-static int CclShowManaHorizontal(lua_State *l)
-{
-	char lua_equiv[] = "DefineDecorations({Index = \"Mana\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"bar\", {Width = 3, BorderSize = 1}}})\n"
-		"DefineDecorations({Index = \"Transport\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"bar\", {Width = 3, BorderSize = 1}}})\n"
-		"DefineDecorations({Index = \"Research\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"bar\", {Width = 3, BorderSize = 1}}})\n"
-		"DefineDecorations({Index = \"Training\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"bar\", {Width = 3, BorderSize = 1}}})\n"
-		"DefineDecorations({Index = \"UpgradeTo\", HideNeutral = true, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"bar\", {Width = 3, BorderSize = 1}}})\n"
-		"DefineDecorations({Index = \"GiveResource\", HideNeutral = false, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"bar\", {Width = 3, BorderSize = 1}}})\n"
-		"DefineDecorations({Index = \"CarryResource\", HideNeutral = false, CenterX = true,"
-		"OffsetPercent = {50, 100}, Method = {\"bar\", {Width = 3, BorderSize = 1}}})\n";
-
-
-	LuaCheckArgs(l, 0);
-
-	DebugPrint("instead of ShowManaHorizontal, you should write instead :\n%s\n" _C_ lua_equiv);
-
-	CclCommand(lua_equiv);
-	return 0;
-}
-
-/**
-**  Enable display mana as vertical bar.
-**
-**  Equivalent of
-**  DefineDecorations({Index = "Mana", HideNeutral = true,
-**    Method = {"bar", {Width = 3, BorderSize = 1, Orientation = "vertical"}}})
-**  For index Mana, Transport, Research, Training, UpgradeTo, Resource.
-**
-**  @param l  Lua state
-*/
-static int CclShowManaVertical(lua_State *l)
-{
-	char lua_equiv[] = "DefineDecorations({Index = \"Mana\", HideNeutral = true"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})\n"
-		"DefineDecorations({Index = \"Transport\", HideNeutral = true,"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})\n"
-		"DefineDecorations({Index = \"Research\", HideNeutral = true,"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})\n"
-		"DefineDecorations({Index = \"Training\", HideNeutral = true,"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})\n"
-		"DefineDecorations({Index = \"UpgradeTo\", HideNeutral = true,"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})\n"
-		"DefineDecorations({Index = \"GiveResource\", HideNeutral = false,"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})\n"
-		"DefineDecorations({Index = \"CarryResource\", HideNeutral = false,"
-		"Method = {\"bar\", {Width = 3, BorderSize = 1, Orientation = \"vertical\"}}})\n";
-
-
-	LuaCheckArgs(l, 0);
-
-	DebugPrint("instead of ShowManaVertical, you should write instead :\n%s\n" _C_ lua_equiv);
-
-	CclCommand(lua_equiv);
-	return 0;
-}
-
-/**
-**  Enable display mana as mana-bar.
-**
-**  Equivalent of ShowManaVertical()
-**
-**  @param l  Lua state
-*/
-static int CclShowManaBar(lua_State *l)
-{
-	return CclShowManaVertical(l);
-}
-
-/**
-**  Enable energy bars and dots only for selected units
-**
-**  @param l  Lua state
-*/
-static int CclShowEnergySelected(lua_State *l)
-{
-	LuaCheckArgs(l, 0);
-	for (std::vector<CDecoVar *>::const_iterator i = UnitTypeVar.DecoVar.begin();
-		i < UnitTypeVar.DecoVar.end(); ++i) {
-		(*i)->ShowOnlySelected = true;
-	}
-	return 0;
-}
-
-/**
-**  Enable display of full bars/dots.
-**
-**  @param l  Lua state
-*/
-static int CclShowFull(lua_State *l)
-{
-	LuaCheckArgs(l, 0);
-	for (std::vector<CDecoVar *>::const_iterator i = UnitTypeVar.DecoVar.begin();
-		i < UnitTypeVar.DecoVar.end(); ++i) {
-		(*i)->ShowWhenMax = 1;
-	}
-	return 0;
-}
-
-/**
-**  Disable display of full bars/dots.
-**
-**  @param l  Lua state
-*/
-static int CclShowNoFull(lua_State *l)
-{
-	LuaCheckArgs(l, 0);
-	for (std::vector<CDecoVar *>::iterator i = UnitTypeVar.DecoVar.begin();
-		i < UnitTypeVar.DecoVar.end(); ++i) {
-		(*i)->ShowWhenMax = 0;
-	}
-	return 0;
-}
-
-/**
-**  Draw decorations always on top.
-**
-**  @param l  Lua state
-*/
-static int CclDecorationOnTop(lua_State *l)
-{
-	LuaCheckArgs(l, 0);
-	// FIXME: not implemented
-	return 0;
-}
-
-#endif
-
 /**
 **  Register CCL features for decorations.
 */
@@ -689,29 +295,6 @@ void DecorationCclRegister(void)
 	memset(&DecoSprite, 0, sizeof(DecoSprite));
 
 	lua_register(Lua, "DefineSprites", CclDefineSprites);
-#if 1 // to be removed.
-	lua_register(Lua, "ManaSprite", CclManaSprite);
-	lua_register(Lua, "HealthSprite", CclHealthSprite);
-#endif
-
-#if 1 // To be deleted ? alredy Replaced by DefineDecoration
-	// These fonctions ara aliases to DefineDecorations
-	lua_register(Lua, "ShowHealthBar", CclShowHealthBar);
-	lua_register(Lua, "ShowHealthDot", CclShowHealthDot);
-// adicionado por protoman
-	lua_register(Lua, "ShowHealthVertical", CclShowHealthVertical);
-	lua_register(Lua, "ShowHealthHorizontal", CclShowHealthHorizontal);
-	lua_register(Lua, "ShowManaVertical", CclShowManaVertical);
-	lua_register(Lua, "ShowManaHorizontal", CclShowManaHorizontal);
-// fim
-
-	lua_register(Lua, "ShowManaBar", CclShowManaBar);
-	lua_register(Lua, "ShowManaDot", CclShowManaDot);
-	lua_register(Lua, "ShowEnergySelectedOnly", CclShowEnergySelected);
-	lua_register(Lua, "ShowFull", CclShowFull);
-	lua_register(Lua, "ShowNoFull", CclShowNoFull);
-	lua_register(Lua, "DecorationOnTop", CclDecorationOnTop);
-#endif
 }
 
 /**
@@ -741,95 +324,6 @@ void CleanDecorations(void)
 	DecoSprite.SpriteArray.clear();
 }
 
-/**
-**  Draw bar for variables.
-**
-**  @param x       X screen pixel position
-**  @param y       Y screen pixel position
-**  @param unit    Unit pointer
-**  @todo fix color configuration.
-*/
-void CDecoVarBar::Draw(int x, int y, const CUnit *unit) const
-{
-	int height;
-	int width;
-	int h;
-	int w;
-	char b;        // BorderSize.
-	Uint32 bcolor; // Border color.
-	Uint32 color;  // inseide color.
-	int f;         // 100 * value / max.
-
-	Assert(unit);
-	Assert(unit->Type);
-	Assert(unit->Variable[this->Index].Max);
-	height = this->Height;
-	if (height == 0) { // Default value
-		height = unit->Type->BoxHeight; // Better size ? {,Box, Tile}
-	}
-	width = this->Width;
-	if (width == 0) { // Default value
-		width = unit->Type->BoxWidth; // Better size ? {,Box, Tile}
-	}
-	if (this->IsVertical)  { // Vertical
-		w = width;
-		h = unit->Variable[this->Index].Value * height / unit->Variable[this->Index].Max;
-	} else {
-		w = unit->Variable[this->Index].Value * width / unit->Variable[this->Index].Max;
-		h = height;
-	}
-
-	if (this->IsCenteredInX) {
-		x -= w / 2;
-	}
-	if (this->IsCenteredInY) {
-		y -= h / 2;
-	}
-
-	b = this->BorderSize;
-	// Could depend of (value / max)
-	f = unit->Variable[this->Index].Value * 100 / unit->Variable[this->Index].Max;
-	bcolor = ColorBlack; // Deco->Data.Bar.BColor
-	color = f > 50 ? (f > 75 ? ColorGreen : ColorYellow) : (f > 25 ? ColorOrange : ColorRed);
-	// Deco->Data.Bar.Color
-	if (b) {
-		if (this->ShowFullBackground) {
-			Video.FillRectangleClip(bcolor, x - b, y - b, 2 * b + width, 2 * b + height);
-		} else {
-			if (this->SEToNW) {
-				Video.FillRectangleClip(bcolor, x - b - w + width, y - b - h + height,
-					2 * b + w, 2 * b + h);
-			} else {
-				Video.FillRectangleClip(bcolor, x - b, y - b, 2 * b + w, 2 * b + h);
-			}
-		}
-	}
-	if (this->SEToNW) {
-		Video.FillRectangleClip(color, x - w + width, y - h + height, w, h);
-	} else {
-		Video.FillRectangleClip(color, x, y, w, h);
-	}
-}
-
-/**
-**  Print variable values (and max....).
-**
-**  @param x       X screen pixel position
-**  @param y       Y screen pixel position
-**  @param unit    Unit pointer
-**  @todo fix font/color configuration.
-*/
-void CDecoVarText::Draw(int x, int y, const CUnit *unit) const
-{
-	if (this->IsCenteredInX) {
-		x -= 2; // GameFont->Width(buf) / 2, with buf = str(Value)
-	}
-	if (this->IsCenteredInY) {
-		y -= this->Font->Height() / 2;
-	}
-	VideoDrawNumberClip(x, y, this->Font, unit->Variable[this->Index].Value);
-}
-
 /**
 **  Draw a sprite with is like a bar (several stages)
 **
@@ -841,7 +335,7 @@ void CDecoVarText::Draw(int x, int y, const CUnit *unit) const
 void CDecoVarSpriteBar::Draw(int x, int y, const CUnit *unit) const
 {
 	int n;                   // frame of the sprite to show.
-	CGraphic *sprite;         // the sprite to show.
+	CGraphic *sprite;        // the sprite to show.
 	Decoration *decosprite;  // Info on the sprite.
 
 	Assert(unit);
@@ -865,33 +359,6 @@ void CDecoVarSpriteBar::Draw(int x, int y, const CUnit *unit) const
 	sprite->DrawFrameClip(n, x, y);
 }
 
-/**
-**  Draw a static sprite.
-**
-**  @param x       X screen pixel position
-**  @param y       Y screen pixel position
-**  @param unit    Unit pointer
-**
-**  @todo fix sprite configuration configuration.
-*/
-void CDecoVarStaticSprite::Draw(int x, int y, const CUnit *unit) const
-{
-	CGraphic *sprite;         // the sprite to show.
-	Decoration *decosprite;  // Info on the sprite.
-
-	decosprite = &DecoSprite.SpriteArray[(int)this->NSprite];
-	sprite = decosprite->Sprite;
-	x += decosprite->HotX; // in addition of OffsetX... Usefull ?
-	y += decosprite->HotY; // in addition of OffsetY... Usefull ?
-	if (this->IsCenteredInX) {
-		x -= sprite->Width / 2;
-	}
-	if (this->IsCenteredInY) {
-		y -= sprite->Height / 2;
-	}
-	sprite->DrawFrameClip(this->n, x, y);
-}
-
 
 extern void UpdateUnitVariables(const CUnit *unit);
 
@@ -916,7 +383,7 @@ static void DrawDecoration(const CUnit *unit, const CUnitType *type, int x, int
 	UpdateUnitVariables(unit);
 	// Now show decoration for each variable.
 	for (std::vector<CDecoVar *>::const_iterator i = UnitTypeVar.DecoVar.begin();
-		i < UnitTypeVar.DecoVar.end(); ++i) {
+			i < UnitTypeVar.DecoVar.end(); ++i) {
 		int value;
 		int max;
 
@@ -924,8 +391,7 @@ static void DrawDecoration(const CUnit *unit, const CUnitType *type, int x, int
 		max = unit->Variable[(*i)->Index].Max;
 		Assert(value <= max);
 
-		if (!((value == 0 && !(*i)->ShowWhenNull) || (value == max && !(*i)->ShowWhenMax) ||
-				((*i)->HideHalf && value != 0 && value != max) ||
+		if (!((value == max && !(*i)->ShowWhenMax) ||
 				(!(*i)->ShowIfNotEnable && !unit->Variable[(*i)->Index].Enable) ||
 				((*i)->ShowOnlySelected && !unit->Selected) ||
 				(unit->Player->Type == PlayerNeutral && (*i)->HideNeutral) ||
@@ -1015,7 +481,7 @@ void DrawShadow(const CUnit *unit, const CUnitType *type, int frame,
 }
 
 /**
-**  Get the location of an unit's order.
+**  Get the location of a unit's order.
 **
 **  @param unit   Pointer to unit.
 **  @param order  Pointer to order.
@@ -1143,14 +609,6 @@ static void ShowSingleOrder(const CUnit *unit, int x1, int y1, const COrder *ord
 			e_color = color = ColorGray;
 			break;
 
-		case UnitActionUpgradeTo:
-			e_color = color = ColorGray;
-			break;
-
-		case UnitActionResearch:
-			e_color = color = ColorGray;
-			break;
-
 		case UnitActionBuild:
 			DrawSelection(ColorGray, x2 - order->Type->BoxWidth / 2,
 				y2 - order->Type->BoxHeight / 2,
@@ -1169,11 +627,6 @@ static void ShowSingleOrder(const CUnit *unit, int x1, int y1, const COrder *ord
 			dest = 1;
 			break;
 
-		case UnitActionReturnGoods:
-			e_color = color = ColorYellow;
-			dest = 1;
-			break;
-
 		default:
 			e_color = color = ColorGray;
 			DebugPrint("Unknown action %d\n" _C_ order->Action);
@@ -1187,7 +640,7 @@ static void ShowSingleOrder(const CUnit *unit, int x1, int y1, const COrder *ord
 }
 
 /**
-**  Show the current order of an unit.
+**  Show the current order of a unit.
 **
 **  @param unit  Pointer to the unit.
 */
@@ -1228,7 +681,7 @@ void ShowOrder(const CUnit *unit)
 }
 
 /**
-**  Draw additional informations of an unit.
+**  Draw additional informations of a unit.
 **
 **  @param unit  Unit pointer of drawn unit.
 **  @param type  Unit-type pointer.
@@ -1443,8 +896,6 @@ void CUnit::Draw() const
 	int frame;
 	int state;
 	int constructed;
-	CPlayerColorGraphic *sprite;
-	ResourceInfo *resinfo;
 	CConstructionFrame *cframe;
 	CUnitType *type;
 
@@ -1462,8 +913,7 @@ void CUnit::Draw() const
 		x = this->IX;
 		x += CurrentViewport->Map2ViewportX(this->X);
 		y += CurrentViewport->Map2ViewportY(this->Y);
-		state = (this->Orders[0]->Action == UnitActionBuilt) |
-			((this->Orders[0]->Action == UnitActionUpgradeTo) << 1);
+		state = (this->Orders[0]->Action == UnitActionBuilt);
 		constructed = this->Constructed;
 		// Reset Type to the type being upgraded to
 		if (state == 2) {
@@ -1507,23 +957,6 @@ void CUnit::Draw() const
 	//
 	DrawUnitSelection(this);
 
-	//
-	// Adjust sprite for Harvesters.
-	//
-	sprite = type->Sprite;
-	if (type->Harvester && this->CurrentResource) {
-		resinfo = type->ResInfo[this->CurrentResource];
-		if (this->ResourcesHeld) {
-			if (resinfo->SpriteWhenLoaded) {
-				sprite = resinfo->SpriteWhenLoaded;
-			}
-		} else {
-			if (resinfo->SpriteWhenEmpty) {
-				sprite = resinfo->SpriteWhenEmpty;
-			}
-		}
-	}
-
 	//
 	// Now draw!
 	// Buildings under construction/upgrade/ready.
@@ -1539,11 +972,11 @@ void CUnit::Draw() const
 	//
 	} else if (state == 2) {
 		// FIXME: this frame is hardcoded!!!
-		DrawUnitType(type, sprite,
+		DrawUnitType(type, type->Sprite,
 			this->RescuedFrom ? this->RescuedFrom->Index : this->Player->Index,
 			frame < 0 ? -1 - 1 : 1, x, y);
 	} else {
-		DrawUnitType(type, sprite,
+		DrawUnitType(type, type->Sprite,
 			this->RescuedFrom ? this->RescuedFrom->Index : this->Player->Index,
 			frame, x, y);
 	}
@@ -1562,10 +995,8 @@ void CUnit::Draw() const
 */
 static int DrawLevelCompare(const void *v1, const void *v2) {
 
-	const CUnit *c1;
-	const CUnit *c2;
-	int drawlevel1;
-	int drawlevel2;
+	const CUnit *c1, *c2;
+	int drawlevel1, drawlevel2;
 	int diffpos;
 
 	c1 = *(CUnit **)v1;
@@ -1581,6 +1012,7 @@ static int DrawLevelCompare(const void *v1, const void *v2) {
 	} else {
 		drawlevel2 = c2->Type->DrawLevel;
 	}
+
 	if (drawlevel1 == drawlevel2) {
 		// diffpos compares unit's Y positions (bottom of sprite) on the map
 		// and uses X position in case Y positions are equal.
@@ -1601,16 +1033,13 @@ static int DrawLevelCompare(const void *v1, const void *v2) {
 */
 int FindAndSortUnits(const CViewport *vp, CUnit **table)
 {
-	int i;
-	int n;
-
 	//
 	//  Select all units touching the viewpoint.
 	//
-	n = UnitCacheSelect(vp->MapX - 1, vp->MapY - 1, vp->MapX + vp->MapWidth + 1,
+	int n = UnitCacheSelect(vp->MapX - 1, vp->MapY - 1, vp->MapX + vp->MapWidth + 1,
 		vp->MapY + vp->MapHeight + 1, table);
 
-	for (i = 0; i < n; i++) {
+	for (int i = 0; i < n; i++) {
 		if (!table[i]->IsVisibleInViewport(vp)) {
 			table[i--] = table[--n];
 		}
