@@ -9,7 +9,7 @@
 //
 /**@name unit_find.cpp - The find/select for units. */
 //
-//      (c) Copyright 1998-2005 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: unit_find.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: unit_find.cpp 8704 2007-04-08 07:08:02Z jim4 $
 
 //@{
 
@@ -81,7 +81,7 @@
 ** @param y       Map Y tile position.
 ** @param type    UnitType::UnitType, naval,fly,land.
 **
-** @return        Unit, if an unit of correct type is on the field.
+** @return        Unit, if a unit of correct type is on the field.
 */
 CUnit *UnitCacheOnXY(int x, int y, unsigned type)
 {
@@ -238,9 +238,9 @@ CUnit *TargetOnMap(const CUnit *source, int x1, int y1, int x2, int y2)
 **
 **  @param tx        X position on map, tile-based.
 **  @param ty        Y position on map, tile-based.
-**  @param resource  resource type.
+**  @param resource  Resource number or -1 for any.
 **
-**  @return          Returns the deposit if found, or NoUnitP.
+**  @return    Returns the resource if found, or NoUnitP.
 */
 CUnit *ResourceOnMap(int tx, int ty, int resource)
 {
@@ -250,40 +250,19 @@ CUnit *ResourceOnMap(int tx, int ty, int resource)
 
 	n = UnitCacheOnTile(tx, ty, table);
 	for (i = 0; i < n; ++i) {
-		if (table[i]->IsUnusable() || !table[i]->Type->CanHarvest ||
-				table[i]->ResourcesHeld == 0) {
+		if (table[i]->IsUnusable() || !table[i]->Type->CanHarvestFrom) {
 			continue;
 		}
-		if (table[i]->Type->GivesResource == resource) {
-			return table[i];
-		}
-	}
-	return NoUnitP;
-}
-
-/**
-**  Resource deposit on map tile
-**
-**  @param tx        X position on map, tile-based.
-**  @param ty        Y position on map, tile-based.
-**  @param resource  resource type.
-**
-**  @return          Returns the deposit if found, or NoUnitP.
-*/
-CUnit *ResourceDepositOnMap(int tx, int ty, int resource)
-{
-	CUnit *table[UnitMax];
-	int i;
-	int n;
-
-	n = UnitCacheOnTile(tx, ty, table);
-	for (i = 0; i < n; ++i) {
-		if (table[i]->IsUnusable()) {
-			continue;
-		}
-		if (table[i]->Type->CanStore[resource]) {
-			return table[i];
+		if (resource == -1) {
+			if (!UnitHoldsResources(table[i])) {
+				continue;
+			}
+		} else {
+			if (table[i]->ResourcesHeld[resource] == 0) {
+				continue;
+			}
 		}
+		return table[i];
 	}
 	return NoUnitP;
 }
