@@ -1,9 +1,9 @@
-//     ____                _       __               
+//     ____                _       __
 //    / __ )____  _____   | |     / /___ ___________
 //   / __  / __ \/ ___/   | | /| / / __ `/ ___/ ___/
-//  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  ) 
-// /_____/\____/____/     |__/|__/\__,_/_/  /____/  
-//                                              
+//  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  )
+// /_____/\____/____/     |__/|__/\__,_/_/  /____/
+//
 //       A futuristic real-time strategy game.
 //          This file is part of Bos Wars.
 //
@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: graphic.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: graphic.cpp 8949 2007-06-29 11:25:27Z Jarod42 $
 
 //@{
 
@@ -79,7 +79,7 @@ void CGraphic::DrawSub(int gx, int gy, int w, int h, int x, int y) const
 {
 #ifndef USE_OPENGL
 	SDL_Rect srect = {gx, gy, w, h};
-	SDL_Rect drect = {x, y};
+	SDL_Rect drect = {x, y, 0, 0};
 
 	SDL_BlitSurface(Surface, &srect, TheScreen, &drect);
 #else
@@ -666,6 +666,7 @@ void CGraphic::Free(CGraphic *g)
 			for (i = 0; i < PlayerMax; ++i) {
 				if (cg->PlayerColorTextures[i]) {
 					glDeleteTextures(cg->NumTextures, cg->PlayerColorTextures[i]);
+					delete[] cg->PlayerColorTextures[i];
 				}
 			}
 		}
@@ -908,7 +909,14 @@ static void MakeTextures2(CGraphic *g, GLuint texture, CUnitColors *colors,
 			tp += 4;
 		}
 	}
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tex);
+
+	GLint internalformat = GL_RGBA;
+	if (GLTextureCompressionSupported && UseGLTextureCompression) {
+		internalformat = GL_COMPRESSED_RGBA;
+	}
+
+	glTexImage2D(GL_TEXTURE_2D, 0, internalformat, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tex);
+
 #ifdef DEBUG
 	int x;
 	if ((x = glGetError())) {
@@ -1238,4 +1246,15 @@ void CGraphic::MakeShadow()
 #endif
 }
 
+#ifdef DEBUG
+void FreeGraphics()
+{
+	std::map<std::string, CGraphic *>::iterator i;
+	while (!GraphicHash.empty()) {
+		i = GraphicHash.begin();
+		CGraphic::Free((*i).second);
+	}
+}
+#endif
+
 //@}
