@@ -26,7 +26,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: linedraw.cpp 8550 2007-02-10 12:33:10Z feb $
+//      $Id: linedraw.cpp 8973 2007-07-01 03:06:29Z jim4 $
 
 //@{
 
@@ -1390,14 +1390,14 @@ void CVideo::DrawRectangle(Uint32 color, int x, int y, int w, int h)
 	glVertex2i(x, y);
 	glVertex2i(x + w, y);
 
-	glVertex2i(x + w - 1, y);
+	glVertex2i(x + w - 1, y + 1);
 	glVertex2i(x + w - 1, y + h);
 
-	glVertex2i(x + w, y + h - 1);
+	glVertex2i(x + w - 1, y + h - 1);
 	glVertex2i(x, y + h - 1);
 
-	glVertex2i(x, y + h);
-	glVertex2i(x, y);
+	glVertex2i(x, y + h - 1);
+	glVertex2i(x, y + 1);
 	glEnd();
 	glEnable(GL_TEXTURE_2D);
 }
