@@ -25,7 +25,7 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: sdl.cpp 8558 2007-02-10 17:17:28Z jim4 $
+//      $Id: sdl.cpp 8913 2007-06-23 19:08:26Z jim4 $
 
 //@{
 
@@ -51,14 +51,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 #endif
-#include "SDL.h"
-#ifdef USE_OPENGL
-#include "SDL_opengl.h"
-#endif
-
-#ifdef USE_BEOS
-#include <sys/socket.h>
-#endif
 
 #ifdef USE_WIN32
 #include "net_lowlevel.h"
@@ -89,14 +81,16 @@ SDL_Surface *TheScreen; /// Internal screen
 static SDL_Rect Rects[100];
 static int NumRects;
 #else
-GLint GLMaxTextureSize;   /// Max texture size supported on the video card
+GLint GLMaxTextureSize;             /// Max texture size supported on the video card
+bool GLTextureCompressionSupported; /// Is OpenGL texture compression supported
 #endif
+bool UseGLTextureCompression;       /// Use OpenGL texture compression
 
 static std::map<int, std::string> Key2Str;
 
-static int FrameTicks; /// Frame length in ms
+static int FrameTicks;     /// Frame length in ms
 static int FrameRemainder; /// Frame remainder 0.1 ms
-static int FrameFraction; /// Frame fractional term
+static int FrameFraction;  /// Frame fractional term
 
 const EventCallback *Callbacks;
 
@@ -104,6 +98,17 @@ const EventCallback *Callbacks;
 --  Functions
 ----------------------------------------------------------------------------*/
 
+#ifdef USE_OPENGL
+// ARB_texture_compression
+PFNGLCOMPRESSEDTEXIMAGE3DARBPROC    glCompressedTexImage3DARB;
+PFNGLCOMPRESSEDTEXIMAGE2DARBPROC    glCompressedTexImage2DARB;
+PFNGLCOMPRESSEDTEXIMAGE1DARBPROC    glCompressedTexImage1DARB;
+PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC glCompressedTexSubImage3DARB;
+PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC glCompressedTexSubImage2DARB;
+PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC glCompressedTexSubImage1DARB;
+PFNGLGETCOMPRESSEDTEXIMAGEARBPROC   glGetCompressedTexImageARB;
+#endif
+
 /*----------------------------------------------------------------------------
 --  Sync
 ----------------------------------------------------------------------------*/
@@ -140,10 +145,85 @@ void SetVideoSync(void)
 
 #ifdef USE_OPENGL
 /**
-**  Initialize open gl for doing 2d with 3d.
+**  Check if an extension is supported
+*/
+static bool IsExtensionSupported(const char *extension)
+{
+	const GLubyte *extensions = NULL;
+	const GLubyte *start;
+	GLubyte *ptr, *terminator;
+	int len;
+
+	// Extension names should not have spaces.
+	ptr = (GLubyte *)strchr(extension, ' ');
+	if (ptr || *extension == '\0') {
+		return false;
+	}
+
+	extensions = glGetString(GL_EXTENSIONS);
+	len = strlen(extension);
+	start = extensions;
+	while (true) {
+		ptr = (GLubyte *)strstr((const char *)start, extension);
+		if (!ptr) {
+			break;
+		}
+
+		terminator = ptr + len;
+		if (ptr == start || *(ptr - 1) == ' ') {
+			if (*terminator == ' ' || *terminator == '\0') {
+				return true;
+			}
+		}
+		start = terminator;
+	}
+	return false;
+}
+
+/**
+**  Initialize OpenGL extensions
+*/
+static void InitOpenGLExtensions()
+{
+	// ARB_texture_compression
+	if (IsExtensionSupported("GL_ARB_texture_compression"))
+	{
+		glCompressedTexImage3DARB =
+			(PFNGLCOMPRESSEDTEXIMAGE3DARBPROC)SDL_GL_GetProcAddress("glCompressedTexImage3DARB");
+		glCompressedTexImage2DARB =
+			(PFNGLCOMPRESSEDTEXIMAGE2DARBPROC)SDL_GL_GetProcAddress("glCompressedTexImage2DARB");
+		glCompressedTexImage1DARB =
+			(PFNGLCOMPRESSEDTEXIMAGE1DARBPROC)SDL_GL_GetProcAddress("glCompressedTexImage1DARB");
+		glCompressedTexSubImage3DARB =
+			(PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC)SDL_GL_GetProcAddress("glCompressedTexSubImage3DARB");
+		glCompressedTexSubImage2DARB =
+			(PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC)SDL_GL_GetProcAddress("glCompressedTexSubImage2DARB");
+		glCompressedTexSubImage1DARB =
+			(PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC)SDL_GL_GetProcAddress("glCompressedTexSubImage1DARB");
+		glGetCompressedTexImageARB =
+			(PFNGLGETCOMPRESSEDTEXIMAGEARBPROC)SDL_GL_GetProcAddress("glGetCompressedTexImageARB");
+
+		if (glCompressedTexImage3DARB && glCompressedTexImage2DARB &&
+			glCompressedTexImage1DARB && glCompressedTexSubImage3DARB &&
+			glCompressedTexSubImage2DARB && glCompressedTexSubImage1DARB &&
+			glGetCompressedTexImageARB)
+		{
+			GLTextureCompressionSupported = true;
+		}
+		else
+		{
+			GLTextureCompressionSupported = false;
+		}
+	}
+}
+
+/**
+**  Initialize OpenGL
 */
 static void InitOpenGL(void)
 {
+	InitOpenGLExtensions();
+
 	glViewport(0, 0, (GLsizei)Video.Width, (GLsizei)Video.Height);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
@@ -160,9 +240,19 @@ static void InitOpenGL(void)
 	glEnable(GL_TEXTURE_2D);
 	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+	glEnable(GL_LINE_SMOOTH);
+	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
 
 	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &GLMaxTextureSize);
 }
+
+void ReloadOpenGL()
+{
+	InitOpenGL();
+	ReloadGraphics();
+	ReloadFonts();
+	UI.Minimap.Reload();
+}
 #endif
 
 #if defined(DEBUG) && !defined(USE_WIN32)
@@ -470,6 +560,9 @@ static void SdlDoEvent(const EventCallback *callbacks, const SDL_Event *event)
 						DoTogglePause = false;
 						UiTogglePause();
 					}
+#ifdef USE_OPENGL
+					Video.ResizeScreen(Video.Width, Video.Height);
+#endif
 				}
 			}
 			break;
@@ -802,10 +895,7 @@ void ToggleFullScreen(void)
 	SDL_ShowCursor(SDL_DISABLE);
 
 #ifdef USE_OPENGL
-	InitOpenGL();
-	ReloadGraphics();
-	ReloadFonts();
-	UI.Minimap.Reload();
+	ReloadOpenGL();
 #else
 	SDL_LockSurface(TheScreen);
 	memcpy(TheScreen->pixels, pixels, framesize);
